@@ -28,7 +28,21 @@
 #define __STDC_LIMIT_MACROS
 
 #include "mmtf_parser.h"
-#include "mmtf_parser_private.h"
+
+//*** Standard libs
+#include <stdbool.h>
+
+//*** MsgPack lib
+#ifdef MMTF_MSGPACK_USE_CPP11
+#include <msgpack.hpp>
+#define msgpack_object msgpack::object
+#define msgpack_object_kv msgpack::object_kv
+#define msgpack_object_str msgpack::object_str
+#define MMTF_MSGPACK_TYPE(T) msgpack::type::T
+#else
+#include <msgpack.h>
+#define MMTF_MSGPACK_TYPE(T) MSGPACK_OBJECT_##T
+#endif
 
 #if MSGPACK_VERSION_MAJOR < 1
 #error "msgpack-c >= 1.0 required (https://github.com/msgpack/msgpack-c)"
@@ -251,7 +265,7 @@ enum {
     }
 
 #define CODEGEN_MMTF_parser_fetch_List(type_, suffix) \
-    type_* MMTF_parser_fetch_##suffix##List( \
+    static type_* MMTF_parser_fetch_##suffix##List( \
             const msgpack_object* object, size_t* length) { \
         CODEGEN_BODY_fetch_OBJECT_ARRAY(type_, { \
             MMTF_parser_put_##suffix(iter, result + i); \
@@ -365,6 +379,7 @@ void array_copy_bigendian_2(void* dst, const char* src, size_t n) {
     }
 }
 
+static
 float* MMTF_parser_float_from_bytes(const char* input, uint32_t input_length, uint32_t* output_length) {
     IF_NOT_MULTIPLE_ERROR_RETURN(input_length, 4, NULL);
 
@@ -378,6 +393,7 @@ float* MMTF_parser_float_from_bytes(const char* input, uint32_t input_length, ui
     return output;
 }
 
+static
 int8_t* MMTF_parser_int8_from_bytes(const char* input, uint32_t input_length, uint32_t* output_length) {
     (*output_length) = input_length;
 
@@ -389,6 +405,7 @@ int8_t* MMTF_parser_int8_from_bytes(const char* input, uint32_t input_length, ui
     return output;
 }
 
+static
 int16_t* MMTF_parser_int16_from_bytes(const char* input, uint32_t input_length, uint32_t* output_length) {
     IF_NOT_MULTIPLE_ERROR_RETURN(input_length, 2, NULL);
 
@@ -402,6 +419,7 @@ int16_t* MMTF_parser_int16_from_bytes(const char* input, uint32_t input_length,
     return output;
 }
 
+static
 int32_t* MMTF_parser_int32_from_bytes(const char* input, const uint32_t input_length, uint32_t* output_length) {
     IF_NOT_MULTIPLE_ERROR_RETURN(input_length, 4, NULL);
 
@@ -415,6 +433,7 @@ int32_t* MMTF_parser_int32_from_bytes(const char* input, const uint32_t input_le
     return output;
 }
 
+static
 char** MMTF_parser_strings_from_bytes(const char* input, uint32_t input_length, uint32_t parameter, uint32_t* output_length) {
     IF_NOT_MULTIPLE_ERROR_RETURN(input_length, parameter, NULL);
 
@@ -436,6 +455,7 @@ char** MMTF_parser_strings_from_bytes(const char* input, uint32_t input_length,
 
 //*** Array decoders
 // Run-length decode
+static
 int32_t* MMTF_parser_run_length_decode(const int32_t* input, uint32_t input_length, uint32_t* output_length) {
     (*output_length) = 0;
 
@@ -468,6 +488,7 @@ int32_t* MMTF_parser_run_length_decode(const int32_t* input, uint32_t input_leng
 }
 
 // Delta decode
+static
 int32_t* MMTF_parser_delta_decode(const int32_t* input, uint32_t input_length, uint32_t* output_length) {
     (*output_length) = input_length;
     int32_t* output = MALLOC_ARRAY(int32_t, (*output_length)); // The output needs to be freed by the calling process
@@ -486,6 +507,7 @@ int32_t* MMTF_parser_delta_decode(const int32_t* input, uint32_t input_length, u
 }
 
 // Recursive indexing decode
+static
 int32_t* MMTF_parser_recursive_indexing_decode_from_16(const int16_t* input, uint32_t input_length, uint32_t* output_length) {
     (*output_length) = 0;
     uint32_t i;
@@ -517,6 +539,7 @@ int32_t* MMTF_parser_recursive_indexing_decode_from_16(const int16_t* input, uin
     return output;
 }
 
+static
 int32_t* MMTF_parser_recursive_indexing_decode_from_8(const int8_t* input, uint32_t input_length, uint32_t* output_length) {
     (*output_length) = 0;
     uint32_t i;
@@ -545,6 +568,7 @@ int32_t* MMTF_parser_recursive_indexing_decode_from_8(const int8_t* input, uint3
 }
 
 // Integer decoding
+static
 float* MMTF_parser_integer_decode_from_16(const int16_t* input, uint32_t input_length, int32_t parameter, uint32_t* output_length) {
     (*output_length) = input_length;
     float* output = (float*)MALLOC_ARRAY(float, (*output_length));
@@ -559,6 +583,7 @@ float* MMTF_parser_integer_decode_from_16(const int16_t* input, uint32_t input_l
     return output;
 }
 
+static
 float* MMTF_parser_integer_decode_from_32(const int32_t* input, uint32_t input_length, int32_t parameter, uint32_t* output_length) {
     (*output_length) = input_length;
     float* output = (float*)MALLOC_ARRAY(float, (*output_length));
@@ -744,6 +769,7 @@ void MMTF_parser_put_string(const msgpack_object* object, char** out) {
 }
 
 //*** Unpacking from MsgPack and applying strategy
+static
 char* MMTF_parser_fetch_string(const msgpack_object* object) {
     switch (object->type) {
 #ifdef MMTF_ACCEPT_MSGPACK_BIN_AS_STR
@@ -781,6 +807,7 @@ char MMTF_parser_fetch_char(const msgpack_object* object) {
     return *(object->via.str.ptr);
 }
 
+static
 int64_t MMTF_parser_fetch_int(const msgpack_object* object) {
     int64_t result;
 
@@ -796,6 +823,7 @@ int64_t MMTF_parser_fetch_int(const msgpack_object* object) {
     return result;
 }
 
+static
 float MMTF_parser_fetch_float(const msgpack_object* object) {
     switch (object->type) {
     case /* FLOAT64 */ MMTF_MSGPACK_TYPE(FLOAT):
@@ -867,6 +895,7 @@ bool MMTF_parser_compare_msgpack_string_char_array(const msgpack_object_str* m_s
 
 static
 void MMTF_parser_put_entity(const msgpack_object* object, MMTF_Entity* entity) {
+    MMTF_Entity_init(entity);
     MAP_ITERATE_BEGIN(object);
     FETCH_AND_ASSIGN(entity, string, description);
     FETCH_AND_ASSIGN(entity, string, type);
@@ -877,6 +906,7 @@ void MMTF_parser_put_entity(const msgpack_object* object, MMTF_Entity* entity) {
 
 static
 void MMTF_parser_put_group(const msgpack_object* object, MMTF_GroupType* group_type) {
+    MMTF_GroupType_init(group_type);
     MAP_ITERATE_BEGIN(object);
     FETCH_AND_ASSIGN_DUMMYCOUNT(group_type, int32_array, formalChargeList);
     FETCH_AND_ASSIGN_WITHCOUNT(group_type, string_array, atomNameList);
@@ -889,8 +919,12 @@ void MMTF_parser_put_group(const msgpack_object* object, MMTF_GroupType* group_t
     MAP_ITERATE_END();
 }
 
+static
+MMTF_Transform* MMTF_parser_fetch_transformList(const msgpack_object*, size_t*);
+
 static
 void MMTF_parser_put_bioAssembly(const msgpack_object* object, MMTF_BioAssembly* bio_assembly) {
+    MMTF_BioAssembly_init(bio_assembly);
     MAP_ITERATE_BEGIN(object);
     FETCH_AND_ASSIGN(bio_assembly, string, name);
     FETCH_AND_ASSIGN_WITHCOUNT(bio_assembly, transformList, transformList);
