@@ -27,7 +27,7 @@ Z* -------------------------------------------------------------------
 #include"Vector.h"
 #include "Setting.h"
 
-/*
+/**
  * Get a field as NumPy array. If copy is false, then return an array wrapper
  * around the internal data of field. USE WITH CAUTION, the data pointer will
  * be invalid if the field is freed (e.g. its map object is deleted). If copy
@@ -141,7 +141,7 @@ CField *FieldNewFromPyList(PyMOLGlobals * G, PyObject * list)
   if(ok)
     ok = PyList_Check(list);
   if(ok)
-    ok = PConvPyIntToInt(PyList_GetItem(list, 0), &I->type);
+    ok = PConvFromPyListItem(G, list, 0, I->type);
   if(ok)
     ok = CPythonVal_PConvPyIntToInt_From_List(G, list, 1, &n_dim);
   if(ok)
@@ -404,7 +404,7 @@ void FieldZero(CField * I)
 }
 
 CField::CField(
-    PyMOLGlobals* G, const int* const dim, int n_dim, unsigned int base_size, int type)
+    PyMOLGlobals* G, const int* const dim, int n_dim, unsigned int base_size, cField_t type)
     : type(type)
     , base_size(base_size)
 {
