@@ -27,6 +27,8 @@ Z* ------------------------------------------------------------------- */
 #include"MemoryCache.h"
 #include"Base.h"
 
+#include "pymol/algorithm.h"
+
 static MapType *_MapNew(PyMOLGlobals * G, float range, const float *vert, int nVert,
                         const float *extent, const int *flag, int group_id, int block_id);
 
@@ -113,68 +115,6 @@ void MapCacheFree(MapCache * M, int group_id, int block_base)
 
 #define MapSafety 0.01F
 
-int MapInside(MapType * I, const float *v, int *a, int *b, int *c)
-{                               /* special version for ray-tracing */
-  int atmp, btmp, ctmp;
-  float iDiv = I->recipDiv;
-
-  atmp = (int) ((v[0] - I->Min[0]) * iDiv) + MapBorder;
-  btmp = (int) ((v[1] - I->Min[1]) * iDiv) + MapBorder;
-  ctmp = (int) ((v[2] - I->Min[2]) * iDiv) + MapBorder;
-
-  if(atmp < I->iMin[0]) {
-    if((I->iMin[0] - atmp) > 3)
-      return (-1);
-    else
-      atmp = I->iMin[0];
-  } else if(atmp > I->iMax[0]) {
-    if((atmp - I->iMax[0]) > 3)
-      return (-1);
-    else
-      atmp = I->iMax[0];
-  }
-
-  if(btmp < I->iMin[1]) {
-    if((I->iMin[1] - btmp) > 3)
-      return (-1);
-    else
-      btmp = I->iMin[1];
-  } else if(btmp > I->iMax[1]) {
-    if((btmp - I->iMax[1]) > 3)
-      return (-1);
-    else
-      btmp = I->iMax[1];
-  }
-
-  /*    printf("%d %d %d\n",ctmp,I->iMin[2],I->iMax[2]); */
-  if(ctmp < I->iMin[2]) {
-    if((I->iMin[2] - ctmp) > 3)
-      return (-1);
-    else
-      ctmp = I->iMin[2];
-  } else if(ctmp > I->iMax[2]) {
-    if((ctmp - I->iMax[2]) > 3)
-      return (0);               /* just keep searching... */
-    else
-      ctmp = I->iMax[2];
-  }
-
-  /*   printf("%d %d %d %d %d %d %d %d %d\n",
-     atmp,I->iMin[0],I->iMax[0],
-     btmp,I->iMin[1],I->iMax[1],
-     ctmp,I->iMin[2],I->iMax[2]);
-   */
-
-  if(!*(MapEStart(I, atmp, btmp, ctmp)))
-    return (0);
-
-  *a = atmp;
-  *b = btmp;
-  *c = ctmp;
-
-  return (1);
-}
-
 int MapInsideXY(MapType * I, const float *v, int *a, int *b, int *c)
 {                               /* special version for ray-tracing */
   int atmp, btmp, ctmp;
@@ -659,7 +599,13 @@ int MapSetupExpress(MapType * I)
   return ok;
 }
 
-void MapLocus(MapType * I, const float *v, int *a, int *b, int *c)
+/**
+ * Get the grid indices for a 3D query point. If the point is outside the grid,
+ * get the indices of the closest grid cell (clamp `v` to grid boundary).
+ * @param v 3D query point
+ * @param[out] a,b,c
+ */
+void MapLocus(const MapType * I, const float *v, int *a, int *b, int *c)
 {
   int at, bt, ct;
   float invDiv = I->recipDiv;
@@ -668,50 +614,32 @@ void MapLocus(MapType * I, const float *v, int *a, int *b, int *c)
   bt = (int) ((v[1] - I->Min[1]) * invDiv) + MapBorder;
   ct = (int) ((v[2] - I->Min[2]) * invDiv) + MapBorder;
 
-  /* range checking... */
-  if(at < I->iMin[0])
-    at = I->iMin[0];
-  else if(at > I->iMax[0])
-    at = I->iMax[0];
-
-  if(bt < I->iMin[1])
-    bt = I->iMin[1];
-  else if(bt > I->iMax[1])
-    bt = I->iMax[1];
-
-  if(ct < I->iMin[2])
-    ct = I->iMin[2];
-  else if(ct > I->iMax[2])
-    ct = I->iMax[2];
-
-  *a = at;
-  *b = bt;
-  *c = ct;
+  *a = pymol::clamp(at, I->iMin[0], I->iMax[0]);
+  *b = pymol::clamp(bt, I->iMin[1], I->iMax[1]);
+  *c = pymol::clamp(ct, I->iMin[2], I->iMax[2]);
 }
 
+/**
+ * Get EList start index for points in proximity to `v`.
+ * Clamps `v` to grid boundaries.
+ * @param v 3D query point
+ * @return pointer to EList start index
+ */
 int *MapLocusEStart(MapType * I, const float *v)
 {
   int a, b, c;
-  float invDiv = I->recipDiv;
-  a = (int) (((v[0] - I->Min[0]) * invDiv) + MapBorder);
-  b = (int) (((v[1] - I->Min[1]) * invDiv) + MapBorder);
-  c = (int) (((v[2] - I->Min[2]) * invDiv) + MapBorder);
-  if(a < I->iMin[0])
-    a = I->iMin[0];
-  else if(a > I->iMax[0])
-    a = I->iMax[0];
-  if(b < I->iMin[1])
-    b = I->iMin[1];
-  else if(b > I->iMax[1])
-    b = I->iMax[1];
-  if(c < I->iMin[2])
-    c = I->iMin[2];
-  else if(c > I->iMax[2])
-    c = I->iMax[2];
-  return (I->EHead + ((a) * I->D1D2) + ((b) * I->Dim[2]) + (c));
+  MapLocus(I, v, &a, &b, &c);
+  return MapEStart(I, a, b, c);
 }
 
-int MapExclLocus(MapType * I, const float *v, int *a, int *b, int *c)
+/**
+ * Get the grid indices for a 3D query point. If the point is outside the grid,
+ * return false and leave (a,b,c) in an unspecified state.
+ * @param v 3D query point
+ * @param[out] a,b,c Grid indices, but only if function returned true.
+ * @return True if `v` is within grid boundaries
+ */
+static bool MapExclLocus(MapType* I, const float* v, int* a, int* b, int* c)
 {
   float invDiv = I->recipDiv;
 
@@ -736,8 +664,9 @@ int MapExclLocus(MapType * I, const float *v, int *a, int *b, int *c)
 /**
  * Return EList start index for points in proximity to `v`.
  * Return 0 if `v` is outside the grid or there are no points.
+ * @param v 3D query point
  */
-int MapExclLocusEStart(MapType* map, const float* v)
+static int MapExclLocusEStart(MapType* map, const float* v)
 {
   int h, k, l;
   if (!MapExclLocus(map, v, &h, &k, &l))
@@ -1050,3 +979,37 @@ static MapType *_MapNew(PyMOLGlobals * G, float range, const float *vert, int nV
 
   return (I);
 }
+
+MapEIter::MapEIter(MapType& map, const float* v, bool excl)
+{
+  if (!map.EList) {
+    MapSetupExpress(&map);
+  }
+
+  m_elist = map.EList;
+
+  if (excl) {
+    m_i = MapExclLocusEStart(&map, v);
+  } else {
+    m_i = *MapLocusEStart(&map, v);
+  }
+}
+
+/**
+ * True if `v_query` is within `cutoff` of any point in the map.
+ *
+ * @param map A hash map
+ * @param v_map The points used to build the map
+ * @param v_query A query point
+ * @param cutoff The distance cutoff
+ */
+bool MapAnyWithin(
+    MapType& map, const float* v_map, const float* v_query, float cutoff)
+{
+  for (const auto j : MapEIter(map, v_query)) {
+    if (within3f(v_map + 3 * j, v_query, cutoff)) {
+      return true;
+    }
+  }
+  return false;
+}
