@@ -38,6 +38,15 @@ inline float DIP2PIXEL(float v) { return v * _gScaleFactor; }
 typedef int lexidx_t;
 typedef int lexborrow_t;
 
+using SelectorID_t = int;
+using SelectorMemberOffset_t = int;
+
+using StateIndex_t = int; ///< 0-based state index (C/C++/JavaScript)
+using StateIndexPython_t = int; ///< 1-based state index (Python/Settings)
+
+constexpr StateIndex_t cStateAll = -1;
+constexpr StateIndex_t cStateCurrent = -2;
+
 typedef struct _CMemoryCache CMemoryCache;
 struct CIsosurf;
 typedef struct _CTetsurf CTetsurf;
@@ -55,8 +64,8 @@ typedef struct _CCharacter CCharacter;
 struct CPop;
 class CScene;
 struct CSeq;
-typedef struct _CSetting CSetting;
-typedef struct _CSettingUnique CSettingUnique;
+struct CSetting;
+struct CSettingUnique;
 struct CText;
 struct CWizard;
 typedef struct _CAtomInfo CAtomInfo;
@@ -64,7 +73,7 @@ typedef struct _CSculptCache CSculptCache;
 typedef struct _CVFont CVFont;
 typedef struct _CEditor CEditor;
 struct CExecutive;
-typedef struct _CSeeker CSeeker;
+struct CSeeker;
 struct CSelector;
 struct CSelectorManager;
 typedef struct _CTexture CTexture;
@@ -101,9 +110,6 @@ typedef struct _OVOneToOne OVOneToOne;
 #define OVONETOONE_DEFINED
 #endif
 
-
-struct CObject;
-
 #ifndef CPyMOL_DEFINED
 typedef struct _CPyMOL CPyMOL;
 #define CPyMOL_DEFINED
@@ -117,8 +123,7 @@ class CGO;
 #define cPyMOLGlobals_LaunchStatus_StereoFailed 0x1
 #define cPyMOLGlobals_LaunchStatus_MultisampleFailed 0x2
 
-typedef struct _PyMOLGlobals PyMOLGlobals;
-struct _PyMOLGlobals {
+struct PyMOLGlobals {
 
   /* singleton objects */
 
