@@ -72,8 +72,6 @@ Z* -------------------------------------------------------------------
 
  */
 
-using namespace std;
-
 #ifndef _DEAD_CODE_DIE
 #define SUPPRESS_GEOMETRY_SHADER_ERRORS
 #endif
@@ -91,7 +89,7 @@ using namespace std;
   } \
 }
 
-static void glShaderSource1String(GLuint shad, const string &strobj){
+static void glShaderSource1String(GLuint shad, const std::string &strobj){
   const GLchar *str = (const GLchar *)strobj.c_str();
   glShaderSource(shad, 1, (const GLchar **)&str, nullptr);
 }
@@ -102,7 +100,7 @@ bool CShaderPrg::reload(){
   if (is_valid || vertfile.empty())
     return true;
 
-  string gs, vs, fs;
+  std::string gs, vs, fs;
   CShaderMgr *I = G->ShaderMgr;
   GLint status;
 
@@ -244,7 +242,7 @@ PFNGLTEXIMAGE3DPROC getTexImage3D(){
 }
 #endif
 
-/*
+/**
  * Use this to turn off shaders if the renderer cannot use them.
  */
 void disableShaders(PyMOLGlobals * G) {
@@ -262,7 +260,7 @@ static void disableGeometryShaders(PyMOLGlobals * G) {
       " Geometry shaders not available\n" ENDFB(G);
 }
 
-/*
+/**
  * Replace strings from a list of pairs.
  *
  * src: string to modify
@@ -270,13 +268,13 @@ static void disableGeometryShaders(PyMOLGlobals * G) {
  *                 array like {from1, to1, from2, to2, ..., ""}
  * returns: new string
  */
-static string stringReplaceAll(const string &src, const string * replaceStrings) {
-  string dest = src;
+static std::string stringReplaceAll(const std::string &src, const std::string * replaceStrings) {
+  std::string dest = src;
   for (int i = 0; !replaceStrings[i].empty(); i += 2) {
     int slen1 = replaceStrings[i].length();
     int slen2 = replaceStrings[i + 1].length();
     for (size_t pl = 0;
-        (pl = dest.find(replaceStrings[i], pl)) != string::npos;
+        (pl = dest.find(replaceStrings[i], pl)) != std::string::npos;
         pl += slen2) {
       dest.replace(pl, slen1, replaceStrings[i + 1]);
     }
@@ -284,7 +282,7 @@ static string stringReplaceAll(const string &src, const string * replaceStrings)
   return dest;
 }
 
-/*
+/**
  * Reload "CallComputeColorForLight" shader replacement string
  */
 void CShaderMgr::Reload_CallComputeColorForLight(){
@@ -301,11 +299,11 @@ void CShaderMgr::Reload_CallComputeColorForLight(){
 
   int light_count = SettingGetGlobal_i(G, cSetting_light_count);
   int spec_count = SettingGetGlobal_i(G, cSetting_spec_count);
-  ostringstream accstr;
+  std::ostringstream accstr;
 
-  string rawtemplate = GetShaderSource("call_compute_color_for_light.fs");
+  std::string rawtemplate = GetShaderSource("call_compute_color_for_light.fs");
 
-  string lightstrings[] = {
+  std::string lightstrings[] = {
     "`light`", "0",
     "`postfix`", "_0",
     ""
@@ -324,7 +322,7 @@ void CShaderMgr::Reload_CallComputeColorForLight(){
   lightstrings[3] = "";
 
   for (int i=1; i<light_count; i++){
-    ostringstream lstr;
+    std::ostringstream lstr;
     lstr << i;
     lightstrings[1] = lstr.str(); // std::to_string(i)
 
@@ -368,10 +366,10 @@ void CShaderMgr::Reload_All_Shaders(){
 #define LOOKUP  32   // #ifdef or #ifndef or #include
 
 // preprocessor directive (like '#ifdef') -> bitmask
-static map<string, short> preprocmap;
+static std::map<std::string, short> preprocmap;
 
 // filename -> contents (static filesystem)
-static map<string, const char *> shader_cache_raw;
+static std::map<std::string, const char *> shader_cache_raw;
 
 // preproc variable -> NULL terminated list of filenames ("used by")
 std::map<std::string, const char **> ifdef_deps;
@@ -379,7 +377,7 @@ std::map<std::string, const char **> ifdef_deps;
 // filename -> NULL terminated list of filenames ("included by")
 std::map<std::string, const char **> include_deps;
 
-/*
+/**
  * Return a pointer to the next whitespace character or to the end of the string
  */
 static const char * nextwhitespace(const char * p) {
@@ -391,7 +389,7 @@ static const char * nextwhitespace(const char * p) {
   }
 }
 
-/*
+/**
  * Return a pointer to the next line beginning or to the end of the string.
  * Skips blank lines.
  */
@@ -413,24 +411,25 @@ switch2:
   }
 }
 
-/*
- * Get the processed shader file contents with all #ifdef and #include
+/**
+ * Get the processed shader file contents with all `#ifdef` and `#include`
  * preprocessors processed.
  *
  * Note: There must be a single whitespace character between preprocessor
  * directive and argument.
  *
  * Valid:
- * #ifdef foo
+ *
+ *     #ifdef foo
  *
  * Invalid:
- * # ifdef foo
- * #ifdef  foo
  *
- * Function arguments:
- * filename: file name of the shader file inside $PYMOL_DATA/shaders
+ *     # ifdef foo
+ *     #ifdef  foo
+ *
+ * @param filename file name of the shader file inside `$PYMOL_DATA/shaders`
  */
-string CShaderMgr::GetShaderSource(const string &filename)
+std::string CShaderMgr::GetShaderSource(const std::string &filename)
 {
   // processed cache
   auto it = shader_cache_processed.find(filename);
@@ -454,7 +453,7 @@ string CShaderMgr::GetShaderSource(const string &filename)
     const char * pymol_data = getenv("PYMOL_DATA");
 
     if (pymol_data && pymol_data[0]) {
-      string path(pymol_data);
+      std::string path(pymol_data);
       path.append(PATH_SEP).append("shaders").append(PATH_SEP).append(filename);
 
       try {
@@ -491,10 +490,10 @@ string CShaderMgr::GetShaderSource(const string &filename)
       tpl = nextwhitespace(pl);
 
       // copy of first word
-      string tmp_str(pl, tpl - pl);
+      std::string tmp_str(pl, tpl - pl);
 
       // lookup word in preprocmap
-      map<string, short>::const_iterator
+      std::map<std::string, short>::const_iterator
         preprocit = preprocmap.find(tmp_str);
       if (preprocit != preprocmap.end()) {
         preproc = preprocit->second;
@@ -503,7 +502,7 @@ string CShaderMgr::GetShaderSource(const string &filename)
           if (if_depth == true_depth) {
             // copy of second word
             tpl++;
-            tmp_str = string(tpl, nextwhitespace(tpl) - tpl);
+            tmp_str = std::string(tpl, nextwhitespace(tpl) - tpl);
 
             if (preproc & IFDEF) { // #ifdef or #ifndef
               bool if_value = false;
@@ -520,7 +519,7 @@ string CShaderMgr::GetShaderSource(const string &filename)
                 true_depth++;
 
             } else if (preproc & INCLUDE) { //#include
-              tmp_str = string(tpl, nextwhitespace(tpl) - tpl);
+              tmp_str = std::string(tpl, nextwhitespace(tpl) - tpl);
               newbuffer << GetShaderSource(tmp_str);
             }
           }
@@ -548,7 +547,7 @@ string CShaderMgr::GetShaderSource(const string &filename)
     }
   }
 
-  string result = newbuffer.str();
+  std::string result = newbuffer.str();
   shader_cache_processed[filename] = result;
   return result;
 }
@@ -576,7 +575,7 @@ void CShaderMgr::Reload_Shader_Variables() {
     SetPreprocVar("bg_image_mode_2_or_3", (bg_image_mode == 2 || bg_image_mode == 3));
   }
 
-#ifndef PYMOL_EDU
+#ifdef _PYMOL_IP_EXTRAS
   SetPreprocVar("volume_mode", SettingGetGlobal_i(G, cSetting_volume_mode));
 #endif
 
@@ -651,7 +650,7 @@ bool ShaderMgrInit(PyMOLGlobals * G) {
   return true;
 }
 
-/*
+/**
  * Print the given message as ShaderMgr-Error, followed by the shader info log.
  */
 void CShaderPrg::ErrorMsgWithShaderInfoLog(const GLuint sid, const char * msg) {
@@ -660,7 +659,7 @@ void CShaderPrg::ErrorMsgWithShaderInfoLog(const GLuint sid, const char * msg) {
 
   GLint infoLogLength = 0;
   glGetShaderiv(sid, GL_INFO_LOG_LENGTH, &infoLogLength);
-  vector<GLchar> infoLog(infoLogLength);
+  std::vector<GLchar> infoLog(infoLogLength);
   glGetShaderInfoLog(sid, infoLogLength, nullptr, infoLog.data());
 
   PRINTFB(G, FB_ShaderPrg, FB_Errors) " ShaderPrg-Error: %s; name='%s'\n",
@@ -920,7 +919,7 @@ CShaderMgr::~CShaderMgr() {
 int CShaderMgr::AddShaderPrg(CShaderPrg * s) {
   if (!s)
     return 0;
-  const string& name = s->name;
+  const std::string& name = s->name;
   if (programs.find(name)!=programs.end()){
     delete programs[name];
   }
@@ -928,20 +927,20 @@ int CShaderMgr::AddShaderPrg(CShaderPrg * s) {
   return 1;
 }
 
-int CShaderMgr::RemoveShaderPrg(const string& name) {
+int CShaderMgr::RemoveShaderPrg(const std::string& name) {
   if (programs.find(name) != programs.end()){
     delete programs[name];
   }
   return 1;
 }
 
-/*
+/**
  * Lookup a shader program by name and set it as the `current_shader` of the
  * shader manager. If `pass` is provided and is less than zero, and we are
  * in transparency_mode 3, then look up the NO_ORDER_TRANSP derivative.h
  */
-CShaderPrg * CShaderMgr::GetShaderPrg(std::string name, short set_current_shader, short pass) {
-  if (pass < 0 && SettingGetGlobal_i(G, cSetting_transparency_mode) == 3) {
+CShaderPrg * CShaderMgr::GetShaderPrg(std::string name, short set_current_shader, RenderPass pass) {
+  if (pass == RenderPass::Transparent && SettingGetGlobal_i(G, cSetting_transparency_mode) == 3) {
     name += "_t";
   }
 
@@ -967,7 +966,7 @@ int CShaderMgr::GeometryShadersPresent() {
   return shaders_present & MASK_SHADERS_PRESENT_GEOMETRY;
 }
 
-/*
+/**
  * glDeleteBuffers for vbos_to_free
  */
 void CShaderMgr::FreeAllVBOs() {
@@ -993,7 +992,7 @@ void CShaderMgr::AddVBOsToFree(GLuint *vboid, int nvbos){
   }
 }
 
-/*
+/**
  * thread-safe deferred glDeleteBuffers(1, &vboid)
  */
 void CShaderMgr::AddVBOToFree(GLuint vboid){
@@ -1011,27 +1010,27 @@ void CShaderMgr::AddVBOToFree(GLuint vboid){
 
 CShaderPrg *CShaderMgr::Enable_DefaultShaderWithSettings(
     const CSetting *set1,
-    const CSetting *set2, int pass) {
+    const CSetting *set2, RenderPass pass) {
   CShaderPrg * shaderPrg = Get_DefaultShader(pass);
   return Setup_DefaultShader(shaderPrg, set1, set2);
 }
 
-CShaderPrg *CShaderMgr::Enable_DefaultShader(int pass){
+CShaderPrg *CShaderMgr::Enable_DefaultShader(RenderPass pass){
   CShaderPrg * shaderPrg = Get_DefaultShader(pass);
   return Setup_DefaultShader(shaderPrg, nullptr, nullptr);
 }
 
-CShaderPrg *CShaderMgr::Enable_LineShader(int pass){
+CShaderPrg *CShaderMgr::Enable_LineShader(RenderPass pass){
   CShaderPrg * shaderPrg = Get_LineShader(pass);
   return Setup_DefaultShader(shaderPrg, nullptr, nullptr);
 }
 
-CShaderPrg *CShaderMgr::Enable_SurfaceShader(int pass){
+CShaderPrg *CShaderMgr::Enable_SurfaceShader(RenderPass pass){
   CShaderPrg * shaderPrg = Get_SurfaceShader(pass);
   return Setup_DefaultShader(shaderPrg, nullptr, nullptr);
 }
 
-CShaderPrg *CShaderMgr::Enable_ConnectorShader(int pass){
+CShaderPrg *CShaderMgr::Enable_ConnectorShader(RenderPass pass){
   CShaderPrg * shaderPrg = Get_ConnectorShader(pass);
   if (!shaderPrg)
     return nullptr;
@@ -1093,11 +1092,11 @@ CShaderPrg *CShaderMgr::Setup_DefaultShader(CShaderPrg * shaderPrg,
   shaderPrg->Set_Matrices();
   return (shaderPrg);
 }
-CShaderPrg *CShaderMgr::Enable_CylinderShader(int pass){
+CShaderPrg *CShaderMgr::Enable_CylinderShader(RenderPass pass){
   return Enable_CylinderShader("cylinder", pass);
 }
 
-CShaderPrg *CShaderMgr::Enable_CylinderShader(const char *shader_name, int pass){
+CShaderPrg *CShaderMgr::Enable_CylinderShader(const char *shader_name, RenderPass pass){
   int width, height;
   CShaderPrg *shaderPrg;
 
@@ -1130,12 +1129,12 @@ CShaderPrg *CShaderMgr::Enable_CylinderShader(const char *shader_name, int pass)
   return shaderPrg;
 }
 
-CShaderPrg *CShaderMgr::Get_DefaultSphereShader(int pass){
+CShaderPrg *CShaderMgr::Get_DefaultSphereShader(RenderPass pass){
   return GetShaderPrg("sphere", 1, pass);
 }
 
 
-CShaderPrg *CShaderMgr::Enable_DefaultSphereShader(int pass) {
+CShaderPrg *CShaderMgr::Enable_DefaultSphereShader(RenderPass pass) {
   CShaderPrg *shaderPrg = Get_DefaultSphereShader(pass);
   if (!shaderPrg) return nullptr;
   shaderPrg->Enable();
@@ -1152,49 +1151,27 @@ CShaderPrg *CShaderMgr::Enable_DefaultSphereShader(int pass) {
   return (shaderPrg);
 }
 
-#ifdef _PYMOL_ARB_SHADERS
-CShaderPrg *CShaderMgr::Enable_SphereShaderARB(){
-  CShaderPrg *shaderPrg = nullptr;
-  /* load the vertex program */
-  Disable_Current_Shader();
-  shaderPrg = GetShaderPrg("sphere_arb");
-  glBindProgramARB(GL_VERTEX_PROGRAM_ARB, shaderPrg->vid);
-  
-  /* load the fragment program */
-  glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, shaderPrg->fid);
-  
-  /* load some safe initial values  */
-  glProgramEnvParameter4fARB(GL_VERTEX_PROGRAM_ARB, 0, 0.0F, 0.0F, 1.0F, 0.0F);
-  glProgramEnvParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, 0.5F, 2.0F, 0.0F, 0.0F);
-
-  glEnable(GL_VERTEX_PROGRAM_ARB);
-  glEnable(GL_FRAGMENT_PROGRAM_ARB);
-
-  return shaderPrg;
-}
-#endif
-
-CShaderPrg *CShaderMgr::Get_ConnectorShader(int pass){
+CShaderPrg *CShaderMgr::Get_ConnectorShader(RenderPass pass){
   return GetShaderPrg("connector", 1, pass);
 }
 
-CShaderPrg *CShaderMgr::Get_DefaultShader(int pass){
+CShaderPrg *CShaderMgr::Get_DefaultShader(RenderPass pass){
   return GetShaderPrg("default", 1, pass);
 }
 
-CShaderPrg *CShaderMgr::Get_LineShader(int pass){
+CShaderPrg *CShaderMgr::Get_LineShader(RenderPass pass){
   return GetShaderPrg("line", 1, pass);
 }
 
-CShaderPrg *CShaderMgr::Get_SurfaceShader(int pass){
+CShaderPrg *CShaderMgr::Get_SurfaceShader(RenderPass pass){
   return GetShaderPrg("surface", 1, pass);
 }
 
-CShaderPrg *CShaderMgr::Get_CylinderShader(int pass, short set_current_shader) {
+CShaderPrg *CShaderMgr::Get_CylinderShader(RenderPass pass, short set_current_shader) {
   return GetShaderPrg("cylinder", set_current_shader, pass);
 }
 
-CShaderPrg *CShaderMgr::Get_CylinderNewShader(int pass, short set_current_shader) {
+CShaderPrg *CShaderMgr::Get_CylinderNewShader(RenderPass pass, short set_current_shader) {
   return GetShaderPrg("cylinder_new", set_current_shader, pass);
 }
 
@@ -1282,7 +1259,7 @@ CShaderPrg *CShaderMgr::Enable_OITCopyShader() {
   return shaderPrg;
 }
 
-CShaderPrg *CShaderMgr::Enable_LabelShader(int pass){
+CShaderPrg *CShaderMgr::Enable_LabelShader(RenderPass pass){
   CShaderPrg *shaderPrg;
   shaderPrg = Get_LabelShader(pass);  
   if (!shaderPrg)
@@ -1348,7 +1325,7 @@ CShaderPrg *CShaderMgr::Setup_LabelShader(CShaderPrg *shaderPrg) {
   return shaderPrg;
 }
 
-CShaderPrg *CShaderMgr::Get_LabelShader(int pass){
+CShaderPrg *CShaderMgr::Get_LabelShader(RenderPass pass){
   return GetShaderPrg("label", 1, pass);
 }
 
@@ -1413,7 +1390,7 @@ int light_setting_indices[] = {
   cSetting_light9
 };
 
-/*
+/**
  * Generate and upload a precomputed vec2(ambient, specular) lighting texture.
  *
  * Must be equivalent to "ComputeLighting" in "compute_color_for_light.fs"
@@ -1586,7 +1563,7 @@ GLfloat *CShaderMgr::GetLineWidthRange() {
 #ifndef _PYMOL_NO_AA_SHADERS
 #endif
 
-/*
+/**
  * Register filename -> shader dependencies for `shader`
  */
 void CShaderMgr::RegisterDependantFileNames(CShaderPrg * shader) {
@@ -1596,7 +1573,7 @@ void CShaderMgr::RegisterDependantFileNames(CShaderPrg * shader) {
     shader_deps[shader->geomfile].push_back(shader->name);
 }
 
-/*
+/**
  * Recursive function to insert `filename` and all the files where
  * `filename` is included into the given output vector.
  */
@@ -1612,7 +1589,7 @@ void CShaderMgr::CollectDependantFileNames(const std::string &filename,
   filenames.push_back(filename);
 }
 
-/*
+/**
  * Make derived shaders for all shaders that depend on `variable`
  */
 void CShaderMgr::MakeDerivatives(const std::string &suffix, const std::string &variable) {
@@ -1643,7 +1620,7 @@ void CShaderMgr::MakeDerivatives(const std::string &suffix, const std::string &v
   }
 }
 
-/*
+/**
  * Reload the derivative shaders for `variable`
  */
 void CShaderMgr::Reload_Derivatives(const std::string &variable, bool value) {
@@ -1657,7 +1634,7 @@ void CShaderMgr::Reload_Derivatives(const std::string &variable, bool value) {
   SetPreprocVar(variable, !value, false);
 }
 
-/*
+/**
  * Removes `filename` and all it's parents from the shader source cache,
  * and if `invshaders` is true, also clear the `is_valid` flag for all
  * shader infos that depend on `filename`.
@@ -1687,8 +1664,8 @@ void CShaderMgr::ShaderSourceInvalidate(const char * filename, bool invshaders)
   }
 }
 
-/*
- * Set the value for the #ifdef variable `key` and if the value has changed,
+/**
+ * Set the value for the `#ifdef` variable `key` and if the value has changed,
  * then invalidate all its dependant shader source files.
  */
 void CShaderMgr::SetPreprocVar(const std::string &key, bool value, bool invshaders) {
@@ -1702,7 +1679,7 @@ void CShaderMgr::SetPreprocVar(const std::string &key, bool value, bool invshade
   }
 }
 
-/*
+/**
  * Insert `filename` -> `contents` (processed source) into the shader source
  * cache and invalidate its parents
  */
