@@ -44,10 +44,6 @@ Z* -------------------------------------------------------------------
 #define LOCK_GUARD_MUTEX(name, var)
 #endif
 
-#ifndef GL_FRAGMENT_PROGRAM_ARB
-#define GL_FRAGMENT_PROGRAM_ARB                         0x8804
-#endif
-
 /* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */
 #if 0
 PFNGLTEXIMAGE3DPROC getTexImage3D();
@@ -101,11 +97,6 @@ public:
     return copy;
   }
 
-#ifdef _PYMOL_ARB_SHADERS
-  static CShaderPrg *NewARB(PyMOLGlobals * G, const char * name, const std::string& vert, const std::string& frag);
-  int DisableARB();
-#endif
-
 /* Enable */
   int Enable();
 
@@ -215,7 +206,7 @@ public:
   int RemoveShaderPrg(const std::string& name);
 
 /* GetShaderPrg -- gets a ptr to the installed shader */
-  CShaderPrg * GetShaderPrg(std::string name, short set_current_shader = 1, short pass = 0);
+  CShaderPrg * GetShaderPrg(std::string name, short set_current_shader = 1, RenderPass pass = RenderPass::Antialias);
 
   int ShaderPrgExists(const char * name);
 
@@ -227,21 +218,18 @@ public:
   void AddVBOToFree(GLuint vboid);
   void FreeAllVBOs();
 
-  CShaderPrg *Enable_DefaultShader(int pass);
-  CShaderPrg *Enable_LineShader(int pass);
-  CShaderPrg *Enable_SurfaceShader(int pass);
-  CShaderPrg *Enable_DefaultShaderWithSettings(const CSetting * set1, const CSetting * set2, int pass);
-  CShaderPrg *Enable_CylinderShader(const char *, int pass);
-  CShaderPrg *Enable_CylinderShader(int pass);
-  CShaderPrg *Enable_DefaultSphereShader(int pass);
-#ifdef _PYMOL_ARB_SHADERS
-  CShaderPrg *Enable_SphereShaderARB();
-#endif
+  CShaderPrg *Enable_DefaultShader(RenderPass pass);
+  CShaderPrg *Enable_LineShader(RenderPass pass);
+  CShaderPrg *Enable_SurfaceShader(RenderPass pass);
+  CShaderPrg *Enable_DefaultShaderWithSettings(const CSetting * set1, const CSetting * set2, RenderPass pass);
+  CShaderPrg *Enable_CylinderShader(const char *, RenderPass pass);
+  CShaderPrg *Enable_CylinderShader(RenderPass pass);
+  CShaderPrg *Enable_DefaultSphereShader(RenderPass pass);
   CShaderPrg *Enable_RampShader();
-  CShaderPrg *Enable_ConnectorShader(int pass);
+  CShaderPrg *Enable_ConnectorShader(RenderPass pass);
   CShaderPrg *Enable_TriLinesShader();
   CShaderPrg *Enable_ScreenShader();
-  CShaderPrg *Enable_LabelShader(int pass);
+  CShaderPrg *Enable_LabelShader(RenderPass pass);
   CShaderPrg *Enable_OITShader();
   CShaderPrg *Enable_OITCopyShader();
   CShaderPrg *Enable_IndicatorShader();
@@ -250,18 +238,18 @@ public:
   void Disable_Current_Shader();
 
   CShaderPrg *Get_ScreenShader();
-  CShaderPrg *Get_ConnectorShader(int pass);
-  CShaderPrg *Get_DefaultShader(int pass);
-  CShaderPrg *Get_LineShader(int pass);
-  CShaderPrg *Get_SurfaceShader(int pass);
-  CShaderPrg *Get_CylinderShader(int pass, short set_current_shader=1);
-  CShaderPrg *Get_CylinderNewShader(int pass, short set_current_shader=1);
-  CShaderPrg *Get_DefaultSphereShader(int pass);
+  CShaderPrg *Get_ConnectorShader(RenderPass pass);
+  CShaderPrg *Get_DefaultShader(RenderPass pass);
+  CShaderPrg *Get_LineShader(RenderPass pass);
+  CShaderPrg *Get_SurfaceShader(RenderPass pass);
+  CShaderPrg *Get_CylinderShader(RenderPass pass, short set_current_shader=1);
+  CShaderPrg *Get_CylinderNewShader(RenderPass pass, short set_current_shader=1);
+  CShaderPrg *Get_DefaultSphereShader(RenderPass pass);
   CShaderPrg *Get_RampShader();
   CShaderPrg *Get_Current_Shader();
   CShaderPrg *Get_IndicatorShader();
   CShaderPrg *Get_BackgroundShader();
-  CShaderPrg *Get_LabelShader(int pass);
+  CShaderPrg *Get_LabelShader(RenderPass pass);
 
   void Reload_CallComputeColorForLight();
   void Reload_All_Shaders();
@@ -357,7 +345,7 @@ public:
    * Activates/Binds offscreen render target.
    * @param textureIdx offset of texture unit to assign (0 for GL_TEXTURE0, 1
    * for GL_TEXTURE1, etc...)
-   * @Note: indices should preferably be passed in as enum or named variable for
+   * @note: indices should preferably be passed in as enum or named variable for
    * clarity
    */
   void activateOffscreenTexture(GLuint textureIdx);
