@@ -18,23 +18,24 @@ Z* -------------------------------------------------------------------
 #ifndef _H_Tetsurf
 #define _H_Tetsurf
 
-#include"Map.h"
-#include"MemoryDebug.h"
-#include"Crystal.h"
-#include"Field.h"
 #include"PyMOLGlobals.h"
+#include"PyMOLEnums.h"
+#include"vla.h"
 
-#define F3(field,P1,P2,P3) Ffloat3(field,P1,P2,P3)
-#define F3Ptr(field,P1,P2,P3) Ffloat3p(field,P1,P2,P3)
+struct CCrystal;
+struct Isofield;
+class CarveHelper;
 
-#define F4(field,P1,P2,P3,P4) Ffloat4(field,P1,P2,P3,P4)
-#define F4Ptr(field,P1,P2,P3,P4) Ffloat4p(field,P1,P2,P3,P4)
+int TetsurfVolume(PyMOLGlobals* G, Isofield* field, float level,
+    pymol::vla<int>& num,    //
+    pymol::vla<float>& vert, //
+    const int* range,        //
+    cIsosurfaceMode mode,    //
+    const CarveHelper*,      //
+    cIsosurfaceSide side);
 
-int TetsurfVolume(PyMOLGlobals * G, Isofield * field, float level, int **num,
-                  float **vert, int *range, int mode,
-                  MapType * voxelmap, const float* a_vert, float carvebuffer, int side);
-void TetsurfGetRange(PyMOLGlobals * G, Isofield * field, CCrystal * cryst, float *mn,
-                     float *mx, int *range);
+void TetsurfGetRange(PyMOLGlobals* G, const Isofield* field,
+    const CCrystal* cryst, const float* mn, const float* mx, int* range);
 
 int TetsurfInit(PyMOLGlobals * G);
 void TetsurfFree(PyMOLGlobals * G);
