@@ -658,6 +658,9 @@ namespace cgo {
       };
       float vertex1[3], vertex2[3], radius, color1[3], color2[3];
       float cap1, cap2;
+
+      cCylCap get_cap1() const { return static_cast<cCylCap>(int(cap1)); }
+      cCylCap get_cap2() const { return static_cast<cCylCap>(int(cap2)); }
     };
 
     struct custom_cylinder_alpha {
@@ -675,6 +678,9 @@ namespace cgo {
       };
       float vertex1[3], vertex2[3], radius, color1[4], color2[4];
       float cap1, cap2;
+
+      cCylCap get_cap1() const { return static_cast<cCylCap>(int(cap1)); }
+      cCylCap get_cap2() const { return static_cast<cCylCap>(int(cap2)); }
     };
   };
 };
@@ -684,6 +690,8 @@ public:
   CGO(PyMOLGlobals* G, int size = 0);
   ~CGO();
 
+  CGO(CGO const& other) = delete;
+
   PyMOLGlobals *G { nullptr };
   float *op { nullptr };
   size_t c = 0;
@@ -792,8 +800,9 @@ public:
   }
 
   // Appends the source CGO onto this CGO
-  int append(const CGO * source, bool stopAtEnd);
-  void move_append(CGO * source);
+  bool append(const CGO& source, bool stopAtEnd = false);
+  void move_append(CGO&& source);
+  void free_append(CGO * &&source);
   void free_append(CGO * &source);
 
   // Allocates in our CGO data pool
@@ -866,25 +875,27 @@ void CGOFree(CGO * &I, bool withVBOs=true);
 
 CGO *CGODrawText(const CGO * I, int est, float *camera);
 
-CGO *CGOSimplify(const CGO * I, int est, short sphere_quality = -1, bool stick_round_nub = true);
+CGO* CGOSimplify(const CGO* I, int est = 0, short sphere_quality = -1,
+    bool stick_round_nub = true);
 CGO *CGOSimplifyNoCompress(const CGO * I, int est, short sphere_quality = -1, bool stick_round_nub = true);
 
 // -1 - no lines, 0 - some no interpolation, 1 - all interpolation, 2 - all no interpolation
 bool CGOCombineBeginEnd(CGO ** I, bool do_not_split_lines = false);
-CGO *CGOCombineBeginEnd(const CGO * I, int est, bool do_not_split_lines = false);
+CGO* CGOCombineBeginEnd(const CGO* I, int est = 0, bool do_not_split_lines = false);
 
 void CGOFreeVBOs(CGO *I);
 
-CGO *CGOOptimizeToVBOIndexed(CGO * I, int est=0, const float *color=NULL, bool addshaders=true, bool embedTransparencyInfo=false);
+CGO *CGOOptimizeToVBOIndexed(const CGO * I, int est=0, const float *color=NULL, bool addshaders=true, bool embedTransparencyInfo=false);
 #define CGOOptimizeToVBOIndexedWithColorEmbedTransparentInfo(I, est, color, addshaders) CGOOptimizeToVBOIndexed(I, est, color, addshaders, true)
 #define CGOOptimizeToVBOIndexedWithColor CGOOptimizeToVBOIndexed
 #define CGOOptimizeToVBOIndexedNoShader(I, est) CGOOptimizeToVBOIndexed(I, est, NULL, false)
 
 bool CGOOptimizeToVBONotIndexed(CGO ** I);
-CGO *CGOOptimizeToVBONotIndexed(const CGO * I, int est, bool addshaders=true, float **returnedData=NULL);
+CGO* CGOOptimizeToVBONotIndexed(const CGO* I, int est = 0,
+    bool addshaders = true, float** returnedData = nullptr);
 
 #define CGOOptimizeToVBONotIndexedWithReturnedData CGOOptimizeToVBONotIndexed
-#define CGOOptimizeToVBONotIndexedNoShader(I, est) CGOOptimizeToVBONotIndexed(I, est, false)
+#define CGOOptimizeToVBONotIndexedNoShader(I) CGOOptimizeToVBONotIndexed(I, 0, false)
 
 
 CGO *CGOOptimizeSpheresToVBONonIndexed(const CGO * I, int est=0, bool addshaders=false, CGO *leftOverCGO=NULL);
@@ -902,12 +913,9 @@ int CGOEnd(CGO * I);
 
 int CGOSphere(CGO * I, const float *v1, float r);
 int CGOEllipsoid(CGO * I, const float *v1, float r, const float *n1, const float *n2, const float *n3);
-int CGOQuadric(CGO * I, const float *v1, float r, const float *p); /* NOT WORKING YET */
 int CGOVertex(CGO * I, float v1, float v2, float v3);
 int CGOVertexv(CGO * I, const float *v);
 int CGOVertexCrossv(CGO * I, const float *v);
-int CGOVertexBeginLineStripv(CGO * I, const float *v);
-int CGOInterpolated(CGO * I, const bool interp);
 int CGOAlpha(CGO * I, float alpha);
 int CGOColor(CGO * I, float v1, float v2, float v3);
 int CGOColorv(CGO * I, const float *v);
@@ -948,9 +956,7 @@ int CGOSpecialWithArg(CGO * I, int v, float arg);
 #define LINE_LIGHTING  2
 
 int CGODotwidth(CGO * I, float v);
-int CGOChar(CGO * I, char c);
 int CGOFontVertex(CGO * I, float x, float y, float z);
-int CGOFontVertexv(CGO * I, const float *v);
 int CGOFontScale(CGO * I, float v1, float v2);
 int CGOIndent(CGO * I, char c, float dir);
 int CGOWrite(CGO * I, const char *str);
@@ -966,7 +972,6 @@ int CGOBoundingBox(CGO *I, const float *min, const float *max);
 int CGOAccessibility(CGO * I, const float a);
 
 int CGODrawTexture(CGO *I, int texture_id, float *worldPos, float *screenMin, float *screenMax, float *textExtent);
-int CGODrawLabel(CGO *I, int texture_id, float *targetPos, float *worldPos, float *screenWorldOffset, float *screenMin, float *screenMax, float *textExtent, short relativeMode);
 int CGODrawConnector(CGO *I, float *targetPt3d, float *labelCenterPt3d, float text_width, float text_height, float *screenOffset, float *screenWorldOffset, float *connectorColor, short relativeMode, int draw_bkgrd, float bkgrd_transp, float *bkgrd_color, float rel_ext_length, float connectorWidth);
 CGO *CGOOptimizeLabels(const CGO * I, int est, bool addshaders=false);
 CGO *CGOOptimizeTextures(const CGO * I, int est);
@@ -990,9 +995,6 @@ int CGODisable(CGO * I, int mode);
 
 int CGOStop(CGO * I);
 
-int CGOConev(CGO * I, const float *p1, const float *p2, float r1, float r2, const float *c1, const float *c2,
-              float cap1, float cap2);
-
 int CGOAlphaTriangle(CGO * I,
 		     const float *v1, const float *v2, const float *v3,
 		     const float *n1, const float *n2, const float *n3,
@@ -1037,12 +1039,10 @@ bool CGOFilterOutCylinderOperationsInto(const CGO *I, CGO *cgo);
 bool CGOCheckWhetherToFree(PyMOLGlobals * G, CGO *I);
 
 CGO *CGOConvertLinesToShaderCylinders(const CGO * I, int est);
-CGO *CGOSplitUpLinesForPicking(const CGO * I);
 CGO *CGOConvertLinesToTrilines(const CGO * I, bool addshaders=true);
 CGO *CGOConvertToLabelShader(const CGO *I, CGO * addTo);
 
 void CGOChangeShadersTo(CGO *I, int frommode, int tomode);
-void CGOCountNumVerticesDEBUG(const CGO *I);
 CGO *CGOOptimizeScreenTexturesAndPolygons(CGO * I, int est);
 CGO *CGOColorByRamp(PyMOLGlobals * G, const CGO *I, ObjectGadgetRamp *ramp, int state, CSetting * set1);
 
@@ -1092,10 +1092,6 @@ int CGOUniform3f(CGO *I, int uniform_id, const float *value);
 
 CGO *CGOConvertSpheresToPoints(const CGO *I);
 
-#ifdef _PYMOL_ARB_SHADERS
-void CGORenderSpheresARB(RenderInfo *info, const CGO *I, const float *fog_info);
-#endif
-
 CGO *CGOConvertToShader(const CGO *I, AttribDataDesc &attrData, AttribDataDesc &pickData, int mode, const VertexBuffer::buffer_layout layout=VertexBuffer::INTERLEAVED, bool check_attr_for_data=true, int *idx_array=NULL, int nindicesperfrag=0, int nfragspergroup = 1);
 
 bool CGOCheckSplitLineInterpolationIsSame(const CGO *I, bool &interp_value);
