@@ -105,9 +105,12 @@ int MovieViewModify(PyMOLGlobals *G,int action, int index, int count,int target,
       }
       break;
     case cViewElemModifyDelete:
-      I->Sequence.erase(index, count);
-      I->Cmd.erase(I->Cmd.begin() + index, I->Cmd.begin() + index + count);
-      I->NFrame = VLAGetSize(I->Sequence);
+      {
+        I->Sequence.erase(index, count);
+        int end_pos = std::min(index + count, static_cast<int>(I->Cmd.size()));
+        I->Cmd.erase(I->Cmd.begin() + index, I->Cmd.begin() + end_pos);
+        I->NFrame = VLAGetSize(I->Sequence);
+      }
       break;
     case cViewElemModifyMove:
       if((index>=0) && (target>=0) && (index<I->NFrame) && (target<I->NFrame)) {
@@ -850,7 +853,7 @@ int MoviePNG(PyMOLGlobals * G, const char* prefix, int save, int start,
     SettingSetGlobal_b(G, cSetting_seq_view, 0);
     // force viewport update
     SeqChanged(G);
-    OrthoDoDraw(G, 0);
+    OrthoDoDraw(G, OrthoRenderMode::Main);
   }
 
   M->modal = modal;
@@ -867,7 +870,20 @@ int MoviePNG(PyMOLGlobals * G, const char* prefix, int save, int start,
 
 
 /*========================================================================*/
-void MovieAppendSequence(PyMOLGlobals * G, const char *str, int start_from,int freeze)
+void MovieSet(PyMOLGlobals* G, pymol::zstring_view specification,
+    int start_from, bool freeze)
+{
+  MovieAppendSequence(G, specification.c_str(), start_from, freeze);
+  SceneCountFrames(G);
+
+  // fix for PYMOL-1465
+  // force GUI update for movie panel
+  if(G->HaveGUI)
+  OrthoReshape(G, -1, -1, false);
+}
+
+/*========================================================================*/
+void MovieAppendSequence(PyMOLGlobals* G, const char* str, int start_from, bool freeze)
 {
   CMovie *I = G->Movie;
   int c = 0;
@@ -1448,7 +1464,7 @@ Block *MovieGetBlock(PyMOLGlobals * G)
 
 
 void MoviePrepareDrag(PyMOLGlobals *G, BlockRect * rect, 
-                      CObject * obj, int mode, int x, int y, int nearest)
+                      pymol::CObject * obj, int mode, int x, int y, int nearest)
 {
   CMovie *I = G->Movie;
   I->DragMode = mode;
