@@ -33,6 +33,7 @@ Z* -------------------------------------------------------------------
 #include"Deferred.h"
 #include"Image.h"
 #include"pymol/memory.h"
+#include"PyMOLEnums.h"
 
 #define cOrthoScene 1
 #define cOrthoTool 2
@@ -47,7 +48,7 @@ void OrthoDetach(PyMOLGlobals * G, Block * block);
 void OrthoReshape(PyMOLGlobals * G, int width, int height, int force);
 int OrthoGetWidth(PyMOLGlobals * G);
 int OrthoGetHeight(PyMOLGlobals * G);
-void OrthoDoDraw(PyMOLGlobals * G, int render_mode);
+void OrthoDoDraw(PyMOLGlobals * G, OrthoRenderMode render_mode);
 void OrthoDoViewportWhenReleased(PyMOLGlobals *G);
 void OrthoPushMatrix(PyMOLGlobals * G);
 void OrthoPopMatrix(PyMOLGlobals * G);
@@ -110,7 +111,7 @@ void OrthoDefer(PyMOLGlobals * G, std::unique_ptr<CDeferred> && D);
 void OrthoExecDeferred(PyMOLGlobals * G);
 int OrthoDeferredWaiting(PyMOLGlobals * G);
 
-int OrthoGetRenderMode(PyMOLGlobals * G);
+OrthoRenderMode OrthoGetRenderMode(PyMOLGlobals * G);
 void OrthoDrawBuffer(PyMOLGlobals * G, GLenum mode);
 int OrthoGetWrapClickSide(PyMOLGlobals * G);
 float *OrthoGetOverlayColor(PyMOLGlobals * G);
