@@ -22,6 +22,7 @@ Z* -------------------------------------------------------------------
 #include"PyMOLGlobals.h"
 #include"Base.h"
 #include"OVLexicon.h"
+#include "Result.h"
 
 #include <array>
 #include <map>
@@ -64,8 +65,10 @@ Z* -------------------------------------------------------------------
 #define CPythonVal_PConvPyListToIntArrayInPlace_From_List(G, list, i, ff, ll)   PConvPyListToIntArrayInPlace(PyList_GetItem(list, i), ff, ll)
 #define CPythonVal_PConvPyListToFloatArrayInPlaceAutoZero_From_List(G, list, i, ...) \
                    PConvPyListToFloatArrayInPlaceAutoZero(PyList_GetItem(list, i), __VA_ARGS__)
+#define CPythonVal_PConvPyListToFloatArrayInPlace_From_List(G, list, i, ...) \
+                   PConvPyListToFloatArrayInPlace(PyList_GetItem(list, i), __VA_ARGS__)
 #define CPythonVal_PConvPyListToFloatVLANoneOkay_From_List(G, list, i, f)       PConvPyListToFloatVLANoneOkay(PyList_GetItem(list, i), f)
-#define CPythonVal_PConvPyListToLabPosVLA(G, obj, vla_ptr)                      PConvPyListToLabPosVLA(obj, vla_ptr)
+#define CPythonVal_PConvPyListToLabPosVec(G, obj)                               PConvPyListToLabPosVec(obj)
 #define CPythonVal_PConvPyStrToStr_From_List(G, list, i, ptr, l)                PConvPyStrToStr(PyList_GetItem(list, i), ptr, l)
 
 #define CPythonVal_Free(obj)
@@ -113,7 +116,7 @@ int PConvPyFloatToFloat(PyObject * obj, float *ptr);
 int PConvPyIntToChar(PyObject * obj, char *ptr);
 int PConvPyIntToInt(PyObject * obj, int *ptr);
 int PConvPyBoolToInt(PyObject * obj, int *ptr);
-int PConvPyListToLabPosVLA(PyObject * obj, LabPosType ** vla_ptr);
+pymol::Result<std::vector<LabPosType>> PConvPyListToLabPosVec(PyObject* obj);
 
 
 /* Jenarix conventions -- returns before args */
@@ -134,7 +137,7 @@ PyObject *PConvIntVLAToPyTuple(int *vla);
 PyObject *PConvIntArrayToPyList(const int *f, int l, bool dump_binary=false);
 PyObject *PConvSIntArrayToPyList(const short int *f, int l);
 PyObject *PConvSCharArrayToPyList(const signed char *f, int l);
-PyObject *PConvLabPosVLAToPyList(const LabPosType * vla, int l);
+PyObject *PConvLabPosVecToPyList(const std::vector<LabPosType>& vec);
 
 void PConvFloat3ToPyObjAttr(PyObject * obj, const char *attr, const float *v);
 void PConvFloatToPyObjAttr(PyObject * obj, const char *attr, float f);
@@ -204,6 +207,14 @@ inline PyObject * PConvToPyObject(int v) {
   return PyInt_FromLong(v);
 }
 
+inline PyObject * PConvToPyObject(std::size_t v) {
+#ifndef _PYMOL_NOPY
+  return PyLong_FromSize_t(v);
+#else
+  return PyInt_FromLong(v);
+#endif
+}
+
 inline PyObject * PConvToPyObject(float v) {
   return PyFloat_FromDouble(v);
 }
@@ -252,7 +263,7 @@ PyObject * PConvToPyObject(const std::array<T, N> &arr) {
   return o;
 }
 
-/*
+/**
  * Convert a set to a Python list
  */
 template <class T>
@@ -267,10 +278,10 @@ PyObject * PConvToPyObject(const std::set<T> &v) {
   return o;
 }
 
-/*
+/**
  * Convert a map to a flat Python list
  *
- * {k1: v1, k2: v2, ...} -> [k1, v1, k2, v2, ...]
+ *     {k1: v1, k2: v2, ...} -> [k1, v1, k2, v2, ...]
  */
 template <class K, class V>
 PyObject * PConvToPyObject(const std::map<K, V> &v) {
@@ -285,7 +296,7 @@ PyObject * PConvToPyObject(const std::map<K, V> &v) {
   return o;
 }
 
-/*
+/**
  * Convert a pair to a Python tuple
  */
 template <class T1, class T2>
@@ -311,17 +322,18 @@ inline PyObject* PConvToPyObject(const pymol::Void&)
 }
 
 /* ============================================================ */
-/*
+/**
  * PConvFromPyObject: Templated conversion of a python object to a
  * standart type (primitives and c++ std library).
  */
-
 template <typename Int,
-    typename std::enable_if<std::is_integral<Int>::value>::type* = nullptr>
+    typename std::enable_if<std::is_integral<Int>::value ||
+                            std::is_enum<Int>::value>::type* = nullptr>
 inline bool PConvFromPyObject(PyMOLGlobals*, PyObject* obj, Int& out)
 {
-  out = PyInt_AsLong(obj);
-  return true;
+  auto const value = PyInt_AsLong(obj);
+  out = static_cast<Int>(value);
+  return value != -1 || !PyErr_Occurred();
 }
 
 template <typename Float,
@@ -330,11 +342,21 @@ template <typename Float,
 inline bool PConvFromPyObject(PyMOLGlobals*, PyObject* obj, Float& out)
 {
   out = PyFloat_AsDouble(obj);
-  return true;
+  return out != -1.0 || !PyErr_Occurred();
 }
 
 inline bool PConvFromPyObject(PyMOLGlobals *, PyObject * obj, std::string &out) {
+#ifdef _PYMOL_NOPY
+  // this would also work with real Python but without error handling (could
+  // segfault if `obj` is not of type `str`)
   out = PyString_AsSomeString(obj);
+#else
+  const char* buffer = PyUnicode_AsUTF8(obj);
+  if (!buffer) {
+    return false;
+  }
+  out = buffer;
+#endif
   return true;
 }
 
@@ -380,7 +402,7 @@ bool PConvFromPyObject(PyMOLGlobals * G, PyObject * obj, std::vector<T> &out) {
   return true;
 }
 
-/*
+/**
  * Convert a Python list to a set
  */
 template <class T>
@@ -405,11 +427,11 @@ bool PConvFromPyObject(PyMOLGlobals * G, PyObject * obj, std::set<T> &out) {
   return true;
 }
 
-/*
+/**
  * Convert a flat Python list to a map, even indices are keys and odd
  * indices are values.
  *
- * [a, b, c, d, ...] -> {a: b, c: d, ...}
+ *     [a, b, c, d, ...] -> {a: b, c: d, ...}
  */
 template <class K, class V>
 bool PConvFromPyObject(PyMOLGlobals * G, PyObject * obj, std::map<K, V> &out) {
