@@ -38,21 +38,21 @@ Z* -------------------------------------------------------------------
 #include"CGO.h"
 #include"Selector.h"
 #include"vla.h"
+#include"pymol/type_traits.h"
 
-void ObjectPurgeSettings(CObject * I)
+void ObjectPurgeSettings(pymol::CObject * I)
 {
-  SettingFreeP(I->Setting);
-  I->Setting = NULL;
+  I->Setting.reset();
 }
 
-void ObjectMotionTrim(CObject *I, int n_frame)
+void ObjectMotionTrim(pymol::CObject *I, int n_frame)
 {
   if(I->ViewElem) {
     VLASize(I->ViewElem,CViewElem,n_frame);
   }
 }
 
-int ObjectMotionGetLength(CObject *I)
+int ObjectMotionGetLength(pymol::CObject *I)
 {
   if(I->ViewElem) {
     return VLAGetSize(I->ViewElem);
@@ -60,16 +60,16 @@ int ObjectMotionGetLength(CObject *I)
   return 0;
 }
 
-void ObjectMotionReinterpolate(CObject *I)
+void ObjectMotionReinterpolate(pymol::CObject *I)
 {
-  float power  = SettingGet_f(I->G, NULL, I->Setting, cSetting_motion_power);
-  float bias   = SettingGet_f(I->G, NULL, I->Setting, cSetting_motion_bias);
-  int simple   = SettingGet_i(I->G, NULL, I->Setting, cSetting_motion_simple);
-  float linear = SettingGet_f(I->G, NULL, I->Setting, cSetting_motion_linear);
-  int hand     = SettingGet_i(I->G, NULL, I->Setting, cSetting_motion_hand);
+  float power  = SettingGet_f(I->G, NULL, I->Setting.get(), cSetting_motion_power);
+  float bias   = SettingGet_f(I->G, NULL, I->Setting.get(), cSetting_motion_bias);
+  int simple   = SettingGet_i(I->G, NULL, I->Setting.get(), cSetting_motion_simple);
+  float linear = SettingGet_f(I->G, NULL, I->Setting.get(), cSetting_motion_linear);
+  int hand     = SettingGet_i(I->G, NULL, I->Setting.get(), cSetting_motion_hand);
 
-  /* 
-     int ObjectMotion(CObject * I, int action, int first,
+  /*
+     int ObjectMotion(pymol::CObject * I, int action, int first,
                  int last, float power, float bias,
                  int simple, float linear, int wrap,
                  int hand, int window, int cycles, int state, int quiet);
@@ -79,7 +79,7 @@ void ObjectMotionReinterpolate(CObject *I)
                hand, 5, 1, -1, 1);
 }
 
-int ObjectMotionModify(CObject *I,int action, int index, int count,int target,int freeze,int localize)
+int ObjectMotionModify(pymol::CObject *I,int action, int index, int count,int target,int freeze,int localize)
 {
   int ok;
 
@@ -198,7 +198,7 @@ static void TTTFromViewElem(float *TTT, CViewElem * elem)
   fp[15] = 1.0F;
 }
 
-int ObjectGetSpecLevel(CObject * I, int frame)
+int ObjectGetSpecLevel(pymol::CObject * I, int frame)
 {
   if(I->ViewElem) {
     int size = VLAGetSize(I->ViewElem);
@@ -218,14 +218,14 @@ int ObjectGetSpecLevel(CObject * I, int frame)
   return -1;
 }
 
-void ObjectDrawViewElem(CObject *I, BlockRect *rect,int frames ORTHOCGOARG)
+void ObjectDrawViewElem(pymol::CObject *I, BlockRect *rect,int frames ORTHOCGOARG)
 {
   if(I->ViewElem) {
     ViewElemDraw(I->G,I->ViewElem,rect,frames,I->Name ORTHOCGOARGVAR);
   }
 }
 
-int ObjectMotion(CObject * I, int action, int first,
+int ObjectMotion(pymol::CObject * I, int action, int first,
                int last, float power, float bias,
                int simple, float linear, int wrap,
                int hand, int window, int cycles, int state, int quiet)
@@ -240,7 +240,7 @@ int ObjectMotion(CObject * I, int action, int first,
     int nFrame = MovieGetLength(I->G);
 
     if(wrap<0) {
-      wrap = SettingGet_b(I->G,NULL, I->Setting, cSetting_movie_loop);
+      wrap = SettingGet_b(I->G,NULL, I->Setting.get(), cSetting_movie_loop);
     }
 
     if(nFrame < 0)
@@ -298,7 +298,7 @@ int ObjectMotion(CObject * I, int action, int first,
             ViewElemSmooth(I->ViewElem + first, I->ViewElem + last, window, wrap);
           }
       }
-      if(SettingGet_b(I->G, NULL, I->Setting, cSetting_movie_auto_interpolate)){
+      if(SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_movie_auto_interpolate)){
         action = 3; /* reinterpolate */
         last = save_last;
       }
@@ -325,7 +325,7 @@ int ObjectMotion(CObject * I, int action, int first,
           if(state>=0) {
             state_tmp = state;
             state_flag = true;
-          } else if(SettingGetIfDefined_i(G, I->Setting, cSetting_state, &state_tmp)) {
+          } else if(SettingGetIfDefined_i(G, I->Setting.get(), cSetting_state, &state_tmp)) {
             state_flag = true;
             state_tmp--;
           }
@@ -545,14 +545,14 @@ int ObjectMotion(CObject * I, int action, int first,
   return 1;
 }
 
-void ObjectAdjustStateRebuildRange(CObject * I, int *start, int *stop)
+void ObjectAdjustStateRebuildRange(pymol::CObject * I, int *start, int *stop)
 {
   /* on entry, start and stop should hold the valid range for the object */
   int defer_builds_mode =
-    SettingGet_i(I->G, NULL, I->Setting, cSetting_defer_builds_mode);
-  int async_builds = SettingGet_b(I->G, NULL, I->Setting, cSetting_async_builds);
-  int max_threads = SettingGet_i(I->G, NULL, I->Setting, cSetting_max_threads);
-  int all_states = SettingGet_i(I->G, NULL, I->Setting, cSetting_all_states);
+    SettingGet_i(I->G, NULL, I->Setting.get(), cSetting_defer_builds_mode);
+  int async_builds = SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_async_builds);
+  int max_threads = SettingGet_i(I->G, NULL, I->Setting.get(), cSetting_max_threads);
+  int all_states = SettingGet_i(I->G, NULL, I->Setting.get(), cSetting_all_states);
   int dummy;
   if (all_states)
     return;
@@ -563,7 +563,7 @@ void ObjectAdjustStateRebuildRange(CObject * I, int *start, int *stop)
   switch (defer_builds_mode) {
   case 1:                      /* defer geometry builds until needed */
   case 2:                      /* defer and destroy continuously for increase memory conservation */
-    if(SettingGetIfDefined_i(I->G, I->Setting, cSetting_state, &dummy)) {  
+    if(SettingGetIfDefined_i(I->G, I->Setting.get(), cSetting_state, &dummy)) {
       /* decoupled...so always build all states.  Otherwise, geometry
       may not be there when we need it... unfortunately, this defeats
       the purpose of defer_builds_mode! */
@@ -719,7 +719,7 @@ void ObjectMakeValidName(PyMOLGlobals * G, char *name, bool quiet)
  * @param state State (0-indexed) or -2/-3 for current state
  * @return NULL if state is out of bounds or empty
  */
-CObjectState* CObject::getObjectState(int state)
+CObjectState* pymol::CObject::getObjectState(int state)
 {
   if (state == -2 /* cSelectorUpdateTableCurrentState */ ||
       state == -3 /* cSelectorUpdateTableEffectiveStates */) {
@@ -736,16 +736,16 @@ CObjectState* CObject::getObjectState(int state)
  * `static_singletons` settings. Will not validate the value of the `state`
  * setting, it could be `<0` or `>=getNFrame()`.
  */
-int CObject::getCurrentState() const
+int pymol::CObject::getCurrentState() const
 {
   if (getNFrame() == 1 &&
-      SettingGet<bool>(G, Setting, nullptr, cSetting_static_singletons))
+      SettingGet<bool>(G, Setting.get(), nullptr, cSetting_static_singletons))
     return 0;
-  return SettingGet<int>(G, Setting, nullptr, cSetting_state) - 1;
+  return SettingGet<int>(G, Setting.get(), nullptr, cSetting_state) - 1;
 }
 
 /**
- * Like CObject::getCurrentState() but will return `-1` if the `all_states`
+ * Like pymol::CObject::getCurrentState() but will return `-1` if the `all_states`
  * setting is set.
  *
  * Note: Clamps negative values at `-1` (all states). The usefulness of this
@@ -753,24 +753,24 @@ int CObject::getCurrentState() const
  * is likely to discard all negative values, including -1.
  *
  * @param ignore_all_states Boolean flag, should be false. You most likely
- * should use CObject::getCurrentState() instead of setting `ignore_all_states`
+ * should use pymol::CObject::getCurrentState() instead of setting `ignore_all_states`
  * to true.
  */
-int ObjectGetCurrentState(CObject * I, int ignore_all_states)
+int ObjectGetCurrentState(const pymol::CObject * I, int ignore_all_states)
 {
-  assert("use CObject::getCurrentState()" && !ignore_all_states);
+  assert("use pymol::CObject::getCurrentState()" && !ignore_all_states);
 
   // the previous implementation (up to PyMOL 1.7.6) ignored
   // object-level state=0 (all states)
 
   if (!ignore_all_states &&
-      SettingGet_b(I->G, I->Setting, NULL, cSetting_all_states))
+      SettingGet_b(I->G, I->Setting.get(), NULL, cSetting_all_states))
     return -1;
 
   return std::max(-1, I->getCurrentState());
 }
 
-PyObject *ObjectAsPyList(CObject * I)
+PyObject *ObjectAsPyList(pymol::CObject * I)
 {
   PyObject *result = NULL;
   result = PyList_New(14);
@@ -782,10 +782,10 @@ PyObject *ObjectAsPyList(CObject * I)
   PyList_SetItem(result, 5, PConvFloatArrayToPyList(I->ExtentMax, 3));
   PyList_SetItem(result, 6, PyInt_FromLong(I->ExtentFlag));
   PyList_SetItem(result, 7, PyInt_FromLong(I->TTTFlag));
-  PyList_SetItem(result, 8, SettingAsPyList(I->Setting));
+  PyList_SetItem(result, 8, SettingAsPyList(I->Setting.get()));
 
   PyList_SetItem(result, 9, PyInt_FromLong(I->Enabled));
-  PyList_SetItem(result, 10, PyInt_FromLong(I->Context));
+  PyList_SetItem(result, 10, PyInt_FromLong(static_cast<long>(I->getRenderContext())));
   PyList_SetItem(result, 11, PConvFloatArrayToPyList(I->TTT, 16));
   if(I->ViewElem) {
     int nFrame = VLAGetSize(I->ViewElem);
@@ -798,7 +798,7 @@ PyObject *ObjectAsPyList(CObject * I)
   return (PConvAutoNone(result));
 }
 
-int ObjectFromPyList(PyMOLGlobals * G, PyObject * list, CObject * I)
+int ObjectFromPyList(PyMOLGlobals * G, PyObject * list, pymol::CObject * I)
 {
   int ok = true;
   int ll = 0;
@@ -834,12 +834,16 @@ int ObjectFromPyList(PyMOLGlobals * G, PyObject * list, CObject * I)
     ok = PConvPyIntToInt(PyList_GetItem(list, 6), &I->ExtentFlag);
   if(ok)
     ok = PConvPyIntToInt(PyList_GetItem(list, 7), &I->TTTFlag);
-  if(ok)
-    I->Setting = SettingNewFromPyList(G, PyList_GetItem(list, 8));
+  if(ok){
+    CPythonVal *val = CPythonVal_PyList_GetItem(G, list, 8);
+    I->Setting.reset(SettingNewFromPyList(G, val));
+    CPythonVal_Free(val);
+  }
   if(ok && (ll > 9))
-    ok = PConvPyIntToInt(PyList_GetItem(list, 9), &I->Enabled);
-  if(ok && (ll > 10))
-    ok = PConvPyIntToInt(PyList_GetItem(list, 10), &I->Context);
+    ok = CPythonVal_PConvPyIntToInt_From_List(G, list, 9, &I->Enabled);
+  if(ok && (ll > 10)) {
+    // I->Context removed.
+  }
   if(ok && (ll > 11))
     ok = PConvPyListToFloatArrayInPlaceAutoZero(PyList_GetItem(list, 11), I->TTT, 16);
   if(ok && (ll > 13)) {
@@ -861,7 +865,7 @@ int ObjectFromPyList(PyMOLGlobals * G, PyObject * list, CObject * I)
   return (ok);
 }
 
-int ObjectCopyHeader(CObject * I, const CObject * src)
+int ObjectCopyHeader(pymol::CObject * I, const pymol::CObject * src)
 {
   int ok = true;
 
@@ -875,9 +879,8 @@ int ObjectCopyHeader(CObject * I, const CObject * src)
 
   I->ExtentFlag = src->ExtentFlag;
   I->TTTFlag = src->TTTFlag;
-  I->Setting = NULL;            /* to do */
+  I->Setting = src->Setting;
   I->Enabled = src->Enabled;
-  I->Context = src->Context;
   {
     int a;
     for(a = 0; a < 16; a++)
@@ -890,7 +893,7 @@ int ObjectCopyHeader(CObject * I, const CObject * src)
 
 
 /*========================================================================*/
-void ObjectCombineTTT(CObject * I, const float *ttt, int reverse_order, int store)
+void ObjectCombineTTT(pymol::CObject * I, const float *ttt, int reverse_order, int store)
 {
   if(I->type == cObjectGroup) {
     ExecutiveGroupCombineTTT(I->G, I, ttt, reverse_order,store);
@@ -908,7 +911,7 @@ void ObjectCombineTTT(CObject * I, const float *ttt, int reverse_order, int stor
       combineTTT44f44f(ttt, cpy, I->TTT);
     }
     if(store<0) 
-      store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
+      store = SettingGet_i(I->G, I->Setting.get(), NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
         I->ViewElem = pymol::vla<CViewElem>(0);
@@ -924,7 +927,7 @@ void ObjectCombineTTT(CObject * I, const float *ttt, int reverse_order, int stor
   }
 }
 /*========================================================================*/
-void ObjectTranslateTTT(CObject * I, const float *v, int store)
+void ObjectTranslateTTT(pymol::CObject * I, const float *v, int store)
 {
   if(I->type == cObjectGroup) {
     ExecutiveGroupTranslateTTT(I->G, I, v, store);
@@ -939,7 +942,7 @@ void ObjectTranslateTTT(CObject * I, const float *v, int store)
       I->TTT[11] += v[2];
     }
     if(store<0) 
-      store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
+      store = SettingGet_i(I->G, I->Setting.get(), NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
         I->ViewElem = pymol::vla<CViewElem>(0);
@@ -957,7 +960,7 @@ void ObjectTranslateTTT(CObject * I, const float *v, int store)
 
 
 /*========================================================================*/
-void ObjectSetTTT(CObject * I, const float *ttt, int state, int store)
+void ObjectSetTTT(pymol::CObject * I, const float *ttt, int state, int store)
 {
   if(state < 0) {
     if(ttt) {
@@ -968,7 +971,7 @@ void ObjectSetTTT(CObject * I, const float *ttt, int state, int store)
       return;
     }
     if(store<0) 
-      store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
+      store = SettingGet_i(I->G, I->Setting.get(), NULL, cSetting_movie_auto_store);
     if(store && MovieDefined(I->G)) {
       if(!I->ViewElem)  
         I->ViewElem = pymol::vla<CViewElem>(0);
@@ -987,7 +990,7 @@ void ObjectSetTTT(CObject * I, const float *ttt, int state, int store)
 }
 
 /*========================================================================*/
-int ObjectGetTTT(CObject * I, const float **ttt, int state)
+int ObjectGetTTT(pymol::CObject * I, const float **ttt, int state)
 {
   if(state < 0) {
     if(I->TTTFlag) {
@@ -1004,12 +1007,12 @@ int ObjectGetTTT(CObject * I, const float **ttt, int state)
 
 
 /*========================================================================*/
-void ObjectResetTTT(CObject * I,int store)
+void ObjectResetTTT(pymol::CObject * I,int store)
 {
   
   I->TTTFlag = false;
   if(store<0) 
-    store = SettingGet_i(I->G, I->Setting, NULL, cSetting_movie_auto_store);
+    store = SettingGet_i(I->G, I->Setting.get(), NULL, cSetting_movie_auto_store);
   if(store && MovieDefined(I->G)) {
     if(!I->ViewElem)  
       I->ViewElem = pymol::vla<CViewElem>(0);
@@ -1031,12 +1034,12 @@ void ObjectResetTTT(CObject * I,int store)
  * Get the combined transformation of TTT and state matrix. State matrix is
  * only included if `history=true` or `matrix_mode > 0`.
  *
- * @param state See CObject::getObjectState
+ * @param state See pymol::CObject::getObjectState
  * @param history Boolean flag
  * @param[out] matrix Homogeneous 4x4 matrix
  * @return True if `matrix` was populated
  */
-int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
+int ObjectGetTotalMatrix(pymol::CObject * I, int state, int history, double *matrix)
 {
   int result = false;
   if(I->TTTFlag) {
@@ -1046,7 +1049,7 @@ int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
 
   if (!history) {
     history =
-        SettingGet<int>(I->G, I->Setting, nullptr, cSetting_matrix_mode) > 0;
+        SettingGet<int>(I->G, I->Setting.get(), nullptr, cSetting_matrix_mode) > 0;
   }
 
   if (history) {
@@ -1070,7 +1073,7 @@ int ObjectGetTotalMatrix(CObject * I, int state, int history, double *matrix)
 
 
 /*========================================================================*/
-void ObjectPrepareContext(CObject * I, RenderInfo * info)
+void ObjectPrepareContext(pymol::CObject * I, RenderInfo * info)
 {
   CRay * ray = info ? info->ray : NULL;
 
@@ -1088,12 +1091,12 @@ void ObjectPrepareContext(CObject * I, RenderInfo * info)
           I->TTTFlag = true;
         }
         if(I->ViewElem[frame].state_flag) {
-          SettingCheckHandle(I->G,&I->Setting);
+          SettingCheckHandle(I->G, I->Setting);
           if(I->Setting) {
             /* note: this assumes that the state has already been
                calculated and can thus be displayed.  How can we
                guarantee this to be true? */
-            SettingSet_i(I->Setting,cSetting_state,I->ViewElem[frame].state + 1);
+            SettingSet_i(I->Setting.get(),cSetting_state,I->ViewElem[frame].state + 1);
           }
         }
       }
@@ -1142,7 +1145,7 @@ void ObjectPrepareContext(CObject * I, RenderInfo * info)
 
 
 /*========================================================================*/
-void ObjectSetTTTOrigin(CObject * I, float *origin)
+void ObjectSetTTTOrigin(pymol::CObject * I, float *origin)
 {
   float homo[16];
   float *dst;
@@ -1174,21 +1177,20 @@ void ObjectSetTTTOrigin(CObject * I, float *origin)
 
 
 /*========================================================================*/
-CSetting **CObject::getSettingHandle(int state)
+pymol::copyable_ptr<CSetting>* pymol::CObject::getSettingHandle(int state)
 {
   return &Setting;
 }
 
 
 /*========================================================================*/
-void CObject::describeElement(int index, char* buffer) const
+std::string pymol::CObject::describeElement(int index) const
 {
-  buffer[0] = 0;
+  return {};
 }
 
-
 /*========================================================================*/
-void ObjectToggleRepVis(CObject * I, int rep)
+void ObjectToggleRepVis(pymol::CObject * I, int rep)
 {
   if((rep >= 0) && (rep < cRepCnt))
     I->visRep ^= (1 << rep);
@@ -1196,7 +1198,7 @@ void ObjectToggleRepVis(CObject * I, int rep)
 
 
 /*========================================================================*/
-void ObjectSetRepVisMask(CObject * I, int repmask, int value)
+void ObjectSetRepVisMask(pymol::CObject * I, int repmask, int value)
 {
   switch (value) {
     case cVis_HIDE:
@@ -1218,7 +1220,7 @@ void ObjectSetRepVisMask(CObject * I, int repmask, int value)
 
 
 /*========================================================================*/
-void ObjectSetName(CObject * I, const char *name)
+void ObjectSetName(pymol::CObject * I, const char *name)
 {
   UtilNCopy(I->Name, name, WordLength);
   if(SettingGetGlobal_b(I->G, cSetting_validate_object_names))
@@ -1227,15 +1229,14 @@ void ObjectSetName(CObject * I, const char *name)
 
 
 /*========================================================================*/
-CObject::~CObject()
+pymol::CObject::~CObject()
 {
   SceneObjectDel(this->G, this, false);
-  SettingFreeP(this->Setting);
 }
 
 
 /*========================================================================*/
-void ObjectUseColor(CObject * I)
+void ObjectUseColor(pymol::CObject * I)
 {
   PyMOLGlobals *G = I->G;
   if(G->HaveGUI && G->ValidContext) {
@@ -1243,7 +1244,7 @@ void ObjectUseColor(CObject * I)
   }
 }
 
-void ObjectUseColorCGO(CGO *cgo, CObject * I)
+void ObjectUseColorCGO(CGO *cgo, pymol::CObject * I)
 {
   PyMOLGlobals *G = I->G;
   if(G->HaveGUI && G->ValidContext) {
@@ -1255,7 +1256,7 @@ void ObjectUseColorCGO(CGO *cgo, CObject * I)
 /**
  * Render a unit box (dummy representation)
  */
-void CObject::render(RenderInfo * info)
+void pymol::CObject::render(RenderInfo * info)
 {
   if(G->HaveGUI && G->ValidContext) {
 #ifdef PURE_OPENGL_ES_2
@@ -1290,7 +1291,7 @@ void CObject::render(RenderInfo * info)
 
 
 /*========================================================================*/
-CObject::CObject(PyMOLGlobals * G) : G(G)
+pymol::CObject::CObject(PyMOLGlobals * G) : G(G)
 {
   OrthoRemoveSplash(G);         /* HMM... this seems like an inappropriate sideeffect */
   visRep = cRepBitmask & ~(cRepCellBit | cRepExtentBit);
@@ -1307,7 +1308,7 @@ void ObjectStatePurge(CObjectState * I)
 {
 }
 
-int ObjectStateSetMatrix(CObjectState * I, double *matrix)
+int ObjectStateSetMatrix(CObjectState * I, const double *matrix)
 {
   int ok = true;
   if(matrix) {
@@ -1320,7 +1321,7 @@ int ObjectStateSetMatrix(CObjectState * I, double *matrix)
   return ok;
 }
 
-void ObjectStateRightCombineMatrixR44d(CObjectState * I, double *matrix)
+void ObjectStateRightCombineMatrixR44d(CObjectState * I, const double *matrix)
 {
   if(matrix) {
     if(I->Matrix.empty()) {
@@ -1333,7 +1334,7 @@ void ObjectStateRightCombineMatrixR44d(CObjectState * I, double *matrix)
   I->InvMatrix.clear();
 }
 
-void ObjectStateLeftCombineMatrixR44d(CObjectState * I, double *matrix)
+void ObjectStateLeftCombineMatrixR44d(CObjectState * I, const double *matrix)
 {
   if(matrix) {
     if(I->Matrix.empty()) {
@@ -1346,7 +1347,7 @@ void ObjectStateLeftCombineMatrixR44d(CObjectState * I, double *matrix)
   I->InvMatrix.clear();
 }
 
-void ObjectStateCombineMatrixTTT(CObjectState * I, float *matrix)
+void ObjectStateCombineMatrixTTT(CObjectState * I, const float *matrix)
 {
 
   if(matrix) {
@@ -1370,19 +1371,30 @@ double *ObjectStateGetMatrix(CObjectState * I)
   return nullptr;
 }
 
-/*
+const double* ObjectStateGetMatrix(const CObjectState* I)
+{
+  return I->Matrix.empty() ? nullptr : I->Matrix.data();
+}
+
+/**
  * Get the Matrix inverse
  */
-double *ObjectStateGetInvMatrix(CObjectState * I)
+const double *ObjectStateGetInvMatrix(const CObjectState * self)
 {
-  if(!I->Matrix.empty() && I->InvMatrix.empty()) {
+  if (self->Matrix.empty()) {
+    return nullptr;
+  }
+
+  if (self->InvMatrix.empty()) {
+    auto I = const_cast<CObjectState*>(self);
     I->InvMatrix = std::vector<double>(16);
     xx_matrix_invert(I->InvMatrix.data(), I->Matrix.data(), 4);
   }
-  return I->InvMatrix.data();
+
+  return self->InvMatrix.data();
 }
 
-void ObjectStateTransformMatrix(CObjectState * I, double *matrix)
+void ObjectStateTransformMatrix(CObjectState * I, const double *matrix)
 {
   if(I->Matrix.empty()) {
     I->Matrix = std::vector<double>(16);
