@@ -22,11 +22,15 @@ Z* -------------------------------------------------------------------
 #include"PyMOLObject.h"
 #include"Ortho.h"
 #include"View.h"
+#include"Result.h"
+#include"SceneView.h"
 #include"SceneDef.h"
 #include"SceneRender.h"
 #include"ShaderMgr.h"
 #include"pymol/zstring_view.h"
 
+#include <vector>
+
 #define SDOF_NORMAL_MODE 0
 #define SDOF_CLIP_MODE 1
 #define SDOF_DRAG_MODE 2
@@ -39,8 +43,9 @@ Z* -------------------------------------------------------------------
 #define cSceneImage_Draw 1
 #define cSceneImage_Ray 2
 
-#define cSceneViewSize 25
-typedef float SceneViewType[cSceneViewSize];
+#define SceneScrollBarMargin DIP2PIXEL(1)
+#define SceneScrollBarWidth DIP2PIXEL(13)
+#define SceneGetExactScreenVertexScale SceneGetScreenVertexScale
 
 enum cSceneClip {
   cSceneClip_invalid = -1,
@@ -51,8 +56,6 @@ enum cSceneClip {
   cSceneClip_atoms = 4,
 };
 
-cSceneClip SceneClipGetEnum(pymol::zstring_view mode);
-
 float SceneGetDynamicLineWidth(RenderInfo * info, float line_width);
 void SceneInvalidateStencil(PyMOLGlobals * G);
 int SceneHasImage(PyMOLGlobals * G);
@@ -64,7 +67,6 @@ void SceneSetFogUniforms(PyMOLGlobals * G, CShaderPrg *);
 void SceneSetFrame(PyMOLGlobals * G, int mode, int frame);
 int SceneGetFrame(PyMOLGlobals * G);
 int SceneGetState(PyMOLGlobals * G);
-int SceneGetButtonMargin(PyMOLGlobals * G);
 
 void SceneDirty(PyMOLGlobals * G);      /* scene dirty, but leave the overlay if one exists */
 void SceneInvalidate(PyMOLGlobals * G); /* scene dirty and remove the overlay */
@@ -98,8 +100,8 @@ int SceneMakeMovieImage(PyMOLGlobals * G,
     int width=0, int height=0);
 int SceneValidateImageMode(PyMOLGlobals * G, int mode, bool defaultdraw);
 
-bool ScenePNG(PyMOLGlobals * G, const char *png, float dpi, int quiet,
-             int prior_only, int format);
+bool ScenePNG(PyMOLGlobals* G, const char* png, float dpi, int quiet,
+    int prior_only, int format, std::vector<unsigned char>* outbuf = nullptr);
 int SceneCopyExternal(PyMOLGlobals * G, int width, int height, int rowbytes,
                       unsigned char *dest, int mode);
 
@@ -109,13 +111,17 @@ void SceneRestartFrameTimer(PyMOLGlobals * G);
 
 void SceneGetEyeNormal(PyMOLGlobals * G, float *v1, float *normal);
 
-void SceneRotate(PyMOLGlobals * G, float angle, float x, float y, float z);
+void SceneRotateAxis(PyMOLGlobals * G, float angle, char axis);
+void SceneRotate(
+    PyMOLGlobals* G, float angle, float x, float y, float z, bool dirty = true);
 void SceneTranslate(PyMOLGlobals * G, float x, float y, float z);
 
 void SceneTranslateScaled(PyMOLGlobals * G, float x, float y, float z, int sdof_mode);
 void SceneRotateScaled(PyMOLGlobals * G, float rx, float ry, float rz, int sdof_mode);
 
 void SceneClip(PyMOLGlobals * G, int plane, float movement, const char *sele, int state);
+pymol::Result<> SceneClipFromMode(PyMOLGlobals* G, pymol::zstring_view mode, float movement,
+    pymol::zstring_view sele, int state);
 std::pair<int, int> SceneGetImageSize(PyMOLGlobals * G);
 float SceneGetGridAspectRatio(PyMOLGlobals * G);
 void SceneScale(PyMOLGlobals * G, float scale);
@@ -126,9 +132,17 @@ void SceneResetNormalToViewVector(PyMOLGlobals * G, short use_shader);
 void SceneResetNormalUseShaderAttribute(PyMOLGlobals * G, int lines, short use_shader, int attr);
 void SceneGetResetNormal(PyMOLGlobals * G, float *normal, int lines);
 
-int SceneObjectAdd(PyMOLGlobals * G, CObject * obj);
-int SceneObjectDel(PyMOLGlobals * G, CObject * obj, int allow_purge);
-int SceneObjectIsActive(PyMOLGlobals * G, CObject * obj);
+int SceneObjectAdd(PyMOLGlobals * G, pymol::CObject* obj);
+int SceneObjectDel(PyMOLGlobals * G, pymol::CObject* obj, int allow_purge);
+
+/**
+ * @brief removes object from Scene
+ * @param obj Object to be removed
+ * @return if Object removed--always true if obj==nullptr
+ */
+
+bool SceneObjectRemove(PyMOLGlobals* G, pymol::CObject* obj);
+int SceneObjectIsActive(PyMOLGlobals* G, pymol::CObject* obj);
 void SceneOriginSet(PyMOLGlobals * G, const float *origin, int preserve);
 void SceneOriginGet(PyMOLGlobals * G, float *origin);
 void SceneWindowSphere(PyMOLGlobals * G, const float *location, float radius);
@@ -226,7 +240,8 @@ int SceneIncrementTextureRefreshes(PyMOLGlobals * G);
 
 void SceneResetTextureRefreshes(PyMOLGlobals * G);
 
-void SceneGetScaledAxes(PyMOLGlobals * G, CObject *obj, float *xn, float *yn);
+void SceneGetScaledAxes(
+    PyMOLGlobals* G, pymol::CObject* obj, float* xn, float* yn);
 void SceneGetScaledAxesAtPoint(PyMOLGlobals * G, float *pt, float *xn, float *yn);
 
 int SceneGetCopyType(PyMOLGlobals * G);
@@ -268,5 +283,14 @@ void SceneInvalidatePicking(PyMOLGlobals * G);
 
 pymol::Image* SceneImagePrepare(PyMOLGlobals * G, bool prior_only);
 
+void SceneDoRoving(PyMOLGlobals * G, float old_front,
+                   float old_back, float old_origin,
+                   int adjust_flag, int zoom_flag);
+void UpdateFrontBackSafe(CScene *I);
+int stereo_via_adjacent_array(int stereo_mode);
+int SceneDeferredClick(DeferredMouse * dm);
+int SceneDeferredDrag(DeferredMouse * dm);
+int SceneDeferredRelease(DeferredMouse * dm);
+
 #endif
 
