@@ -256,7 +256,6 @@ bool SceneRay(PyMOLGlobals * G,
         int *slot_vla = I->SlotVLA;
         int state = SceneGetState(G);
         RenderInfo info;
-        UtilZeroMem(&info, sizeof(RenderInfo));
         info.ray = ray;
         info.ortho = ortho;
         info.vertex_scale = SceneGetScreenVertexScale(G, NULL);
@@ -276,11 +275,11 @@ bool SceneRay(PyMOLGlobals * G,
             if(SceneGetDrawFlag(&I->grid, slot_vla, obj->grid_slot)) {
               float color[3];
               ColorGetEncoded(G, obj->Color, color);
-              RaySetContext(ray, obj->Context);
+              RaySetContext(ray, obj->getRenderContext());
               ray->color3fv(color);
 
               auto icx = SettingGetWD<int>(
-                  obj->Setting, cSetting_ray_interior_color, cColorDefault);
+                  obj->Setting.get(), cSetting_ray_interior_color, cColorDefault);
 
               if (icx == cColorDefault) {
                 ray->interiorColor3fv(color, true);
