@@ -557,6 +557,7 @@ void SceneRender(PyMOLGlobals * G, Picking * pick, int x, int y,
     " SceneRender: leaving...\n" ENDFD;
 }
 
+#ifndef _PYMOL_NO_AA_SHADERS
 static
 void AppendCopyWithChangedShader(PyMOLGlobals * G, CGO *destCGO, CGO *srcCGO, int frommode, int tomode){
   CGO *cgo = CGONew(G);
@@ -565,6 +566,7 @@ void AppendCopyWithChangedShader(PyMOLGlobals * G, CGO *destCGO, CGO *srcCGO, in
   CGOAppendNoStop(destCGO, cgo);
   CGOFreeWithoutVBOs(cgo);
 }
+#endif
 
 /* SceneRenderAA: renders Anti-aliasing from the I->offscreen_texture texture,
                   depending on the antialias_shader setting, FXAA (1 stage)
@@ -627,7 +629,7 @@ void SceneRenderAllObject(PyMOLGlobals * G,
     RenderInfo *info,
     float *normal,
     int state,
-    CObject *obj,
+    pymol::CObject *obj,
     GridInfo * grid,
     int *slot_vla,
     int fat)
@@ -635,17 +637,14 @@ void SceneRenderAllObject(PyMOLGlobals * G,
   if (!SceneGetDrawFlag(grid, slot_vla, obj->grid_slot))
     return;
 
-  if (obj->Context == 2)
-    return;
-
   auto use_shader = info->use_shaders;
 
 #ifndef _WEBGL
   glLineWidth(fat ? 3.0 : 1.0);
 #endif
 
-    switch (obj->Context) {
-    case 1:              /* unit context */
+    switch (obj->getRenderContext()) {
+    case pymol::RenderContext::UnitWindow:
       // e.g. Gadgets/Ramps
       {
         float projSave[16];
@@ -697,7 +696,7 @@ void SceneRenderAllObject(PyMOLGlobals * G,
 #endif
       }
       break;
-    case 0:              /* context/grid 0 is all slots */
+    case pymol::RenderContext::Camera:              /* context/grid 0 is all slots */
     default:
       ScenePushModelViewMatrix(G);
 
@@ -735,15 +734,11 @@ void SceneRenderAllObject(PyMOLGlobals * G,
  * width_scale: specifies width_scale and sampling
  * grid: grid information
  * dynamic_pass: for specific stereo modes dynamic and clone_dynamic
- * which: 0 - all objects
- *        1 - only gadgets
- *        2 - only non-gadgets
- *        3 - gadgets last
+ * which: enum specifying which objects (AllObjects, OnlyGadgets, OnlyNonGadgets, GadgetsLast)
  */
-void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
-                    float *normal, PickColorManager* pickmgr,
-                    int pass, int fat, float width_scale,
-                    GridInfo * grid, int dynamic_pass, short which_objects)
+void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context, float *normal, PickColorManager* pickmgr,
+                    RenderPass pass, int fat, float width_scale,
+                    GridInfo * grid, int dynamic_pass, SceneRenderWhich which_objects)
 {
   CScene *I = G->Scene;
   int state = SceneGetState(G);
@@ -752,7 +747,6 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
   if (!OrthoEvalCheck(G))
     return;
 #endif
-  UtilZeroMem(&info, sizeof(RenderInfo));
   info.pick = pickmgr;
   info.pass = pass;
   info.vertex_scale = I->VertexScale;
@@ -795,7 +789,7 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
   info.back = I->m_view.m_clipSafe.m_back;
   SceneGetViewNormal(G, info.view_normal);
 
-  if(info.alpha_cgo && (pass == 1)) {
+  if(info.alpha_cgo && (pass == RenderPass::Opaque)) {
     CGOReset(info.alpha_cgo);
     CGOSetZVector(info.alpha_cgo, I->ModMatrix[2], I->ModMatrix[6], I->ModMatrix[10]);
   }
@@ -817,7 +811,7 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
   {
     int *slot_vla = I->SlotVLA;
     switch (which_objects) {
-    case 0:
+    case SceneRenderWhich::AllObjects:
       for (auto obj : I->Obj) {
         /* EXPERIMENTAL RAY-VOLUME COMPOSITION CODE */
         if (!rayVolume || obj->type == cObjectVolume) {
@@ -826,13 +820,13 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
         }
       }
       break;
-    case 1:
+    case SceneRenderWhich::OnlyGadgets:
       for (auto obj : I->GadgetObjs) {
           SceneRenderAllObject(
               G, I, context, &info, normal, state, obj, grid, slot_vla, fat);
       }
       break;
-    case 2:
+    case SceneRenderWhich::OnlyNonGadgets:
       for (auto obj : I->NonGadgetObjs) {
         // ObjectGroup used to have fRender = NULL
         if (obj->type != cObjectGroup) {
@@ -841,7 +835,7 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
         }
       }
       break;
-    case 3:
+    case SceneRenderWhich::GadgetsLast:
       // Gadgets Last
       for (auto obj : I->NonGadgetObjs) {
         /* EXPERIMENTAL RAY-VOLUME COMPOSITION CODE */
@@ -863,7 +857,7 @@ void SceneRenderAll(PyMOLGlobals * G, SceneUnitContext * context,
   if(info.alpha_cgo) {
     CGOStop(info.alpha_cgo);
     /* this only works when all objects are rendered in the same frame of reference */
-    if(pass == -1) {
+    if(pass == RenderPass::Transparent) {
       CGORenderGLAlpha(info.alpha_cgo, &info, 0);
     }
   }
@@ -884,7 +878,7 @@ static
 void DoRendering(PyMOLGlobals * G, CScene *I, short offscreen, GridInfo *grid, int times, 
                  int curState, float *normal, SceneUnitContext *context, 
                  float width_scale, short onlySelections, short excludeSelections){
-  int pass;
+  const RenderPass passes[] = { RenderPass::Opaque, RenderPass::Antialias, RenderPass::Transparent };
   bool use_shaders = (bool)SettingGetGlobal_b(G, cSetting_use_shaders);
   bool t_mode_3_os = use_shaders && SettingGetGlobal_i(G, cSetting_transparency_mode) == 3;
   bool t_mode_3 = !onlySelections && t_mode_3_os;
@@ -912,9 +906,12 @@ void DoRendering(PyMOLGlobals * G, CScene *I, short offscreen, GridInfo *grid, i
     bool cont = true;
     bool t_first_pass = true;
     G->ShaderMgr->stereo_draw_buffer_pass = 0;
-    for(pass = 1; cont && pass > -2; pass--) {        /* render opaque, then antialiased, then transparent... */
+    for (auto pass : passes) {        /* render opaque, then antialiased, then transparent... */
+      if (!cont) {
+        break;
+      }
 #if !defined(PURE_OPENGL_ES_2) || defined(_WEBGL)
-      if (t_mode_3 && pass == -1){
+      if (t_mode_3 && pass == RenderPass::Transparent){
         G->ShaderMgr->Disable_Current_Shader();
         int drawbuf = 1;
         if (TM3_IS_ONEBUF){
@@ -966,29 +963,29 @@ void DoRendering(PyMOLGlobals * G, CScene *I, short offscreen, GridInfo *grid, i
         if (!onlySelections){
 #if !defined(PURE_OPENGL_ES_2) || defined(_WEBGL)
           if (t_mode_3){
-            if (pass == 1){
+            if (pass == RenderPass::Opaque){
               EditorRender(G, curState);
             }
             // transparency-mode == 3 render all objects for this pass
-            SceneRenderAll(G, context, normal, NULL, pass, false, width_scale, grid, times, 2 /* non-gadgets */); // opaque
+            SceneRenderAll(G, context, normal, NULL, pass, false, width_scale, grid, times, SceneRenderWhich::OnlyNonGadgets); // opaque
           } else {
 #else
             {
 #endif
               // transparency-mode != 3 render all objects for each pass
-              for(pass = 1; pass > -2; pass--) {        /* render opaque, then antialiased, then transparent... */
-                SceneRenderAll(G, context, normal, NULL, pass, false, width_scale, grid,
-                               times, 3 /* gadgets last */);
+              for (const auto pass2 : passes) { /* render opaque, then antialiased, then transparent... */
+                SceneRenderAll(G, context, normal, NULL, pass2, false, width_scale, grid,
+                               times, SceneRenderWhich::GadgetsLast);
               }
               cont = false;
             }
-          } else if (t_mode_3_os && pass > 0){  /* opaque pass*/
+          } else if (t_mode_3_os && pass == RenderPass::Opaque){
             // onlySelections and t_mode_3, render only gadgets
             glEnable(GL_BLEND);  // need to blend for the text onto the gadget background
             glBlendFunc_default();
 
-            SceneRenderAll(G, context, normal, NULL, -1 /* gadgets render in transp pass */, false, width_scale, grid,
-                           times, 1 /* only gadgets */);
+            SceneRenderAll(G, context, normal, NULL, RenderPass::Transparent /* gadgets render in transp pass */, false, width_scale, grid,
+                           times, SceneRenderWhich::OnlyGadgets);
             glDisable(GL_BLEND);
           }
 #ifdef PURE_OPENGL_ES_2
@@ -1017,14 +1014,14 @@ void DoRendering(PyMOLGlobals * G, CScene *I, short offscreen, GridInfo *grid, i
 #endif
         } // end slot loop
     if (TM3_IS_ONEBUF){
-      if (t_mode_3 && pass == -1 && t_first_pass){
-        pass++;
+      if (t_mode_3 && pass == RenderPass::Transparent && t_first_pass){
+        pass = RenderPass::Antialias;
         t_first_pass = false;
         continue;
       }
     }
 #if !defined(PURE_OPENGL_ES_2) || defined(_WEBGL)
-      if (t_mode_3 && pass == -1){
+      if (t_mode_3 && pass == RenderPass::Transparent){
         glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, currentFrameBuffer);
         glBindTexture(GL_TEXTURE_2D, 0);
         if(grid->active)
@@ -1050,8 +1047,8 @@ void DoRendering(PyMOLGlobals * G, CScene *I, short offscreen, GridInfo *grid, i
 
         if ((currentFrameBuffer == G->ShaderMgr->default_framebuffer_id) && t_mode_3){
           // onlySelections and t_mode_3, render only gadgets
-          SceneRenderAll(G, context, normal, NULL, -1 /* gadgets render in transp pass */, false, width_scale, grid,
-                         times, 1 /* only gadgets */);
+          SceneRenderAll(G, context, normal, NULL, RenderPass::Transparent /* gadgets render in transp pass */, false, width_scale, grid,
+                         times, SceneRenderWhich::OnlyGadgets);
         }
 
         glDisable(GL_BLEND);
@@ -1658,29 +1655,15 @@ void SceneDrawStencilInBuffer(PyMOLGlobals * G, CScene *I, int stereo_mode){
 #endif
 }
 
-CGO *GenerateUnitScreenCGO(PyMOLGlobals * G){
-  int ok = true;
-  CGO *cgo2 = NULL, *ret = NULL;
-  CGO *cgo = CGONew(G);
-  ok &= CGOBegin(cgo, GL_TRIANGLE_STRIP);
-  if (ok)
-    ok &= CGOVertex(cgo, -1.f, -1.f, 0.98f);
-  if (ok)
-    ok &= CGOVertex(cgo, 1.f, -1.f, 0.98f);
-  if (ok)
-    ok &= CGOVertex(cgo, -1.f, 1.f, 0.98f);
-  if (ok)
-    ok &= CGOVertex(cgo, 1.f, 1.f, 0.98f);
-  if (ok)
-    ok &= CGOEnd(cgo);
-  if (ok)
-    ok &= CGOStop(cgo);
-  if (ok)
-    cgo2 = CGOCombineBeginEnd(cgo, 0);
-  CHECKOK(ok, cgo2);
-  CGOFree(cgo);
-  if (ok)
-    ret = CGOOptimizeToVBONotIndexed(cgo2, 0);
-  CGOFree(cgo2);
-  return ret;
+CGO* GenerateUnitScreenCGO(PyMOLGlobals* G)
+{
+  CGO cgo(G);
+  CGOBegin(&cgo, GL_TRIANGLE_STRIP);
+  CGOVertex(&cgo, -1.f, -1.f, 0.98f);
+  CGOVertex(&cgo, 1.f, -1.f, 0.98f);
+  CGOVertex(&cgo, -1.f, 1.f, 0.98f);
+  CGOVertex(&cgo, 1.f, 1.f, 0.98f);
+  CGOEnd(&cgo);
+  assert(cgo.has_begin_end);
+  return CGOOptimizeToVBONotIndexed(&cgo, 0);
 }
