@@ -22,7 +22,7 @@ Z* -------------------------------------------------------------------
 #include"os_std.h"
 
 #include"Base.h"
-#include"OOMac.h"
+#include"Err.h"
 #include"MemoryDebug.h"
 #include"Ortho.h"
 #include"Setting.h"
@@ -48,15 +48,15 @@ Z* -------------------------------------------------------------------
 #include"OpenVRMode.h"
 #endif
 
-/*
+/**
  * Setting level info table
  *
  * Levels are not hierarchical at the atom/bond level, that's why a simple
  * sorted enumeration is not sufficient.
  *
- * global < object < object-state
- *                   object-state < atom < atom-state
- *                   object-state < bond < bond-state
+ *     global < object < object-state
+ *                       object-state < atom < atom-state
+ *                       object-state < bond < bond-state
  */
 const SettingLevelInfoType SettingLevelInfo[] = {
   {"unused"        , 0x00}, // 0b00000000
@@ -74,10 +74,10 @@ const SettingLevelInfoType SettingLevelInfo[] = {
 #define SETTINGINFO_IMPLEMENTATION
 #include "SettingInfo.h"
 
-template <> const char * SettingGet<const char *>(int index, const CSetting * I);
+template <> const char * _SettingGet<const char *>(int index, const CSetting * I);
 
 // get level name for setting index (for feedback)
-const char * SettingLevelGetName(PyMOLGlobals * G, int index) {
+const char * SettingLevelGetName(unsigned index) {
   return SettingLevelInfo[SettingInfo[index].level].name;
 }
 
@@ -94,34 +94,38 @@ bool SettingLevelCheck(PyMOLGlobals * G, int index, unsigned char level) {
 
 /* ================================================================== */
 
-static CSetting *SettingCopyAll(PyMOLGlobals * G, const CSetting * src, CSetting * dst)
+static void SettingRecCopy(
+    int const index, SettingRec const& src, SettingRec& dst)
 {
-  if(!dst) {
-    dst = pymol::calloc<CSetting>(1);
-  } else {
-    SettingPurge(dst);
+  switch (SettingInfo[index].type) {
+  case cSetting_string:
+    dst.set_s(src.str_ ? src.str_->c_str() : nullptr);
+    break;
+  case cSetting_float3:
+    dst.set_3f(src.float3_);
+    break;
+  default:
+    dst.set_i(src.int_);
+    break;
+  }
+  dst.defined = src.defined;
+}
+
+CSetting *SettingCopyAll(PyMOLGlobals * G, const CSetting * src, CSetting * dst)
+{
+  if (!src) {
+    delete dst;
+    return nullptr;
   }
 
-  SettingInit(G, dst);
-
-  if(dst && src) {
-
-    /* simply overwriting existing data (if any) ... in the future we
-       may need to release references etc. before doing this */
-
-    unsigned int size = VLAGetSize(src->info);
-    VLACheck(dst->info, SettingRec, size - 1);
-    UtilCopyMem(dst->info, src->info, sizeof(SettingRec) * size);
-    dst->size = src->size;
-
-    // need to properly copy strings
-    for (int index = 0; index < cSetting_INIT; ++index) {
-      if (SettingInfo[index].type == cSetting_string
-          && src->info[index].str_) {
-        dst->info[index].str_ = new std::string(*src->info[index].str_);
-      }
-    }
+  if (!dst) {
+    dst = SettingNew(G);
   }
+
+  for (int index = 0; index < cSetting_INIT; ++index) {
+    SettingRecCopy(index, src->info[index], dst->info[index]);
+  }
+
   return dst;
 }
 
@@ -132,11 +136,7 @@ void SettingStoreDefault(PyMOLGlobals * G)
 
 void SettingPurgeDefault(PyMOLGlobals * G)
 {
-  if(G->Default) {
-    SettingPurge(G->Default);
-    FreeP(G->Default);
-    G->Default = NULL;
-  }
+  DeleteP(G->Default);
 }
 
 void SettingUniqueDetachChain(PyMOLGlobals * G, int unique_id)
@@ -202,7 +202,7 @@ int SettingUniqueCheck(PyMOLGlobals * G, int unique_id, int setting_id)
   return SettingFindSettingUniqueEntry(G, unique_id, setting_id) != NULL;
 }
 
-/*
+/**
  * Return true for convertible types and set int-compatible types to int
  */
 inline bool type_upcast(int &type) {
@@ -248,11 +248,15 @@ bool SettingUniqueGetTypedValuePtr(PyMOLGlobals * G, int unique_id, int setting_
   return true;
 }
 
-/*
+/**
  * Warning: Returns colors as (fff) tuple instead of color index
+ *
+ * @pre GIL
  */
 PyObject *SettingUniqueGetPyObject(PyMOLGlobals * G, int unique_id, int index)
 {
+  assert(PyGILState_Check());
+
   int type = SettingGetType(G, index);
 
   union {
@@ -328,7 +332,7 @@ static void SettingUniqueEntry_Set(SettingUniqueEntry *entry, int value_type, co
   }
 }
 
-/*
+/**
  * Return false if setting was not set (nothing changed)
  */
 bool SettingUniqueUnset(PyMOLGlobals * G, int unique_id, int setting_id)
@@ -435,8 +439,13 @@ int SettingUniqueSetTypedValue(PyMOLGlobals * G, int unique_id, int setting_id,
 }
 
 #ifndef _PYMOL_NOPY
+/**
+ * @pre GIL
+ */
 bool SettingUniqueSetPyObject(PyMOLGlobals * G, int unique_id, int index, PyObject *value)
 {
+  assert(PyGILState_Check());
+
   if (!value)
     return SettingUniqueUnset(G, unique_id, index);
 
@@ -614,7 +623,7 @@ static void SettingUniqueFree(PyMOLGlobals * G)
   FreeP(I);
 }
 
-/*
+/**
  * For unique_id remapping during partial session loading
  */
 int SettingUniqueConvertOldSessionID(PyMOLGlobals * G, int old_unique_id)
@@ -635,7 +644,7 @@ int SettingUniqueConvertOldSessionID(PyMOLGlobals * G, int old_unique_id)
   return unique_id;
 }
 
-/*
+/**
  * Return true if the given setting index should not be stored to PSE.
  *
  * Blacklisted are unused and system-dependent settings.
@@ -709,8 +718,13 @@ static bool is_session_blacklisted(int index) {
   return false;
 }
 
+/**
+ * @pre GIL
+ */
 int SettingUniqueFromPyList(PyMOLGlobals * G, PyObject * list, int partial_restore)
 {
+  assert(PyGILState_Check());
+
   int ok = true;
   if(!partial_restore) {
     SettingUniqueResetAll(G);
@@ -768,6 +782,10 @@ int SettingUniqueFromPyList(PyMOLGlobals * G, PyObject * list, int partial_resto
                   case cSetting_boolean:
                     ok = PConvPyIntToInt(PyList_GetItem(entry_list, 2),
                                          &value_store.int_);
+                    if (setting_type == cSetting_color) {
+                      value_store.int_ =
+                          ColorConvertOldSessionIndex(G, value_store.int_);
+                    }
                     break;
                   case cSetting_float:
                     ok = PConvPyFloatToFloat(PyList_GetItem(entry_list, 2),
@@ -795,8 +813,13 @@ int SettingUniqueFromPyList(PyMOLGlobals * G, PyObject * list, int partial_resto
   return ok;
 }
 
+/**
+ * @pre GIL
+ */
 PyObject *SettingUniqueAsPyList(PyMOLGlobals * G)
 {
+  assert(PyGILState_Check());
+
   PyObject *result = NULL;
   CSettingUnique *I = G->SettingUnique;
   {
@@ -895,8 +918,13 @@ int SettingSetSmart_i(PyMOLGlobals * G, CSetting * set1, CSetting * set2, int in
   return SettingSetGlobal_i(G, index, value);
 }
 
+/**
+ * @pre GIL
+ */
 int SettingSetGlobalsFromPyList(PyMOLGlobals * G, PyObject * list)
 {
+  assert(PyGILState_Check());
+
   int ok = true;
 
   CSetting *I = G->Setting;
@@ -915,16 +943,26 @@ int SettingSetGlobalsFromPyList(PyMOLGlobals * G, PyObject * list)
   return (ok);
 }
 
+/**
+ * @pre GIL
+ */
 PyObject *SettingGetGlobalsAsPyList(PyMOLGlobals * G)
 {
+  assert(PyGILState_Check());
+
   PyObject *result = NULL;
   CSetting *I = G->Setting;
   result = SettingAsPyList(I);
   return (PConvAutoNone(result));
 }
 
+/**
+ * @pre GIL
+ */
 static PyObject *get_list(CSetting * I, int index, bool incl_blacklisted)
 {
+  assert(PyGILState_Check());
+
   PyObject *result = NULL, *value = NULL;
   int setting_type = SettingInfo[index].type;
 
@@ -960,8 +998,13 @@ static PyObject *get_list(CSetting * I, int index, bool incl_blacklisted)
   return result;
 }
 
+/**
+ * @pre GIL
+ */
 PyObject *SettingAsPyList(CSetting * I, bool incl_blacklisted)
 {
+  assert(PyGILState_Check());
+
   PyObject *result = NULL;
   int a;
 
@@ -1002,8 +1045,13 @@ static int SettingCheckUseShaders(CSetting * I, int quiet)
 }
 
 /*========================================================================*/
+/**
+ * @pre GIL
+ */
 static int set_list(CSetting * I, PyObject * list)
 {
+  assert(PyGILState_Check());
+
   int index = -1;
   int setting_type = -1;
 
@@ -1056,11 +1104,15 @@ ok_except1:
 }
 
 /*========================================================================*/
-/*
+/**
  * Used to set object and object-state level settings from PSEs
+ *
+ * @pre GIL
  */
 CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list)
 {
+  assert(PyGILState_Check());
+
   int ok = true;
   ov_size size;
   ov_size a;
@@ -1081,8 +1133,13 @@ CSetting *SettingNewFromPyList(PyMOLGlobals * G, PyObject * list)
 }
 
 /*========================================================================*/
+/**
+ * @pre GIL
+ */
 int SettingFromPyList(CSetting * I, PyObject * list)
 {
+  assert(PyGILState_Check());
+
   int ok = true;
   ov_size size;
   ov_size a;
@@ -1102,36 +1159,32 @@ int SettingFromPyList(CSetting * I, PyObject * list)
 }
 
 /*========================================================================*/
-/*
+/**
  * Get the indices of all settings that have changed since last calling
  * this function. Resets the "changed" flag.
  *
- * NOTE: assumes blocked interpreter
- *
- * name: object name or NULL/"" for global settings
- * state: object state
+ * @param name object name or NULL/"" for global settings
+ * @param state object state
  */
 std::vector<int> SettingGetUpdateList(PyMOLGlobals * G, const char * name, int state)
 {
-  CSetting **handle, *I = G->Setting;
-  int a;
-  int n;
+  CSetting* I = G->Setting;
+  pymol::copyable_ptr<CSetting>* handle;
   std::vector<int> result;
 
   if (name && name[0]) {
     // object-state settings
 
-    CObject *obj = ExecutiveFindObjectByName(G, name);
+    pymol::CObject *obj = ExecutiveFindObjectByName(G, name);
 
     if (!obj ||
         !(handle = obj->getSettingHandle(state)) ||
-        !(I = *handle))
+        !(I = handle->get()))
       // not found -> empty list
       return result;
   }
 
-  n = VLAGetSize(I->info);
-  for(a = 0; a < n; a++) {
+  for (int a = 0; a < cSetting_INIT; ++a) {
     if(I->info[a].changed) {
       I->info[a].changed = false;
       result.push_back(a);
@@ -1143,10 +1196,10 @@ std::vector<int> SettingGetUpdateList(PyMOLGlobals * G, const char * name, int s
 
 
 /*========================================================================*/
-void SettingCheckHandle(PyMOLGlobals * G, CSetting ** handle)
+void SettingCheckHandle(PyMOLGlobals * G, pymol::copyable_ptr<CSetting>& handle)
 {
-  if(!*handle)
-    *handle = SettingNew(G);
+  if(!handle)
+    handle.reset(SettingNew(G));
 }
 
 
@@ -1171,7 +1224,7 @@ int SettingGetTextValue(PyMOLGlobals * G, const CSetting * set1, const CSetting
 }
 
 /*========================================================================*/
-/*
+/**
  * Returns a pointer to the internal string representation if available,
  * or it formats the value into buffer and returns a pointer to buffer.
  */
@@ -1212,7 +1265,7 @@ const char * SettingGetTextPtr(PyMOLGlobals * G, const CSetting * set1, const CS
         case cColorBack:
           strcpy(buffer, "back");
           break;
-        case -1:
+        case cColorDefault:
           strcpy(buffer, "default");
           break;
         default:
@@ -1234,9 +1287,13 @@ const char * SettingGetTextPtr(PyMOLGlobals * G, const CSetting * set1, const CS
 
 #ifndef _PYMOL_NOPY
 /*========================================================================*/
+/**
+ * @pre GIL
+ */
 int SettingSetFromTuple(PyMOLGlobals * G, CSetting * I, int index, PyObject * tuple)
-/* must have interpret locked to make this call */
 {
+  assert(PyGILState_Check());
+
   PyObject *value;
   int type;
   int ok = true;
@@ -1295,8 +1352,6 @@ int SettingStringToTypedValue(PyMOLGlobals * G, int index, const char *st, int *
       }
       if (newvalue != *value){
           *value = newvalue;
-      } else {
-          ok = false;
       }
     break;
   case cSetting_int:
@@ -1304,8 +1359,6 @@ int SettingStringToTypedValue(PyMOLGlobals * G, int index, const char *st, int *
         ok = false;
     } else if (newvalue!=*value){
         *value = newvalue;
-    } else {
-        ok = false;
     }
     break;
   case cSetting_float:
@@ -1313,8 +1366,6 @@ int SettingStringToTypedValue(PyMOLGlobals * G, int index, const char *st, int *
         ok = false;
     } else if (newfvalue != *((float *) value)){
         *(float*)value = newfvalue;
-    } else {
-        ok = false;
     }
     break;
   case cSetting_color:
@@ -1322,8 +1373,6 @@ int SettingStringToTypedValue(PyMOLGlobals * G, int index, const char *st, int *
       int color_index = ColorGetIndex(G, st);
       if (*(value) != color_index){
           *(value) = color_index;
-      } else {
-          ok = false;
       }
     }
     break;
@@ -1396,11 +1445,15 @@ int SettingSetFromString(PyMOLGlobals * G, CSetting * I, int index, const char *
 
 /*========================================================================*/
 #ifndef _PYMOL_NOPY
-/*
+/**
  * Warning: Returns colors as (fff) tuple instead of color index
+ *
+ * @pre GIL
  */
 PyObject *SettingGetPyObject(PyMOLGlobals * G, const CSetting * set1, const CSetting * set2, int index)
-{                               /* assumes blocked python interpeter */
+{
+  assert(PyGILState_Check());
+
   PyObject *result = NULL;
   const float *ptr;
   int type = SettingGetType(G, index);
@@ -1437,8 +1490,13 @@ PyObject *SettingGetPyObject(PyMOLGlobals * G, const CSetting * set1, const CSet
   return result;
 }
 
+/**
+ * @pre GIL
+ */
 PyObject *SettingGetTuple(PyMOLGlobals * G, const CSetting * set1, const CSetting * set2, int index)
-{                               /* assumes blocked python interpeter */
+{
+  assert(PyGILState_Check());
+
   PyObject *result = NULL;
   const float *ptr;
   int type = SettingGetType(G, index);
@@ -1473,25 +1531,30 @@ PyObject *SettingGetTuple(PyMOLGlobals * G, const CSetting * set1, const CSettin
 /*========================================================================*/
 CSetting *SettingNew(PyMOLGlobals * G)
 {
-  OOAlloc(G, CSetting);
-  SettingInit(G, I);
-  return (I);
+  return new CSetting(G);
 }
 
+CSetting::CSetting(const CSetting& other)
+{
+  *this = other;
+}
+
+CSetting& CSetting::operator=(const CSetting& other)
+{
+  for (int index = 0; index < cSetting_INIT; ++index) {
+    SettingRecCopy(index, other.info[index], this->info[index]);
+  }
+  return *this;
+}
 
 /*========================================================================*/
-void SettingPurge(CSetting * I)
+CSetting::~CSetting()
 {
-  if(I) {
-    // need to free strings
-    for(int index = 0; index < cSetting_INIT; ++index) {
-      if (SettingInfo[index].type == cSetting_string) {
-        I->info[index].delete_s();
-      }
+  // need to free strings
+  for (int index = 0; index < cSetting_INIT; ++index) {
+    if (SettingInfo[index].type == cSetting_string) {
+      info[index].delete_s();
     }
-
-    VLAFreeP(I->info);
-    I->size = 0;
   }
 }
 
@@ -1499,55 +1562,26 @@ void SettingPurge(CSetting * I)
 /*========================================================================*/
 void SettingFreeP(CSetting * I)
 {
-  if(I)
-    SettingPurge(I);
-  OOFreeP(I);
+  delete I;
 }
 
 
 /*========================================================================*/
-void SettingInit(PyMOLGlobals * G, CSetting * I)
+CSetting::CSetting(PyMOLGlobals* G)
+    : G(G)
 {
-  I->G = G;
-  I->size = sizeof(int);        /* insures offset is never zero, except when undef */
-  I->info = (SettingRec*) VLAMalloc(cSetting_INIT, sizeof(SettingRec), 5, 1); /* auto-zero */
 }
 
 
 /*========================================================================*/
-/*
- * Return false if type is numeric and default value is non-zero.
- */
-bool SettingIsDefaultZero(int index)
-{
-  switch (SettingInfo[index].type) {
-    case cSetting_boolean:
-    case cSetting_int:
-    case cSetting_float:
-      if (SettingInfo[index].value.i[0] == 0)
-        return true;
-      return false;
-  }
-
-  return true;
-}
-
-
-/*========================================================================*/
-/*
+/**
  * Restore the default value from `src` or `SettingInfo`
  */
 void SettingRestoreDefault(CSetting * I, int index, const CSetting * src)
 {
   // 1) from stored default if provided
   if (src) {
-    UtilCopyMem(I->info + index, src->info + index, sizeof(SettingRec));
-
-    // need to properly copy strings
-    if (SettingInfo[index].type == cSetting_string && src->info[index].str_) {
-      I->info[index].str_ = new std::string(*src->info[index].str_);
-    }
-
+    SettingRecCopy(index, src->info[index], I->info[index]);
     return;
   }
 
@@ -1586,12 +1620,12 @@ void SettingRestoreDefault(CSetting * I, int index, const CSetting * src)
 int SettingUnset(CSetting * I, int index)
 {
   if(I) {
-    SettingRec *sr = I->info + index;
-    if (!sr->defined) {
+    SettingRec& sr = I->info[index];
+    if (!sr.defined) {
       return false;
     }
-    sr->defined = false;
-    sr->changed = true;
+    sr.defined = false;
+    sr.changed = true;
   }
   return true;
 }
@@ -1606,7 +1640,7 @@ int SettingGetType(int index)
 
 /*========================================================================*/
 template <>
-int SettingGet<int>(int index, const CSetting * I)
+int _SettingGet<int>(int index, const CSetting * I)
 {
   PyMOLGlobals *G = I->G;
   int result;
@@ -1631,7 +1665,7 @@ int SettingGet<int>(int index, const CSetting * I)
 
 /*========================================================================*/
 template <>
-bool SettingGet<bool>(int index, const CSetting * I)
+bool _SettingGet<bool>(int index, const CSetting * I)
 {
   PyMOLGlobals *G = I->G;
   switch (SettingInfo[index].type) {
@@ -1649,7 +1683,7 @@ bool SettingGet<bool>(int index, const CSetting * I)
 
 /*========================================================================*/
 template <>
-float SettingGet<float>(int index, const CSetting * I)
+float _SettingGet<float>(int index, const CSetting * I)
 {
   float result;
   PyMOLGlobals *G = I->G;
@@ -1675,7 +1709,7 @@ float SettingGet<float>(int index, const CSetting * I)
 
 /*========================================================================*/
 template <>
-const char * SettingGet<const char *>(int index, const CSetting * I)
+const char * _SettingGet<const char *>(int index, const CSetting * I)
 {
   const char *result;
   PyMOLGlobals *G = I->G;
@@ -1698,7 +1732,7 @@ const char * SettingGet<const char *>(int index, const CSetting * I)
 
 /*========================================================================*/
 template <>
-const float * SettingGet<const float *>(int index, const CSetting * I)
+const float * _SettingGet<const float *>(int index, const CSetting * I)
 {
   if (SettingInfo[index].type != cSetting_float3) {
     PyMOLGlobals *G = I->G;
@@ -2444,8 +2478,8 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
     break;
   case cSetting_button_mode_name:
     {
-      int internal_gui_mode = SettingGetGlobal_i(G, cSetting_internal_gui_mode);
-      if (internal_gui_mode){
+      auto internal_gui_mode = SettingGet<InternalGUIMode>(cSetting_internal_gui_mode, G->Setting);
+      if (internal_gui_mode != InternalGUIMode::Default){
 	OrthoInvalidateDoDraw(G);
       }
     }
@@ -2480,6 +2514,13 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
     SceneChanged(G);
     break;
   case cSetting_sphere_mode:
+#ifndef _PYMOL_IP_EXTRAS
+    if (SettingGet<int>(G, cSetting_sphere_mode) > 9) {
+      PRINTFB(G, FB_Setting, FB_Warnings)
+        " Setting-Warning: sphere_mode > 9 is not supported in Open-Source "
+        "version of PyMOL\n" ENDFB(G);
+    }
+#endif
     ExecutiveInvalidateRep(G, inv_sele, cRepSphere, cRepInvRep);
     SceneInvalidate(G);
     break;
@@ -2574,6 +2615,7 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
   case cSetting_surface_use_shader:
     SceneChanged(G);
     break;
+  case cSetting_isosurface_algorithm:
   case cSetting_surface_negative_visible:
     ExecutiveInvalidateRep(G, inv_sele, cRepSurface, cRepInvRep);
     SceneChanged(G);
@@ -2666,6 +2708,8 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
   case cSetting_cartoon_round_helices:
   case cSetting_cartoon_flat_sheets:
   case cSetting_cartoon_refine_normals:
+  case cSetting_cartoon_smooth_cylinder_cycles:
+  case cSetting_cartoon_smooth_cylinder_window:
   case cSetting_cartoon_smooth_loops:
   case cSetting_cartoon_dumbbell_width:
   case cSetting_cartoon_dumbbell_length:
@@ -2986,14 +3030,9 @@ void SettingGenerateSideEffects(PyMOLGlobals * G, int index, const char *sele, i
 /*========================================================================*/
 void SettingFreeGlobal(PyMOLGlobals * G)
 {
-  CSetting *I = G->Setting;
   SettingUniqueFree(G);
-  SettingPurge(I);
-  if(G->Default) {
-    SettingPurge(G->Default);
-    FreeP(G->Default);
-  }
-  FreeP(G->Setting);
+  DeleteP(G->Setting);
+  DeleteP(G->Default);
 }
 
 
@@ -3009,9 +3048,8 @@ void SettingInitGlobal(PyMOLGlobals * G, int alloc, int reset_gui, int use_defau
   int (*set_b) (CSetting * I, int index, int value) = SettingSet_b;
 
   if(alloc || !I) {
-    I = (G->Setting = pymol::calloc<CSetting>(1));
+    I = G->Setting = SettingNew(G);
     SettingUniqueInit(G);
-    SettingInit(G, I);
   }
 
   if(G->Default && use_default) {
@@ -3134,17 +3172,20 @@ int SettingCheckFontID(PyMOLGlobals * G, CSetting * set1, CSetting * set2, int f
   return ret;
 }
 
-/*
+/**
  * State index iterator constructor, see Setting.h for documentation.
+ *
+ * @param set Optional object settings (can be NULL)
+ * @param nstate Maximum number of states
  */
-StateIterator::StateIterator(PyMOLGlobals * G, CSetting * set, int state_, int nstate) {
-  if(state_ == -2) {
-    // current state
+StateIterator::StateIterator(
+    PyMOLGlobals* G, CSetting* set, StateIndex_t state_, int nstate)
+{
+  if (state_ == cStateCurrent) {
     state_ = SettingGet_i(G, set, NULL, cSetting_state) - 1;
   }
 
-  if(state_ == -1) {
-    // all states
+  if (state_ == cStateAll) {
     state = 0;
     end = nstate;
   } else {
@@ -3163,12 +3204,15 @@ StateIterator::StateIterator(PyMOLGlobals * G, CSetting * set, int state_, int n
   state--;
 }
 
-StateIterator::StateIterator(CObject* obj, int state_)
-    : StateIterator(obj->G, obj->Setting, state_, obj->getNFrame())
+/**
+ * Take settings and number of states from given object.
+ */
+StateIterator::StateIterator(pymol::CObject* obj, StateIndex_t state_)
+    : StateIterator(obj->G, obj->Setting.get(), state_, obj->getNFrame())
 {
 }
 
-/*
+/**
  * Helper function to init CPyMOL.Setting, a (name: index) dictionary.
  * Called in PyMOL_InitAPI
  */
@@ -3190,7 +3234,7 @@ bool CPyMOLInitSetting(OVLexicon * Lex, OVOneToOne * Setting) {
 }
 
 #ifndef _PYMOL_NOPY
-/*
+/**
  * Export the settings names to Python a as (name: index) dictionary.
  * Replacement for pymol.settings.SettingIndex
  */
@@ -3213,7 +3257,7 @@ PyObject * SettingGetSettingIndices() {
   return dict;
 }
 
-/*
+/**
  * Return a list of all setting indices for the given unique id
  */
 PyObject * SettingUniqueGetIndicesAsPyList(PyMOLGlobals * G, int unique_id)
