@@ -241,7 +241,7 @@ enum {
   REC_i( 156, two_sided_lighting                      , ostate    , -1 ),
   REC_i( 157, secondary_structure                     , global    , 2, 1, 4),
   REC_b( 158, auto_remove_hydrogens                   , global    , 0 ),
-  REC_b( 159, raise_exceptions                        , global    , 1 ),
+  REC_b( 159, raise_exceptions                        , unused    , 1 ),
   REC_b( 160, stop_on_exceptions                      , global    , 0 ),
   REC_b( 161, sculpting                               , ostate    , 0 ),
   REC_b( 162, auto_sculpt                             , global    , 0 ),
@@ -262,7 +262,7 @@ enum {
   REC_b( 177, legacy_vdw_radii                        , unused    , 0 ),
   REC_b( 178, sculpt_memory                           , ostate    , 1 ),
   REC_i( 179, connect_mode                            , global    , 0 ),
-  REC_b( 180, cartoon_cylindrical_helices             , ostate    , 0 ),
+  REC_i( 180, cartoon_cylindrical_helices             , ostate    , 0, 0, 2 ),
   REC_f( 181, cartoon_helix_radius                    , ostate    , 2.25F ),
   REC_f( 182, connect_cutoff                          , global    , 0.35F ),
   REC_b( 183, save_pdb_ss                             , unused    , 0 ),
@@ -513,7 +513,7 @@ enum {
   REC_b( 418, validate_object_names                   , global    , 1 ),
   REC_b( 419, unused_boolean_def_true                 , unused    , 1 ),
   REC_b( 420, auto_show_spheres                       , global    , 0 ),
-  REC_i( 421, sphere_mode                             , ostate    , 9, -1, 9 ),
+  REC_i( 421, sphere_mode                             , ostate    , 9, -1, 11 ),
   REC_f( 422, sphere_point_max_size                   , ostate    , 18.0f ),
   REC_f( 423, sphere_point_size                       , global    , 1.0f ),
   REC_b( 424, pdb_honor_model_number                  , global    , false ),
@@ -777,7 +777,7 @@ enum {
   REC_b( 677, stick_as_cylinders                      , global    , 1 ),
   REC_b( 678, mesh_as_cylinders                       , global    , 0 ),
   REC_b( 679, line_as_cylinders                       , global    , 0 ),
-  REC_b( 680, ribbon_as_cylinders                     , global    , 0 ),
+  REC_b( 680, ribbon_as_cylinders                     , ostate    , 0 ),
   REC_b( 681, ribbon_use_shader                       , global    , 1 ),
   REC_b( 682, excl_display_lists_shaders              , unused    , 0 ),
   REC_b( 683, dash_use_shader                         , global    , 1 ),
@@ -849,7 +849,7 @@ enum {
   REC_i( 739, volume_mode                             , global    , 1 ),
   REC_b( 740, trilines                                , global    , 0 ),
   REC_i( 741, collada_export_lighting                 , global    , 0 ),
-  REC_i( 742, collada_geometry_mode                   , global    , 0 ),
+  REC_i( 742, collada_geometry_mode                   , global    , 1 ),
   REC_b( 743, precomputed_lighting                    , global    , 0 ),
   REC_i( 744, chromadepth                             , global    , 0 ),
   REC_f( 745, pse_export_version                      , global    , 0.f ),
@@ -892,6 +892,9 @@ enum {
   REC_b( 782, openvr_cut_laser                        , global    , false ), // turn on to enable tu cut laser for molecule picker
   REC_f( 783, openvr_laser_width                      , global    , 3.0f ), // increase to make laser ray wider
   REC_f( 784, openvr_gui_distance                     , global    , 1.5f ),
+  REC_i( 785, cartoon_smooth_cylinder_cycles          , global    , 3 ),
+  REC_i( 786, cartoon_smooth_cylinder_window          , global    , 2 ),
+  REC_i( 787, isosurface_algorithm                    , global    , 0, 0, 2 ),
 
 
 #ifdef SETTINGINFO_IMPLEMENTATION
