@@ -155,7 +155,9 @@ void ViewElemDrawBox(PyMOLGlobals *G, BlockRect *rect, int first, int last,
   }
 }
 
-void ViewElemDraw(PyMOLGlobals *G, CViewElem * view_elem, BlockRect *rect, int frames,
+void ViewElemDraw(PyMOLGlobals *G,
+    const CViewElem * view_elem,
+    const BlockRect *rect, int frames,
     const char *title ORTHOCGOARG)
 {
   if(G->HaveGUI && G->ValidContext && view_elem) {
@@ -301,10 +303,17 @@ void ViewElemDraw(PyMOLGlobals *G, CViewElem * view_elem, BlockRect *rect, int f
     }
 
     if(title)
-      TextDrawStrAt(G, title, rect->right + 1, (rect->bottom+rect->top)/2 - 3 ORTHOCGOARGVAR);
+      ViewElemDrawLabel(G, title, rect, orthoCGO);
   }
 }
 
+void ViewElemDrawLabel(
+    PyMOLGlobals* G, const char* label, const BlockRect* rect, CGO* orthoCGO)
+{
+  TextDrawStrAt(
+      G, label, rect->right + 1, (rect->bottom + rect->top) / 2 - 3, orthoCGO);
+}
+
 void ViewElemCopy(PyMOLGlobals * G, const CViewElem * src, CViewElem * dst)
 {
   if(dst->scene_flag && dst->scene_name) {
@@ -329,7 +338,7 @@ void ViewElemArrayPurge(PyMOLGlobals * G, CViewElem * view, int nFrame)
   }
 }
 
-PyObject *ViewElemAsPyList(PyMOLGlobals * G, CViewElem * view)
+PyObject *ViewElemAsPyList(PyMOLGlobals * G, const CViewElem * view)
 {
 #ifdef _PYMOL_NOPY
   return NULL;
@@ -542,7 +551,7 @@ int ViewElemVLAFromPyList(PyMOLGlobals * G, PyObject * list, CViewElem ** vla_pt
   return ok;
 }
 
-PyObject *ViewElemVLAAsPyList(PyMOLGlobals * G, CViewElem * vla, int nFrame)
+PyObject *ViewElemVLAAsPyList(PyMOLGlobals * G, const CViewElem * vla, int nFrame)
 {
 #ifdef _PYMOL_NOPY
   return NULL;
@@ -606,17 +615,29 @@ int ViewIterate(CView * I, CViewIterator * iter, CRay * ray, int at_least_once)
 
       if(elem->pre_flag) {
         /* move the camera to the location we are looking at */
-        GLDOUBLETRANSLATE(elem->pre[0], elem->pre[1], elem->pre[2]);
+#ifdef PURE_OPENGL_ES_2
+        /* TODO */
+#else
+        glTranslated(elem->pre[0], elem->pre[1], elem->pre[2]);
+#endif	
       }
 
       if(elem->matrix_flag) {
         /* rotate about the origin (the the center of rotation) */
-        GLDOUBLEMULTMATRIX(elem->matrix);
+#ifdef PURE_OPENGL_ES_2
+        /* TODO */
+#else
+        glMultMatrixd(elem->matrix);
+#endif
       }
 
       if(elem->post_flag) {
         /* move the origin to the center of rotation */
-	GLDOUBLETRANSLATE(elem->post[0], elem->post[1], elem->post[2]);
+#ifdef PURE_OPENGL_ES_2
+        /* TODO */
+#else
+	glTranslated(elem->post[0], elem->post[1], elem->post[2]);
+#endif
       }
 
     }
@@ -696,7 +717,7 @@ int ViewElemSmooth(CViewElem * first, CViewElem * last, int window, int loop)
     window = (int) n;
   delta = (window - 1) / 2;
   if(n && delta) {
-    CViewElem *cpy = Alloc(CViewElem, (n + 2 * delta));
+    CViewElem *cpy = pymol::malloc<CViewElem>((n + 2 * delta));
     CViewElem *src, *dst;
     int a, b, c, cnt;
     memcpy(cpy + delta, first, sizeof(CViewElem) * n);
@@ -1120,11 +1141,11 @@ int ViewElemInterpolate(PyMOLGlobals * G, CViewElem * first, CViewElem * last,
     dump44f(firstR44f, "first");
 
   for(a = 0; a < n; a++) {
-    float fxn = ((float) a + 1) / (n + 1);
-    float fxn_1;
+    double fxn = (a + 1.0) / (n + 1.0);
+    double fxn_1 = 1.0 - fxn;
 
     if(timing_flag) {
-      timing = (first->timing * (1.0F - fxn) + (last->timing * fxn));
+      timing = (first->timing * fxn_1) + (last->timing * fxn);
     }
 
     if(state_flag) { /* states are interpolated linearly by default */
