@@ -159,11 +159,9 @@ int AtomInfoCheckUniqueBondID(PyMOLGlobals * G, BondType * bi)
 }
 
 void BondTypeInit(BondType *bt){
-#ifdef _PYMOL_IP_EXTRAS
-  bt->oldid = -1;
-#endif
   bt->unique_id = 0;
   bt->has_setting = 0;
+  bt->symop_2 = pymol::SymOp();
 }
 
 /**
@@ -175,8 +173,6 @@ void BondTypeInit2(BondType *bond, int i1, int i2, int order)
   bond->index[0] = i1;
   bond->index[1] = i2;
   bond->order = order;
-  bond->id = -1;
-  bond->stereo = 0;
 }
 
 int AtomInfoInit(PyMOLGlobals * G)
@@ -1982,55 +1978,6 @@ static int AtomInfoNameCompare(PyMOLGlobals * G, const lexidx_t& name1, const le
   return AtomInfoNameCompare(G, LexStr(G, name1), LexStr(G, name2));
 }
 
-int AtomInfoCompareAll(PyMOLGlobals * G, const AtomInfoType * at1, const AtomInfoType * at2)
-{
-  return (at1->resv != at2->resv ||
-	  at1->customType != at2->customType ||
-	  at1->priority != at2->priority ||
-	  at1->b != at2->b ||
-	  at1->q != at2->q ||
-	  at1->vdw != at2->vdw ||
-	  at1->partialCharge != at2->partialCharge ||
-	  at1->formalCharge != at2->formalCharge ||
-	  //	  at1->selEntry != at2->selEntry ||
-	  at1->color != at2->color ||
-	  at1->id != at2->id ||
-	  at1->flags != at2->flags ||
-	  //	  at1->temp1 != at2->temp1 ||
-	  at1->unique_id != at2->unique_id ||
-	  at1->discrete_state != at2->discrete_state ||
-	  at1->elec_radius != at2->elec_radius ||
-	  at1->rank != at2->rank ||
-	  at1->textType != at2->textType ||
-	  at1->custom != at2->custom ||
-	  at1->label != at2->label ||
-	  //	  !memcmp(at1->visRep, at2->visRep, sizeof(signed char)*cRepCnt) || // should this be in here?
-	  at1->stereo != at2->stereo ||
-	  at1->cartoon != at2->cartoon ||
-	  at1->hetatm != at2->hetatm ||
-	  at1->bonded != at2->bonded ||
-	  //	  at1->chemFlag != at2->chemFlag ||
-	  //	  at1->geom != at2->geom ||
-	  //	  at1->valence != at2->valence ||  // Valence should not be in, since it is not initially computed?
-	  at1->deleteFlag != at2->deleteFlag ||
-	  at1->masked != at2->masked ||
-	  at1->protekted != at2->protekted ||
-	  at1->protons != at2->protons ||
-	  at1->hb_donor != at2->hb_donor ||
-	  at1->hb_acceptor != at2->hb_acceptor ||
-	  at1->has_setting != at2->has_setting ||
-	  at1->chain != at2->chain ||
-	  at1->segi != at2->segi ||
-	  at1->resn != at2->resn ||
-	  at1->name != at2->name ||
-	  strcmp(at1->alt, at2->alt) || 
-	  at1->inscode != at2->inscode ||
-	  strcmp(at1->elem, at2->elem) || 
-	  strcmp(at1->ssType, at2->ssType));
-	  // should these variables be in here?
-	  //  float U11, U22, U33, U12, U13, U23;
-}
-
 /**
  * Compares atoms based on all atom identifiers, discrete state, priority,
  * hetatm (optional) and rank (optional)
@@ -2824,16 +2771,6 @@ void AtomInfoAssignParameters(PyMOLGlobals * G, AtomInfoType * I)
   }
 }
 
-int BondTypeCompare(PyMOLGlobals * G, const BondType * bt1, const BondType * bt2){
-  return (bt1->index[0] != bt2->index[0] ||
-	  bt1->index[1] != bt2->index[1] ||
-	  bt1->order != bt2->order ||
-	  bt1->id != bt2->id ||
-	  bt1->unique_id != bt2->unique_id ||
-	  bt1->stereo != bt2->stereo ||
-	  bt1->has_setting != bt2->has_setting);
-}
-
 /**
  * Get the element symbol. E.g. "He" for Helium.
  * @param[out] dst output buffer of length cElemNameLen
@@ -3011,3 +2948,8 @@ void AtomInfoGetAlignedPDBAtomName(PyMOLGlobals * G,
 
   name[4] = 0;
 }
+
+bool BondType::hasSymOp() const
+{
+  return symop_2;
+}
