@@ -40,55 +40,87 @@ Z* -------------------------------------------------------------------
 
 int DistSetGetLabelVertex(DistSet * I, int at, float *v)
 {
-  if((at >= 0) && (at < I->NLabel) && I->LabCoord) {
-    float *vv = I->LabCoord + 3 * at;
-    copy3f(vv, v);
+  if (at >= 0 && at < I->LabCoord.size()) {
+    const auto& vv = I->LabCoord[at];
+    copy3f(vv.data(), v);
     return true;
   }
   return false;
 }
 
-int DistSetMoveLabel(DistSet * I, int at, float *v, int mode)
+/**
+ * Retrieves label offset for given distance set
+ * @param atm atom index
+ */
+
+pymol::Result<pymol::Vec3> DistSet::getLabelOffset(int atm) const
 {
-  ObjectDist *obj;
-  int a1 = at;
-  int result = 0;
-  LabPosType *lp;
-
-  obj = I->Obj;
-
-  if(a1 >= 0) {
-
-    if(!I->LabPos)
-      I->LabPos.resize(I->NLabel);
-    if(I->LabPos) {
-      result = 1;
-      lp = I->LabPos + a1;
-      if(!lp->mode) {
-        const float *lab_pos = SettingGet_3fv(obj->G, NULL, obj->Setting,
-                                        cSetting_label_position);
-        copy3f(lab_pos, lp->pos);
-      }
-      lp->mode = 1;
-      if(mode) {
-        add3f(v, lp->offset, lp->offset);
-      } else {
-        copy3f(v, lp->offset);
-      }
-    }
+  std::array<float, 3> result;
+  if (atm < 0 || atm >= this->LabPos.size()) {
+    return pymol::make_error("Invalid index");
   }
 
-  return (result);
+  auto lp = &this->LabPos[atm];
+  if (lp->mode) {
+    std::copy_n(lp->offset, result.size(), result.data());
+    return result;
+  }
+
+  auto obj = this->Obj;
+  auto G = obj->G;
+  auto lab_pos =
+      SettingGet_3fv(G, nullptr, obj->Setting.get(), cSetting_label_position);
+  std::copy_n(lab_pos, result.size(), result.data());
+  return result;
+}
+
+/**
+ * Retrieves label offset for given distance set
+ * @param atm atom index
+ * @param pos offset to set the label at
+ */
+
+pymol::Result<> DistSet::setLabelOffset(int atm, const float* pos)
+{
+  if (atm < 0) {
+    return pymol::make_error("Invalid index");
+  }
+  VecCheck(this->LabPos, atm);
+  auto& lp = this->LabPos[atm];
+  lp.mode = 1;
+  copy3f(pos, lp.offset);
+  return {};
+}
+
+int DistSetMoveLabel(DistSet * I, int a1, float *v, int mode)
+{
+  if(a1 < 0) {
+    return 0;
+  }
+
+  VecCheck(I->LabPos, a1);
+  auto& lp = I->LabPos[a1];
+  if(!lp.mode) {
+    auto obj = I->Obj;
+    const float *lab_pos = SettingGet_3fv(obj->G, NULL, obj->Setting.get(),
+                                    cSetting_label_position);
+    copy3f(lab_pos, lp.pos);
+  }
+  lp.mode = 1;
+  if(mode) {
+    add3f(v, lp.offset, lp.offset);
+  } else {
+    copy3f(v, lp.offset);
+  }
+
+  return 1;
 }
 
 
-/* -- JV, refactored by TH */
-/*
- * PARAMS
- *   I:   measurement set, must not be NULL
- *   obj: object molecule, can be NULL so then all items in I will be updated
- * RETURNS
- *   number of updated coordinates
+/**
+ * @param I   measurement set, must not be NULL
+ * @param obj object molecule, can be NULL so then all items in I will be updated
+ * @return  number of updated coordinates
  */
 int DistSetMoveWithObject(DistSet * I, struct ObjectMolecule *obj)
 {
@@ -141,7 +173,7 @@ int DistSetMoveWithObject(DistSet * I, struct ObjectMolecule *obj)
   }
 
   if (rVal)
-    I->invalidateRep(-1, cRepInvCoord);
+    I->invalidateRep(cRepAll, cRepInvCoord);
 
   PRINTFD(G, FB_DistSet)
     " DistSet: done updating distance set's vertex\n" ENDFD;
@@ -219,16 +251,14 @@ ok_except1:
   return PConvAutoNone(result);
 }
 
-int DistSetFromPyList(PyMOLGlobals * G, PyObject * list, DistSet ** cs)
+DistSet* DistSetFromPyList(PyMOLGlobals * G, PyObject * list)
 {
   DistSet *I = NULL;
   int ll = 0;
   CPythonVal *val;
 
-  DeleteP(*cs);
-
   if(CPythonVal_IsNone(list)) {         /* allow None for CSet */
-    return true;
+    return nullptr;
   }
 
   ok_assert(1, list && PyList_Check(list));
@@ -243,7 +273,6 @@ int DistSetFromPyList(PyMOLGlobals * G, PyObject * list, DistSet ** cs)
 
   ok_assert(2, ll > 2);
 
-  I->LabCoord = NULL; // will be calculated in RepDistLabelNew
   ok_assert(1, CPythonVal_PConvPyIntToInt_From_List(G, list, 3, &I->NAngleIndex));
   ok_assert(1, CPythonVal_PConvPyListToFloatVLANoneOkay_From_List(G, list, 4, &I->AngleCoord));
   ok_assert(1, CPythonVal_PConvPyIntToInt_From_List(G, list, 5, &I->NDihedralIndex));
@@ -259,7 +288,11 @@ int DistSetFromPyList(PyMOLGlobals * G, PyObject * list, DistSet ** cs)
   ok_assert(2, ll > 8);
 
   val = CPythonVal_PyList_GetItem(G, list, 8);
-  ok_assert(1, CPythonVal_PConvPyListToLabPosVLA(G, val, &I->LabPos));
+  {
+    auto labPosRes = CPythonVal_PConvPyListToLabPosVec(G, val);
+    ok_assert(1, (bool)labPosRes);
+    I->LabPos = std::move(*labPosRes);
+  }
   CPythonVal_Free(val);
 
   ok_assert(2, ll > 9);
@@ -269,11 +302,11 @@ int DistSetFromPyList(PyMOLGlobals * G, PyObject * list, DistSet ** cs)
   CPythonVal_Free(val);
 
 ok_except2:
-  *cs = I;
-  return true;
+  return I;
 ok_except1:
   delete I;
-  return false;
+  return nullptr;
+
 }
 
 PyObject *DistSetAsPyList(DistSet * I)
@@ -295,8 +328,8 @@ PyObject *DistSetAsPyList(DistSet * I)
                                                    I->NDihedralIndex * 3));
     // DistSet->Setting never gets set (removed BB 11/14), was state settings?
     PyList_SetItem(result, 7, PConvAutoNone(NULL) /* SettingAsPyList(I->Setting) */);
-    if(I->LabPos) {
-      PyList_SetItem(result, 8, PConvLabPosVLAToPyList(I->LabPos, VLAGetSize(I->LabPos)));
+    if(!I->LabPos.empty()) {
+      PyList_SetItem(result, 8, PConvLabPosVecToPyList(I->LabPos));
     } else {
       PyList_SetItem(result, 8, PConvAutoNone(NULL));
     }
@@ -354,13 +387,13 @@ int DistSetGetExtent(DistSet * I, float *mn, float *mx)
 
 
 /*========================================================================*/
-/*
+/**
  * Invalidate reps
  *
  * type: rep enum, e.g. cRepDash
  * level: e.g. cRepInvColor
  */
-void DistSet::invalidateRep(int type, int level)
+void DistSet::invalidateRep(cRep_t type, cRepInv_t level)
 {
   int a = 0, a_stop = getNRep();
   bool changed = false;
@@ -377,8 +410,7 @@ void DistSet::invalidateRep(int type, int level)
   for(; a < a_stop; a++) {
     if(Rep[a]) {
       changed = true;
-      Rep[a]->fFree(Rep[a]);
-      Rep[a] = NULL;
+      Rep[a].reset();
     }
   }
 
@@ -396,25 +428,25 @@ void DistSet::update(int state)
   if(!I->Rep[cRepDash]) {
     /* query the dist set looking for the selected atoms for this distance,
      * then update the *coords */
-    I->Rep[cRepDash] = RepDistDashNew(I,state);
+    I->Rep[cRepDash].reset(RepDistDashNew(I,state));
     SceneInvalidate(G);
   }
   if(!I->Rep[cRepLabel]) {
     /* query the dist set looking for the selected atoms for this distance,
      * then update the *coords */
-    I->Rep[cRepLabel] = RepDistLabelNew(I, state);
+    I->Rep[cRepLabel].reset(RepDistLabelNew(I, state));
     SceneInvalidate(G);
   }
   if(!I->Rep[cRepAngle]) {
     /* query the angle set looking for the selected atoms for this distance,
      * then update the *coords */
-    I->Rep[cRepAngle] = RepAngleNew(I, state);
+    I->Rep[cRepAngle].reset(RepAngleNew(I, state));
     SceneInvalidate(G);
   }
   if(!I->Rep[cRepDihedral]) {
     /* query the dihedral set looking for the selected atoms for this distance,
      * then update the *coords */
-    I->Rep[cRepDihedral] = RepDihedralNew(I, state);
+    I->Rep[cRepDihedral].reset(RepDihedralNew(I, state));
     SceneInvalidate(G);
   }
   /* status bar 100% */
@@ -428,38 +460,33 @@ void DistSet::render(RenderInfo * info)
   DistSet * I = this;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  ::Rep *r;
   for (int a = 0; a < I->getNRep(); a++) {
     if(!GET_BIT(I->Obj->visRep, a))
       continue;
     if(!I->Rep[a]) {
       switch(a) {
       case cRepDash:
-        I->Rep[a] = RepDistDashNew(I, -1);
+        I->Rep[a].reset(RepDistDashNew(I, -1));
         break;
       case cRepLabel:
-        I->Rep[a] = RepDistLabelNew(I, -1);
+        I->Rep[a].reset(RepDistLabelNew(I, -1));
         break;
       case cRepAngle:
-        I->Rep[a] = RepAngleNew(I, -1);
+        I->Rep[a].reset(RepAngleNew(I, -1));
         break;
       case cRepDihedral:
-        I->Rep[a] = RepDihedralNew(I, -1);
+        I->Rep[a].reset(RepDihedralNew(I, -1));
         break;
       }
     }
     if(I->Rep[a])
       {
-        r = I->Rep[a];
-        if(ray || pick) {
-          if(ray)
+        if(ray) {
             ray->color3fv(ColorGet(G, I->Obj->Color));
-          r->fRender(r, info);
-        } else {
-          ObjectUseColor((CObject *) I->Obj);
-          // now all filtering for passes happen in ObjectDistRender
-          r->fRender(r, info);
+        } else if (!pick) {
+          ObjectUseColor(I->Obj);
         }
+        Rep[a]->render(info);
       }
   }
 }
@@ -471,11 +498,3 @@ DistSet::DistSet(PyMOLGlobals* G)
 {
 }
 
-/*========================================================================*/
-DistSet::~DistSet()
-{
-  for (int a = 0; a < getNRep(); ++a) {
-    if (Rep[a])
-      Rep[a]->fFree(Rep[a]);
-  }
-}
