@@ -25,9 +25,8 @@ void AppendAtomVertex(CoordSet* cs, unsigned atm, const float* v)
 {
   assert(cs->atmToIdx(atm) == -1);
 
-  int idx = cs->NIndex++;
-  VLACheck(cs->Coord, float, idx * 3 + 2);
-  VLACheck(cs->IdxToAtm, int, idx);
+  int const idx = cs->NIndex;
+  cs->setNIndex(idx + 1);
 
   cs->IdxToAtm[idx] = atm;
 
@@ -69,13 +68,12 @@ bool get_planer_normal_cs(
 
   const float* center_coord = cs->coordPtr(idx);
 
-  int neighbor_atm, tmp;
-  ITERNEIGHBORATOMS(I->Neighbor, atm, neighbor_atm, tmp) {
-    if (h_fix && I->AtomInfo[neighbor_atm].isHydrogen())
+  for (auto const& neighbor : AtomNeighbors(I, atm)) {
+    if (h_fix && I->AtomInfo[neighbor.atm].isHydrogen())
       continue;
 
     // get neighbor coordinate
-    int neighbor_idx = cs->atmToIdx(neighbor_atm);
+    int neighbor_idx = cs->atmToIdx(neighbor.atm);
     if (neighbor_idx == -1)
       continue;
 
@@ -144,16 +142,15 @@ int ObjectMoleculeSetMissingNeighborCoords(
 
   const float* center_coord = cs->coordPtr(idx);
 
-  int neighbor_atm, tmp;
-  ITERNEIGHBORATOMS(I->Neighbor, atm, neighbor_atm, tmp) {
+  for (auto const& neighbor : AtomNeighbors(I, atm)) {
     if (n_present == 4)
       break;
 
     // get neighbor coordinate
-    int neighbor_idx = cs->atmToIdx(neighbor_atm);
+    int neighbor_idx = cs->atmToIdx(neighbor.atm);
     if (neighbor_idx == -1 ||
-        (h_fix && I->AtomInfo[neighbor_atm].isHydrogen())) {
-      missing_atm[n_missing++] = neighbor_atm;
+        (h_fix && I->AtomInfo[neighbor.atm].isHydrogen())) {
+      missing_atm[n_missing++] = neighbor.atm;
       continue;
     }
 
@@ -164,7 +161,7 @@ int ObjectMoleculeSetMissingNeighborCoords(
     subtract3f(neighbor_coord, center_coord, vvec);
     normalize3f(vvec);
 
-    present_atm = neighbor_atm;
+    present_atm = neighbor.atm;
     ++n_present;
   }
 
@@ -300,8 +297,6 @@ int ObjectMoleculeAddSeleHydrogensRefactored(ObjectMolecule* I, int sele, int st
     return false;
   }
 
-  ObjectMoleculeUpdateNeighbors(I);
-
   // add hydrogens (without coordinates)
   for (unsigned atm = 0; atm < n_atom_old; ++atm) {
     const auto ai = I->AtomInfo + atm;
@@ -312,7 +307,7 @@ int ObjectMoleculeAddSeleHydrogensRefactored(ObjectMolecule* I, int sele, int st
     if (!SelectorIsMember(G, ai->selEntry, sele))
       continue;
 
-    int nneighbors = I->Neighbor[I->Neighbor[atm]];
+    int nneighbors = AtomNeighbors(I, atm).size();
     int nimplicit = ai->valence - nneighbors;
 
     if (nimplicit <= 0)
@@ -336,15 +331,9 @@ int ObjectMoleculeAddSeleHydrogensRefactored(ObjectMolecule* I, int sele, int st
   }
 
   // grow index arrays
-  for (StateIterator iter(G, nullptr, cSelectorUpdateTableAllStates, I->NCSet);
-      iter.next();) {
-    CoordSet* cs = I->CSet[iter.state];
-    if (cs)
-      cs->extendIndices(I->NAtom);
-  }
+  ObjectMoleculeExtendIndices(I, cSelectorUpdateTableAllStates);
 
   I->invalidate(cRepAll, cRepInvBonds, state);
-  ObjectMoleculeUpdateNeighbors(I);
 
   AtomInfoUniquefyNames(G,
       I->AtomInfo, n_atom_old,
