@@ -120,14 +120,14 @@ static int AlignmentFindTag(PyMOLGlobals * G, AtomInfoType * ai, int sele,
   return result;
 }
 
-/*
+/**
  * Get single letter abbreviation for CLUSTAL output.
  *
  * See also:
  * pymol.exporting._resn_to_aa
- * AtomInfoKnownNucleicResName
- * AtomInfoKnownProteinResName
- * SeekerGetAbbr
+ * AtomInfoKnownNucleicResName()
+ * AtomInfoKnownProteinResName()
+ * SeekerGetAbbr()
  */
 static char get_abbr(PyMOLGlobals * G, const AtomInfoType * ai) {
   const char * resn = LexStr(G, ai->resn);
@@ -292,7 +292,7 @@ int ObjectAlignmentAsStrVLA(PyMOLGlobals * G, ObjectAlignment * I, int state, in
               int a;
               for(a = 0; a < nRow; a++) {
                 row = row_vla + a;
-                row->txt = pymol::calloc<char>(nCol + 1);
+                row->txt = pymol::vla<char>(nCol + 1);
                 row->len = 0;
                 row->last_ai = NULL;
                 row->cCol = 0;
@@ -449,7 +449,7 @@ int ObjectAlignmentAsStrVLA(PyMOLGlobals * G, ObjectAlignment * I, int state, in
             int a;
             for(a = 0; a < nRow; a++) {
               row = row_vla + a;
-              FreeP(row->txt);
+              row->txt.freeP();
             }
           }
           FreeP(cons_str);
@@ -1065,7 +1065,7 @@ void ObjectAlignment::render(RenderInfo * info)
   int state = info->state;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  int pass = info->pass;
+  const RenderPass pass = info->pass;
   ObjectAlignmentState *sobj = NULL;
   const float *color;
 
@@ -1076,17 +1076,17 @@ void ObjectAlignment::render(RenderInfo * info)
   if (pick)
     return;
 
-  if(pass>0 || ray) {
+  if(pass == RenderPass::Opaque || ray) {
     if((I->visRep & cRepCGOBit)) {
 
-      for(StateIterator iter(G, I->Setting, state, I->getNFrame()); iter.next();) {
+      for(StateIterator iter(G, I->Setting.get(), state, I->getNFrame()); iter.next();) {
         sobj = I->State.data() + iter.state;
 
         if (!sobj->primitiveCGO)
           continue;
 
 	if(ray) {
-	    CGORenderRay(sobj->primitiveCGO.get(), ray, info, color, NULL, I->Setting, NULL);
+	    CGORenderRay(sobj->primitiveCGO.get(), ray, info, color, NULL, I->Setting.get(), NULL);
 	} else if(G->HaveGUI && G->ValidContext) {
 #ifndef PURE_OPENGL_ES_2
 	  if(!info->line_lighting)
@@ -1142,7 +1142,7 @@ void ObjectAlignment::render(RenderInfo * info)
           }
 
           if (cgo) {
-            CGORenderGL(cgo, color, I->Setting, NULL, info, NULL);
+            CGORenderGL(cgo, color, I->Setting.get(), NULL, info, NULL);
           }
 
 #ifndef PURE_OPENGL_ES_2
@@ -1154,10 +1154,10 @@ void ObjectAlignment::render(RenderInfo * info)
   }
 }
 
-void ObjectAlignment::invalidate(int rep, int level, int state)
+void ObjectAlignment::invalidate(cRep_t rep, cRepInv_t level, int state)
 {
   if((rep == cRepAll) || (rep == cRepCGO)) {
-    for(StateIterator iter(G, Setting, state, getNFrame()); iter.next();) {
+    for(StateIterator iter(G, Setting.get(), state, getNFrame()); iter.next();) {
       ObjectAlignmentState& sobj = State[iter.state];
       sobj.valid = false;
       sobj.renderCGO.reset();
@@ -1167,7 +1167,7 @@ void ObjectAlignment::invalidate(int rep, int level, int state)
 
 
 /*========================================================================*/
-ObjectAlignment::ObjectAlignment(PyMOLGlobals * G) : CObject(G)
+ObjectAlignment::ObjectAlignment(PyMOLGlobals * G) : pymol::CObject(G)
 {
   type = cObjectAlignment;
 }
@@ -1225,7 +1225,7 @@ ObjectAlignment *ObjectAlignmentDefine(PyMOLGlobals * G,
   return (I);
 }
 
-CObject* ObjectAlignment::clone() const
+pymol::CObject* ObjectAlignment::clone() const
 {
   return new ObjectAlignment(*this);
 }
