@@ -109,7 +109,7 @@ static void ObjectGadgetRampCalculate(ObjectGadgetRamp * I, float v, float *resu
   clamp3f(result);
 }
 
-/*
+/**
  * Get the "Level" array, eventually spaced out to match number of colors.
  */
 static const float * ObjectGadgetRampGetLevel(ObjectGadgetRamp * I) {
@@ -132,7 +132,7 @@ static const float * ObjectGadgetRampGetLevel(ObjectGadgetRamp * I) {
   return I->LevelTmp;
 }
 
-/*
+/**
  * We support storing special color indices as negative R in the RGB color
  */
 inline int GetSpecial(const float * rgb) {
@@ -475,7 +475,7 @@ int ObjectGadgetRampInterVertex(ObjectGadgetRamp * I, const float *pos, float *c
   case cRampMap:
     if(!I->Map)
       I->Map = ExecutiveFindObjectMapByName(I->G, I->SrcName);
-    if(!ExecutiveValidateObjectPtr(I->G, (CObject *) I->Map, cObjectMap))
+    if(!ExecutiveValidateObjectPtr(I->G, I->Map, cObjectMap))
       ok = false;
     else {
       int src_state;
@@ -496,7 +496,7 @@ int ObjectGadgetRampInterVertex(ObjectGadgetRamp * I, const float *pos, float *c
   case cRampMol:
     if(!I->Mol)
       I->Mol = ExecutiveFindObjectMoleculeByName(I->G, I->SrcName);
-    if(!ExecutiveValidateObjectPtr(I->G, (CObject *) I->Mol, cObjectMolecule))
+    if(!ExecutiveValidateObjectPtr(I->G, I->Mol, cObjectMolecule))
       ok = false;
     else {
       float cutoff = 1.0F;
@@ -517,7 +517,7 @@ int ObjectGadgetRampInterVertex(ObjectGadgetRamp * I, const float *pos, float *c
 	if (I->Mol->NCSet==1) // if only one state, then set state to 0
 	  state = 0;
         if(SettingGet_b
-           (I->G, I->Setting, NULL,
+           (I->G, I->Setting.get(), NULL,
             cSetting_ramp_blend_nearby_colors)) {
           float atomic[3];
           int index =
@@ -830,13 +830,12 @@ static void ObjectGadgetRampBuild(ObjectGadgetRamp * I)
 
   // was a memory leak in < 1.8.3.1
   for (int i = 0; i < og->NGSet; ++i) {
-    og->GSet[i]->fFree();
+    delete og->GSet[i];
     og->GSet[i] = NULL;
   }
 
   og->GSet[0] = gs;
   og->NGSet = 1;
-  og->Context = 1;          /* unit window */
   gs->Obj = (ObjectGadget *) I;
   gs->State = 0;
 
@@ -1031,7 +1030,7 @@ ObjectGadgetRamp *ObjectGadgetRampMolNewAsDefined(PyMOLGlobals * G,
   return (I);
 }
 
-void ObjectGadgetRamp::invalidate(int rep, int level,
+void ObjectGadgetRamp::invalidate(cRep_t rep, cRepInv_t level,
                                        int state)
 {
 }
