@@ -24,7 +24,7 @@ Z* -------------------------------------------------------------------
 #include"ObjectMesh.h"
 #include"Base.h"
 #include"MemoryDebug.h"
-#include"Map.h"
+#include"CarveHelper.h"
 #include"Parse.h"
 #include"Isosurf.h"
 #include"Vector.h"
@@ -64,7 +64,7 @@ static PyObject *ObjectMeshStateAsPyList(ObjectMeshState * I)
   } else {
     PyList_SetItem(result, 12, PConvAutoNone(NULL));
   }
-  PyList_SetItem(result, 13, PyInt_FromLong(I->MeshMode));
+  PyList_SetItem(result, 13, PyInt_FromLong(static_cast<int>(I->MeshMode)));
   PyList_SetItem(result, 14, PyFloat_FromDouble(I->AltLevel));
   PyList_SetItem(result, 15, PyInt_FromLong(I->quiet));
   if(I->Field) {
@@ -84,7 +84,7 @@ int ObjectMeshAllMapsInStatesExist(ObjectMesh * I)
   int a;
   for(a = 0; a < I->NState; a++) {
     if(I->State[a].Active) {
-      if (!ObjectMeshStateMapExists(I, I->State + a)){
+      if (!ObjectMeshStateMapExists(I, &I->State[a])){
 	return 0;
       }
     }
@@ -100,7 +100,7 @@ static PyObject *ObjectMeshAllStatesAsPyList(ObjectMesh * I)
   result = PyList_New(I->NState);
   for(a = 0; a < I->NState; a++) {
     if(I->State[a].Active) {
-      PyList_SetItem(result, a, ObjectMeshStateAsPyList(I->State + a));
+      PyList_SetItem(result, a, ObjectMeshStateAsPyList(&I->State[a]));
     } else {
       PyList_SetItem(result, a, PConvAutoNone(NULL));
     }
@@ -162,7 +162,7 @@ static int ObjectMeshStateFromPyList(PyMOLGlobals * G, ObjectMeshState * I,
           ok = PConvPyListToFloatVLA(tmp, &I->AtomVertex);
       }
       if(ok)
-        ok = PConvPyIntToInt(PyList_GetItem(list, 13), &I->MeshMode);
+        ok = PConvFromPyListItem(G, list, 13, I->MeshMode);
       if(ok) {
         I->RefreshFlag = true;
         I->ResurfaceFlag = true;
@@ -197,13 +197,13 @@ static int ObjectMeshAllStatesFromPyList(ObjectMesh * I, PyObject * list)
 
   int ok = true;
   int a;
-  VLACheck(I->State, ObjectMeshState, I->NState);
+  VecCheckEmplace(I->State, I->NState, I->G);
   if(ok)
     ok = PyList_Check(list);
   if(ok) {
     for(a = 0; a < I->NState; a++) {
       auto *el = PyList_GetItem(list, a);
-      ok = ObjectMeshStateFromPyList(I->G, I->State + a, el);
+      ok = ObjectMeshStateFromPyList(I->G, &I->State[a], el);
       if(!ok)
         break;
     }
@@ -284,7 +284,7 @@ int ObjectMeshInvalidateMapName(ObjectMesh * I, const char *name, const char * n
   ObjectMeshState *ms;
   int result = false;
   for(a = 0; a < I->NState; a++) {
-    ms = I->State + a;
+    ms = &I->State[a];
     if(ms->Active) {
       if(strcmp(ms->MapName, name) == 0) {
         if (new_name)
@@ -314,7 +314,7 @@ void ObjectMeshDump(ObjectMesh * I, const char *fname, int state, int quiet)
       if(n && v)
         while(*n) {
           c = *(n++);
-          if(!I->State[state].MeshMode) {
+          if(I->State[state].MeshMode == cIsomeshMode::isomesh) {
             fprintf(f, "\n");
           }
           while(c--) {
@@ -331,7 +331,7 @@ void ObjectMeshDump(ObjectMesh * I, const char *fname, int state, int quiet)
   }
 }
 
-void ObjectMesh::invalidate(int rep, int level, int state)
+void ObjectMesh::invalidate(cRep_t rep, cRepInv_t level, int state)
 {
   auto I = this;
   if(level >= cRepInvExtents) {
@@ -340,7 +340,7 @@ void ObjectMesh::invalidate(int rep, int level, int state)
   if((rep == cRepMesh) || (rep == cRepAll) || (rep == cRepCell)) {
 
     for(StateIterator iter(I->G, NULL, state, I->NState); iter.next();) {
-      ObjectMeshState *ms = I->State + iter.state;
+      ObjectMeshState *ms = &I->State[iter.state];
 
       ms->shaderCGO.reset();
       ms->shaderUnitCellCGO.reset();
@@ -367,7 +367,7 @@ pymol::Result<float> ObjectMeshGetLevel(ObjectMesh * I, int state)
     if(state < 0) {
       state = 0;
     }
-    auto ms = I->State + state;
+    auto ms = &I->State[state];
     if(ms->Active) {
       return ms->Level;
     } else {
@@ -383,7 +383,7 @@ int ObjectMeshSetLevel(ObjectMesh * I, float level, int state, int quiet)
     ok = false;
   } else {
     for(StateIterator iter(I->G, NULL, state, I->NState); iter.next();) {
-      ObjectMeshState *ms = I->State + iter.state;
+      ObjectMeshState *ms = &I->State[iter.state];
       if(ms->Active) {
         ms->ResurfaceFlag = true;
         ms->RefreshFlag = true;
@@ -400,10 +400,10 @@ static void ObjectMeshStateUpdateColors(ObjectMesh * I, ObjectMeshState * ms)
   int one_color_flag = true;
   int cur_color = -1;
 
-  if(ms->MeshMode == 0) {
-    cur_color = SettingGet_color(I->G, I->Setting, NULL, cSetting_mesh_color);
-  } else if(ms->MeshMode == 1) {
-    cur_color = SettingGet_color(I->G, I->Setting, NULL, cSetting_dot_color);
+  if(ms->MeshMode == cIsomeshMode::isomesh) {
+    cur_color = SettingGet_color(I->G, I->Setting.get(), NULL, cSetting_mesh_color);
+  } else if(ms->MeshMode == cIsomeshMode::isodot) {
+    cur_color = SettingGet_color(I->G, I->Setting.get(), NULL, cSetting_dot_color);
   }
 
   if(cur_color == -1)
@@ -419,7 +419,7 @@ static void ObjectMeshStateUpdateColors(ObjectMesh * I, ObjectMeshState * ms)
     float *vc;
     int *rc;
     int a;
-    int state = ms - I->State;
+    int state = ms - I->State.data();
     int n_vert = VLAGetSize(ms->V) / 3;
     int base_n_vert = ms->base_n_V / 3;
 
@@ -440,7 +440,7 @@ static void ObjectMeshStateUpdateColors(ObjectMesh * I, ObjectMeshState * ms)
     if(vc) {
       for(a = 0; a < n_vert; a++) {
         if(a == base_n_vert) {
-          int new_color = SettingGet_color(I->G, I->Setting,
+          int new_color = SettingGet_color(I->G, I->Setting.get(),
                                            NULL, cSetting_mesh_negative_color);
           if(new_color == -1)
             new_color = cur_color;
@@ -468,7 +468,7 @@ static void ObjectMeshStateUpdateColors(ObjectMesh * I, ObjectMeshState * ms)
       ms->RC.clear();
     } else if((!ramped_flag)
               ||
-              (!SettingGet_b(I->G, NULL, I->Setting, cSetting_ray_color_ramps))) {
+              (!SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_ray_color_ramps))) {
       ms->RC.clear();
     }
   }
@@ -485,20 +485,15 @@ void ObjectMesh::update()
 
   int *n;
   float *v;
-  float carve_buffer;
-  int avoid_flag = false;
   int n_cur;
   int n_seg;
   int n_line;
   int flag;
   int last_flag = 0;
-  int h, k, l;
-  int i, j;
-  int mesh_skip = SettingGet_i(G, I->Setting, NULL, cSetting_mesh_skip);
+  int mesh_skip = SettingGet_i(G, I->Setting.get(), NULL, cSetting_mesh_skip);
 
-  MapType *voxelmap;            /* this has nothing to do with isosurfaces... */
   for(a = 0; a < I->NState; a++) {
-    ms = I->State + a;
+    ms = &I->State[a];
     if(ms->Active) {
 
       map = ExecutiveFindObjectMapByName(I->G, ms->MapName);
@@ -569,22 +564,22 @@ void ObjectMesh::update()
                ms->Range[3],
                ms->Range[4],
                ms->Range[5]); */
-            IsosurfVolume(I->G, I->Setting, NULL,
+            IsosurfVolume(I->G, I->Setting.get(), NULL,
                           field,
                           ms->Level,
                           ms->N, ms->V,
                           ms->Range, ms->MeshMode, mesh_skip, ms->AltLevel);
 
             if(!SettingGet_b
-               (I->G, I->Setting, NULL, cSetting_mesh_negative_visible)) {
+               (I->G, I->Setting.get(), NULL, cSetting_mesh_negative_visible)) {
               ms->base_n_V = VLAGetSize(ms->V);
-            } else if(ms->MeshMode != 3) {
+            } else if(ms->MeshMode != cIsomeshMode::gradient) {
               /* do we want the negative surface too? */
 
               pymol::vla<int> N2(10000);
               pymol::vla<float> V2(10000);
 
-              IsosurfVolume(I->G, I->Setting, NULL,
+              IsosurfVolume(I->G, I->Setting.get(), NULL,
                             field,
                             -ms->Level,
                             N2, V2, ms->Range, ms->MeshMode, mesh_skip, ms->AltLevel);
@@ -633,20 +628,10 @@ void ObjectMesh::update()
 
           }
           if(ms->CarveFlag && ms->AtomVertex && VLAGetSize(ms->N) && VLAGetSize(ms->V)) {
-            carve_buffer = ms->CarveBuffer;
-            if(ms->CarveBuffer < 0.0F) {
-              avoid_flag = true;
-              carve_buffer = -carve_buffer;
-            }
-
             /* cull my friend, cull */
-            voxelmap = MapNew(I->G,
-                              -carve_buffer, ms->AtomVertex,
-                              VLAGetSize(ms->AtomVertex) / 3, NULL);
-            if(voxelmap) {
-
-              MapSetupExpress(voxelmap);
-
+            auto carvehelper = CarveHelper(G, ms->CarveBuffer, ms->AtomVertex,
+                VLAGetSize(ms->AtomVertex) / 3);
+            {
               pymol::vla<int> old_n = std::move(ms->N);
               pymol::vla<float> old_v = std::move(ms->V);
               ms->N = pymol::vla<int>(old_n.size());
@@ -661,21 +646,7 @@ void ObjectMesh::update()
                 last_flag = false;
                 c = *(n++);
                 while(c--) {
-                  flag = false;
-                  MapLocus(voxelmap, v, &h, &k, &l);
-                  i = *(MapEStart(voxelmap, h, k, l));
-                  if(i) {
-                    j = voxelmap->EList[i++];
-                    while(j >= 0) {
-                      if(within3f(ms->AtomVertex + 3 * j, v, carve_buffer)) {
-                        flag = true;
-                        break;
-                      }
-                      j = voxelmap->EList[i++];
-                    }
-                  }
-                  if(avoid_flag)
-                    flag = !flag;
+                  flag = !carvehelper.is_excluded(v);
                   if(flag && (!last_flag)) {
                     VLACheck(ms->V, float, 3 * (n_line + 1));
                     copy3f(v, ms->V + n_line * 3);
@@ -696,6 +667,10 @@ void ObjectMesh::update()
                   }
                   last_flag = flag;
                   v += 3;
+
+                  if (v - old_v.data() == ms->base_n_V) {
+                    ms->base_n_V = n_line * 3;
+                  }
                 }
                 if(last_flag) { /* terminate segment */
                   VLACheck(ms->N, int, n_seg);
@@ -706,7 +681,6 @@ void ObjectMesh::update()
               }
               VLACheck(ms->N, int, n_seg);
               ms->N[n_seg] = 0;
-              MapFree(voxelmap);
             }
           }
         }
@@ -773,10 +747,10 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
   int state = 0;
   CRay *ray = 0;
   bool pick = false;
-  int pass = 0;
+  RenderPass pass = RenderPass::Antialias;
   int *n = NULL;
   int c;
-  float line_width, mesh_width = SettingGet_f(I->G, I->Setting, NULL, cSetting_mesh_width);
+  float line_width, mesh_width = SettingGet_f(I->G, I->Setting.get(), NULL, cSetting_mesh_width);
   ObjectMeshState *ms = NULL;
   int ok = true;
 
@@ -792,35 +766,38 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
   line_width = SceneGetDynamicLineWidth(info, mesh_width);
   ObjectPrepareContext(I, info);
 
-  for(StateIterator iter(I->G, I->Setting, state, I->NState); iter.next();) {
-    ms = I->State + iter.state;
+  for(StateIterator iter(I->G, I->Setting.get(), state, I->NState); iter.next();) {
+    ms = &I->State[iter.state];
 
     if(!ms->Active || !ms->V || !ms->N)
       continue;
 
+    auto transparency =
+        SettingGet<float>(G, I->Setting.get(), nullptr, cSetting_transparency);
+
     {
         v = ms->V.data();
         n = ms->N.data();
         if(ok && ray) {
           if(ms->UnitCellCGO && (I->visRep & cRepCellBit)){
             ok &= CGORenderRay(ms->UnitCellCGO.get(), ray, info, ColorGet(I->G, I->Color),
-			       NULL, I->Setting, NULL);
+			       NULL, I->Setting.get(), NULL);
 	    if (!ok){
 	      ms->UnitCellCGO.reset();
 	      break;
 	    }
 	  }
-          if(ms->MeshMode != 1) {
-            radius = SettingGet_f(I->G, I->Setting, NULL, cSetting_mesh_radius);
+          if(ms->MeshMode != cIsomeshMode::isodot) {
+            radius = SettingGet_f(I->G, I->Setting.get(), NULL, cSetting_mesh_radius);
 
             if(radius == 0.0F) {
               radius = ray->PixelRadius * line_width / 2.0F;
             }
           } else {
-            radius = SettingGet_f(I->G, I->Setting, NULL, cSetting_dot_radius);
+            radius = SettingGet_f(I->G, I->Setting.get(), NULL, cSetting_dot_radius);
             if(radius == 0.0F) {
               radius =
-                ray->PixelRadius * SettingGet_f(I->G, I->Setting, NULL,
+                ray->PixelRadius * SettingGet_f(I->G, I->Setting.get(), NULL,
                                                 cSetting_dot_width) / 1.4142F;
             }
           }
@@ -831,7 +808,10 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
             ColorGetEncoded(G, ms->OneColor, cc);
             vc = ms->VC.data();
             rc = ms->RC.data();
-            if(ms->MeshMode == 1) {
+
+            ray->transparentf(transparency);
+
+            if(ms->MeshMode == cIsomeshMode::isodot) {
               ray->color3fv(cc);
               while(ok && *n) {
                 c = *(n++);
@@ -884,12 +864,15 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
             }
           }
         } else if((G->HaveGUI && G->ValidContext) || returnCGO) {
-          if(!pick && !pass) {
+          if(!pick && pass == RenderPass::Antialias) {
 	      short use_shader;
 	      short mesh_as_cylinders ;
 	      CGO *shaderCGO = NULL;
 	      use_shader = ( SettingGetGlobal_b(G, cSetting_mesh_use_shader) & SettingGetGlobal_b(G, cSetting_use_shaders)) | returnCGO;
-	      mesh_as_cylinders = SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_mesh_as_cylinders) && ms->MeshMode != 1;
+              mesh_as_cylinders =
+                  SettingGetGlobal_b(G, cSetting_render_as_cylinders) &&
+                  SettingGetGlobal_b(G, cSetting_mesh_as_cylinders) &&
+                  ms->MeshMode != cIsomeshMode::isodot;
 
 	      if (ms->shaderCGO && (!use_shader || (mesh_as_cylinders ^ ms->shaderCGO->has_draw_cylinder_buffers))){
             ms->shaderCGO.reset();
@@ -908,18 +891,20 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
 		  break;
 		}
 		shaderCGO->use_shader = true;
+                CGOAlpha(shaderCGO, 1.f - transparency);
 	      }
 
 	      if(ms->UnitCellCGO && (I->visRep & cRepCellBit)) {
 		const float *color = ColorGet(I->G, I->Color);
 		if (!use_shader) {
-		  CGORenderGL(ms->UnitCellCGO.get(), color, I->Setting, NULL, info, NULL);
+		  CGORenderGL(ms->UnitCellCGO.get(), color, I->Setting.get(), NULL, info, NULL);
 		} else if(!ms->shaderUnitCellCGO) {
 		  CGO *newUnitCellCGO = CGONewSized(G, 0);
 		  CGOColorv(newUnitCellCGO, color);
 		  CGOAppend(newUnitCellCGO, ms->UnitCellCGO.get());
-		  ms->shaderUnitCellCGO.reset(CGOOptimizeToVBONotIndexedNoShader(newUnitCellCGO, 0));
-		  CGOFree(newUnitCellCGO);
+                  ms->shaderUnitCellCGO.reset(
+                      CGOOptimizeToVBONotIndexedNoShader(newUnitCellCGO));
+                  CGOFree(newUnitCellCGO);
 		  ms->shaderUnitCellCGO->use_shader = true;
 		}
 	      }
@@ -946,9 +931,9 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
 		    ok &= CGOColorv(shaderCGO, ColorGet(I->G, ms->OneColor));
 
 		  if (!mesh_as_cylinders){
-		    if(ms->MeshMode == 1){
+		    if(ms->MeshMode == cIsomeshMode::isodot){
 		      ok &= CGODotwidth(shaderCGO, SettingGet_f
-				  (I->G, I->Setting, NULL, cSetting_dot_width));
+				  (I->G, I->Setting.get(), NULL, cSetting_dot_width));
 		    } else {
 		      ok &= CGOSpecial(shaderCGO, LINEWIDTH_DYNAMIC_MESH); 
 		    }
@@ -981,7 +966,7 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
 		  } else {
 		    while(ok && *n) {
 		      c = *(n++);
-		      if(ms->MeshMode == 1)
+		      if(ms->MeshMode == cIsomeshMode::isodot)
 			ok &= CGOBegin(shaderCGO, GL_POINTS);
 		      else {
 			if (c < 2){
@@ -1014,15 +999,15 @@ static CGO *ObjectMeshRenderImpl(ObjectMesh * I, RenderInfo * info, int returnCG
 
 		  if(!vc)
 		    glColor3fv(ColorGet(I->G, ms->OneColor));
-		  if(ms->MeshMode == 1){
+		  if(ms->MeshMode == cIsomeshMode::isodot){
 		    glPointSize(SettingGet_f
-				(I->G, I->Setting, NULL, cSetting_dot_width));
+				(I->G, I->Setting.get(), NULL, cSetting_dot_width));
 		  } else {
 		    glLineWidth(line_width);
 		  }
 		  while(*n) {
 		    c = *(n++);
-		    if(ms->MeshMode == 1)
+		    if(ms->MeshMode == cIsomeshMode::isodot)
 		      glBegin(GL_POINTS);
 		    else
 		      glBegin(GL_LINE_STRIP);
@@ -1111,10 +1096,9 @@ int ObjectMesh::getNFrame() const
 
 
 /*========================================================================*/
-ObjectMesh::ObjectMesh(PyMOLGlobals * G) : CObject(G)
+ObjectMesh::ObjectMesh(PyMOLGlobals * G) : pymol::CObject(G)
 {
   auto I = this;
-  I->State = pymol::vla<ObjectMeshState>(10);   /* autozero important */
   I->type = cObjectMesh;
 }
 
@@ -1128,13 +1112,12 @@ ObjectMeshState::ObjectMeshState(PyMOLGlobals* G)
   N = pymol::vla<int>(10000);
 }
 
-
 /*========================================================================*/
 ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap * map,
                                   CSymmetry * sym,
                                   int map_state,
                                   int state, float *mn, float *mx,
-                                  float level, int meshMode,
+                                  float level, cIsomeshMode meshMode,
                                   float carve, float *vert_vla,
                                   float alt_level, int quiet)
 {
@@ -1155,15 +1138,14 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
     if(state < 0)
       state = I->NState;
     if(I->NState <= state) {
-      VLACheck(I->State, ObjectMeshState, state);
-      CHECKOK(ok, I->State);
+      VecCheckEmplace(I->State, state, G);
       if (ok)
 	I->NState = state + 1;
     }
   }
 
   if (ok){
-    ms = I->State + state;
+    ms = &I->State[state];
     *ms = ObjectMeshState(G);
   }
 
@@ -1178,7 +1160,7 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
     ms->quiet = quiet;
   }
   if(ok && oms) {
-    if((meshMode == 3) && (ms->AltLevel < ms->Level)) {
+    if((meshMode == cIsomeshMode::gradient) && (ms->AltLevel < ms->Level)) {
       /* gradient object -- need to auto-set range */
       if(!ObjectMapStateGetDataRange(G, oms, &ms->Level, &ms->AltLevel)) {
         ms->Level = -1.0F;
@@ -1285,7 +1267,7 @@ ObjectMesh *ObjectMeshFromXtalSym(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap
 ObjectMesh *ObjectMeshFromBox(PyMOLGlobals * G, ObjectMesh * obj, ObjectMap * map,
                               int map_state,
                               int state, float *mn, float *mx,
-                              float level, int meshMode,
+                              float level, cIsomeshMode meshMode,
                               float carve, float *vert_vla, float alt_level, int quiet)
 {
   return ObjectMeshFromXtalSym(G, obj, map, NULL, map_state, state, mn, mx,
@@ -1302,7 +1284,7 @@ void ObjectMeshRecomputeExtent(ObjectMesh * I)
   ObjectMeshState *ms;
 
   for(a = 0; a < I->NState; a++) {
-    ms = I->State + a;
+    ms = &I->State[a];
     if(ms->Active) {
       if(ms->ExtentFlag) {
         if(!extent_flag) {
@@ -1331,7 +1313,7 @@ void ObjectMeshRecomputeExtent(ObjectMesh * I)
   }
 }
 
-CObject* ObjectMesh::clone() const
+pymol::CObject* ObjectMesh::clone() const
 {
   return new ObjectMesh(*this);
 }
