@@ -30,6 +30,7 @@ Z* -------------------------------------------------------------------
 #include "Executive_pre.h"
 #include "vla.h"
 #include "Result.h"
+#include "AtomNeighbors.h"
 
 #include "Sculpt.h"
 #include <memory>
@@ -45,16 +46,16 @@ Z* -------------------------------------------------------------------
 
 #define cUndoMask 0xF
 
-/*
+/**
  * ObjectMolecule's Bond Path (BP) Record
  */
-typedef struct ObjectMoleculeBPRec {
+struct ObjectMoleculeBPRec {
   int *dist;
   int *list;
   int n_atom;
-} ObjectMoleculeBPRec;
+};
 
-struct ObjectMolecule : public CObject {
+struct ObjectMolecule : public pymol::CObject {
 	/* array of pointers to coordinate sets; one set per state */
   pymol::vla<CoordSet*> CSet;
 	/* number of coordinate sets */
@@ -74,23 +75,27 @@ struct ObjectMolecule : public CObject {
   /* proposed, for storing uniform trajectory data more efficiently:
      int *UniformAtmToIdx, *UniformIdxToAtm;  */
   int SeleBase = 0;                 /* for internal usage by  selector & only valid during selection process */
-  CSymmetry *Symmetry = 0;
-  int *Neighbor = 0;
+  pymol::copyable_ptr<CSymmetry> Symmetry;
+#if 1
+  // legacy undo
   float *UndoCoord[cUndoMask + 1] {};
   int UndoState[cUndoMask + 1] {};
   int UndoNIndex[cUndoMask + 1] {};
   int UndoIter = 0;
-  CGO *UnitCellCGO = nullptr;
-  int BondCounter = 0;
-  int AtomCounter = 0;
+#endif
+
+private:
+  pymol::cache_ptr<int[]> Neighbor;
+
+public:
+  int const* getNeighborArray() const;
+
+  int AtomCounter = -1;
   /* not stored */
   struct CSculpt *Sculpt =  nullptr;
   int RepVisCacheValid = 0;
   int RepVisCache = 0;     /* for transient storage during updates */
 
-#ifndef _PYMOL_NO_UNDO
-#endif
-
   // for reporting available assembly ids after mmCIF loading - SUBJECT TO CHANGE
   std::shared_ptr<pymol::cif_file> m_ciffile;
   const pymol::cif_data * m_cifdata = nullptr;
@@ -112,16 +117,13 @@ struct ObjectMolecule : public CObject {
   // virtual methods
   void update() override;
   void render(RenderInfo* info) override;
-  void invalidate(int rep, int level, int state) override;
+  void invalidate(cRep_t rep, cRepInv_t level, int state) override;
   int getNFrame() const override;
-  void describeElement(int index, char* buffer) const override;
+  std::string describeElement(int index) const override;
   char* getCaption(char* ch, int len) const override;
-  CSetting **getSettingHandle(int state) override;
-  CObject* clone() const override;
-  CSymmetry const* getSymmetry(int state = 0) const override
-  {
-    return Symmetry;
-  }
+  pymol::copyable_ptr<CSetting>* getSettingHandle(int state) override;
+  pymol::CObject* clone() const override;
+  CSymmetry const* getSymmetry(int state = 0) const override;
   bool setSymmetry(CSymmetry const& symmetry, int state = 0) override;
 
 protected:
@@ -129,7 +131,7 @@ protected:
 };
 
 /* this is a record that holds information for specific types of Operatations on Molecules, eg. translation/rotation/etc */
-typedef struct ObjectMoleculeOpRec {
+struct ObjectMoleculeOpRec {
   unsigned int code;
   Vector3f v1, v2;
   int cs1, cs2;
@@ -147,21 +149,21 @@ typedef struct ObjectMoleculeOpRec {
   float ttt[16], *mat1;
   int nvv1, nvv2;
   int include_static_singletons;
-} ObjectMoleculeOpRec;
+};
 
-typedef struct {
+struct HBondCriteria{
   float maxAngle;
   float maxDistAtMaxAngle;
   float maxDistAtZero;
   float power_a, power_b;
   float factor_a, factor_b;     /* 0.5/(maxAngle^power_a), 0.5/(maxAngle^power_b)) */
   float cone_dangle;
-} HBondCriteria;
+};
 
-typedef struct {
+struct PDBScale{
   int flag[3];
   float matrix[16];
-} PDBScale;
+};
 
 enum {
   PDB_VARIANT_DEFAULT = 0,
@@ -170,7 +172,7 @@ enum {
   PDB_VARIANT_VDB,      /* VIPERdb */
 };
 
-typedef struct {
+struct PDBInfoRec{
   int variant;
   int pqr_workarounds;
   PDBScale scale;
@@ -181,7 +183,7 @@ typedef struct {
   inline bool is_pqr_file() const {
     return variant == PDB_VARIANT_PQR;
   }
-} PDBInfoRec;
+};
 
 
 /* these four letter code are left over from an 
@@ -265,23 +267,6 @@ typedef struct {
 void ObjMolPairwiseInit(ObjMolPairwise * pairwise);
 void ObjMolPairwisePurge(ObjMolPairwise * pairwise);
 
-/**
- * loop iterators for the ObjectMolecule::Neighbor array
- *
- * Arguments: (Neighbor array, const int atom-index, int neighbor-atom-or-bond-index, int used-internally)
- *
- * Example:
- * @verbatim
-   // iterate over neighbors of obj->AtomInfo[at]
-   int neighbor_at, tmp;
-   ITERNEIGHBORATOMS(obj->Neighbor, at, neighbor_at, tmp) {
-     // do something with obj->AtomInfo[neighbor_at]
-   }
-   @endverbatim
- */
-#define ITERNEIGHBORATOMS(N, a, n, i) for(i = N[a] + 1; (n = N[i]) > -1; i += 2)
-#define ITERNEIGHBORBONDS(N, a, b, i) for(i = N[a] + 1; (b = N[i + 1]), N[i] > -1; i += 2)
-
 int ObjectMoleculeGetTopNeighbor(PyMOLGlobals * G,
                                  ObjectMolecule * I, int start, int excluded);
 
@@ -304,7 +289,7 @@ int ObjectMoleculeNewFromPyList(PyMOLGlobals * G, PyObject * list,
                                 ObjectMolecule ** result);
 PyObject *ObjectMoleculeAsPyList(ObjectMolecule * I);
 pymol::Result<> ObjectMoleculeSetStateTitle(ObjectMolecule * I, int state, const char *text);
-const char *ObjectMoleculeGetStateTitle(ObjectMolecule * I, int state);
+const char *ObjectMoleculeGetStateTitle(const ObjectMolecule*, int state);
 int ObjectMoleculeCheckFullStateSelection(ObjectMolecule * I, int sele, int state);
 
 int ObjectMoleculeSetStateOrder(ObjectMolecule * I, int * order, int len);
@@ -328,7 +313,7 @@ ObjectMolecule *ObjectMoleculeLoadChemPyModel(PyMOLGlobals * G, ObjectMolecule *
 ObjectMolecule *ObjectMoleculeLoadTRJFile(PyMOLGlobals * G, ObjectMolecule * obj,
                                           const char *fname, int frame, int interval,
                                           int average, int start, int stop, int max,
-                                          const char *sele, int image, float *shift, int quiet);
+                                          const char *sele, int image, const float *shift, int quiet);
 
 ObjectMolecule *ObjectMoleculeLoadRSTFile(PyMOLGlobals * G, ObjectMolecule * obj,
                                           const char *fname, int frame, int quiet, char mode);
@@ -359,23 +344,23 @@ void ObjectMoleculeInvalidateAtomType(ObjectMolecule *I, int state);
 
 void ObjectMoleculeRenderSele(ObjectMolecule * I, int curState, int sele, int vis_only SELINDICATORARG);
 
-void ObjectMoleculeSeleOp(ObjectMolecule * I, int sele, ObjectMoleculeOpRec * op);
+bool ObjectMoleculeSeleOp(ObjectMolecule * I, int sele, ObjectMoleculeOpRec * op);
 
 int ObjectMoleculeMerge(ObjectMolecule * I, pymol::vla<AtomInfoType>&& ai,
 			struct CoordSet *cs, int bondSearchFlag,
 			int aic_mask, int invalidate);
 void ObjectMoleculeUpdateNonbonded(ObjectMolecule * I);
-int ObjectMoleculeUpdateNeighbors(ObjectMolecule * I);
 int ObjectMoleculeMoveAtom(ObjectMolecule * I, int state, int index, const float *v, int mode,
                            int log);
 int ObjectMoleculeMoveAtomLabel(ObjectMolecule * I, int state, int index, float *v, int log, float *diff);
-int ObjectMoleculeGetAtomVertex(ObjectMolecule * I, int state, int index, float *v);
-int ObjectMoleculeGetAtomTxfVertex(ObjectMolecule * I, int state, int index, float *v);
-int ObjectMoleculeGetAtomIndex(ObjectMolecule * I, int sele);
+int ObjectMoleculeGetAtomVertex(const ObjectMolecule *, int state, int index, float *v);
+int ObjectMoleculeGetAtomTxfVertex(const ObjectMolecule*, int state, int index, float *v);
+int ObjectMoleculeGetAtomIndex(const ObjectMolecule*, SelectorID_t sele);
 int ObjectMoleculeTransformSelection(ObjectMolecule * I, int state,
                                      int sele, const float *TTT, int log,
                                      const char *sname, int homogenous, int global);
-int ObjectMoleculeDoesAtomNeighborSele(ObjectMolecule * I, int index, int sele);
+bool ObjectMoleculeIsAtomBondedToSele(
+    const ObjectMolecule* I, int atm, SelectorID_t sele);
 void ObjectMoleculeInferChemFromNeighGeom(ObjectMolecule * I, int state);
 void ObjectMoleculeInferChemForProtein(ObjectMolecule * I, int state);
 void ObjectMoleculeInferChemFromBonds(ObjectMolecule * I, int state);
@@ -386,7 +371,8 @@ int ObjectMoleculeXferValences(ObjectMolecule * Ia, int sele1, int sele2,
                                int source_state, int quiet);
 void ObjectMoleculeGuessValences(ObjectMolecule * I, int state, int *flag1, int *flag2,
                                  int reset);
-int ObjectMoleculeAddBond(ObjectMolecule * I, int sele0, int sele1, int order);
+int ObjectMoleculeAddBond(ObjectMolecule * I, int sele0, int sele1, int order,
+    pymol::zstring_view symop = "");
 pymol::Result<> ObjectMoleculeAddBondByIndices(
     ObjectMolecule* I, unsigned atm1, unsigned atm2, int order);
 int ObjectMoleculeRemoveBonds(ObjectMolecule * I, int sele1, int sele2);
@@ -401,26 +387,23 @@ int ObjectMoleculePreposReplAtom(ObjectMolecule * I, int index, AtomInfoType * a
 void ObjectMoleculeCreateSpheroid(ObjectMolecule * I, int average);
 int ObjectMoleculeSetAtomVertex(ObjectMolecule * I, int state, int index, float *v);
 int ObjectMoleculeVerifyChemistry(ObjectMolecule * I, int state);
-int ObjectMoleculeFindOpenValenceVector(ObjectMolecule * I, int state,
-                                        int index, float *v, float *seek,
-                                        int ignore_index);
 int ObjectMoleculeFillOpenValences(ObjectMolecule * I, int index);
-int ObjectMoleculeGetTotalAtomValence(ObjectMolecule * I, int atom);
 int ObjectMoleculeAdjustBonds(ObjectMolecule * I, int sele0, int sele1, int mode,
-                              int order);
+                              int order, pymol::zstring_view symop = "");
 int ObjectMoleculeAttach(ObjectMolecule * I, int index,
     pymol::vla<AtomInfoType>&& nai);
-int ObjectMoleculeFuse(ObjectMolecule * I, int index0, ObjectMolecule * src, int index1,
-                        int mode, int move_flag);
+pymol::Result<> ObjectMoleculeFuse(ObjectMolecule* I, int index0,
+    const ObjectMolecule* src, int index1, bool create_bond, bool move_flag);
 int ObjectMoleculeRenameAtoms(ObjectMolecule * I, int *flag, int force);
 int ObjectMoleculeAreAtomsBonded(ObjectMolecule * I, int i0, int i1);
-void ObjectGotoState(CObject* I, int state);
+void ObjectGotoState(pymol::CObject* I, int state);
 float ObjectMoleculeGetAvgHBondVector(ObjectMolecule * I, int atom, int state, float *v,
                                       float *incoming);
 int ObjectMoleculeCheckBondSep(ObjectMolecule * I, int a0, int a1, int dist);
 int ObjectMoleculeGetPhiPsi(ObjectMolecule * I, int ca, float *phi, float *psi,
                             int state);
 void ObjectMoleculeGetAtomSele(const ObjectMolecule * I, int index, char *buffer);
+std::string ObjectMoleculeGetAtomSele(const ObjectMolecule * I, int index);
 
 /**
  * Retrives selection string of an Object Molecule's atom
@@ -430,6 +413,7 @@ void ObjectMoleculeGetAtomSele(const ObjectMolecule * I, int index, char *buffer
  */
 std::string ObjectMoleculeGetAtomSeleFast(const ObjectMolecule * I, int index);
 
+std::string ObjectMoleculeGetAtomSeleLog(const ObjectMolecule* I, int index, int quote);
 void ObjectMoleculeGetAtomSeleLog(const ObjectMolecule * I, int index, char *buffer, int quote);
 
 void ObjectMoleculeUpdateIDNumbers(ObjectMolecule * I);
@@ -449,9 +433,10 @@ int ***ObjectMoleculeGetBondPrint(ObjectMolecule * I, int max_bond, int max_type
                                   int *dim);
 
 bool ObjectMoleculeConnect(ObjectMolecule* I, CoordSet* cs,
-    bool searchFlag = true, int connectModeOverride = -1);
+    bool searchFlag = true, int connectModeOverride = -1, bool pbc = false);
 bool ObjectMoleculeConnect(ObjectMolecule* I, int& nbond, pymol::vla<BondType>& bond,
-                          struct CoordSet *cs, int searchFlag, int connectModeOverride);
+                          struct CoordSet *cs, int searchFlag, int connectModeOverride,
+                          bool pbc = false);
 int ObjectMoleculeSetDiscrete(PyMOLGlobals * G, ObjectMolecule * I, int discrete);
 
 float ObjectMoleculeGetMaxVDW(ObjectMolecule * I);
@@ -495,13 +480,9 @@ int ObjectMoleculeUpdateAtomTypeInfoForState(PyMOLGlobals * G, ObjectMolecule *
 int ObjectMoleculeUpdateMMStereoInfoForState(PyMOLGlobals * G, ObjectMolecule * obj, int state, int initialize=1);
 #endif
 
-#ifndef _PYMOL_NO_UNDO
-void ObjectMoleculeSetAtomBondInfoTypeOldId(PyMOLGlobals * G, ObjectMolecule * obj);
-void ObjectMoleculeSetAtomBondInfoTypeOldIdToNegOne(PyMOLGlobals * G, ObjectMolecule * obj);
+#ifdef _PYMOL_IP_PROPERTIES
 #endif
 
-void ObjectMoleculeAdjustDiscreteAtmIdx(ObjectMolecule *I, int *lookup, int nAtom);
-
 void AtomInfoSettingGenerateSideEffects(PyMOLGlobals * G, ObjectMolecule *obj, int index, int id);
 
 int *AtomInfoGetSortedIndex(PyMOLGlobals * G,
@@ -511,14 +492,9 @@ int *AtomInfoGetSortedIndex(PyMOLGlobals * G,
 
 ObjectMolecule *ObjectMoleculeReadMmtfStr(PyMOLGlobals * G, ObjectMolecule * I,
     const char *st, int st_len, int frame, int discrete, int quiet, int multiplex, int zoom);
-ObjectMolecule *ObjectMoleculeReadCifStr(PyMOLGlobals * G, ObjectMolecule * I,
+pymol::Result<ObjectMolecule*> ObjectMoleculeReadCifStr(PyMOLGlobals * G, ObjectMolecule * I,
     const char *st, int frame, int discrete, int quiet, int multiplex, int zoom);
 
-// object and object-state level setting
-template <typename V> void SettingSet(int index, V value, ObjectMolecule * I, int state=-1) {
-  SettingSet(index, value, (CObject*)I, state);
-}
-
 std::unique_ptr<int[]> LoadTrajSeleHelper(
     const ObjectMolecule* obj, CoordSet* cs, const char* selection);
 
