@@ -203,7 +203,7 @@ PyObject *ObjectSliceAsPyList(ObjectSlice * I)
   return (PConvAutoNone(result));
 }
 
-void ObjectSlice::invalidate(int rep, int level, int state)
+void ObjectSlice::invalidate(cRep_t rep, cRepInv_t level, int state)
 {
   int a;
   int once_flag = true;
@@ -250,12 +250,12 @@ static void ObjectSliceStateUpdate(ObjectSlice * I, ObjectSliceState * oss,
   int min[2] = { 0, 0 }, max[2] = {
   0, 0};                        /* limits of the rectangle */
   int need_normals = false;
-  int track_camera = SettingGet_b(I->G, NULL, I->Setting, cSetting_slice_track_camera);
-  float grid = SettingGet_f(I->G, NULL, I->Setting, cSetting_slice_grid);
+  int track_camera = SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_slice_track_camera);
+  float grid = SettingGet_f(I->G, NULL, I->Setting.get(), cSetting_slice_grid);
   int min_expand = 1;
 
-  if(SettingGet_b(I->G, NULL, I->Setting, cSetting_slice_dynamic_grid)) {
-    float resol = SettingGet_f(I->G, NULL, I->Setting,
+  if(SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_slice_dynamic_grid)) {
+    float resol = SettingGet_f(I->G, NULL, I->Setting.get(),
                                cSetting_slice_dynamic_grid_resolution);
     float scale = SceneGetScreenVertexScale(I->G, oss->origin);
     oss->last_scale = scale;
@@ -422,9 +422,9 @@ static void ObjectSliceStateUpdate(ObjectSlice * I, ObjectSliceState * oss,
 
   if(ok) {
 
-    if(SettingGet_b(I->G, NULL, I->Setting, cSetting_slice_height_map)) {
+    if(SettingGet_b(I->G, NULL, I->Setting.get(), cSetting_slice_height_map)) {
       float height_scale =
-        SettingGet_f(I->G, NULL, I->Setting, cSetting_slice_height_scale);
+        SettingGet_f(I->G, NULL, I->Setting.get(), cSetting_slice_height_scale);
       float *value = oss->values.data();
       float up[3], scaled[3], factor;
       int x, y;
@@ -711,7 +711,7 @@ void ObjectSliceDrag(ObjectSlice * I, int state, int mode, float *pt, float *mov
 
         multiply33f33f(mat, oss->system, oss->system);
 
-        I->invalidate(cRepSlice, cRepAll, state);
+        I->invalidate(cRepSlice, cRepInvAll, state);
         SceneInvalidate(I->G);
 
       }
@@ -728,7 +728,7 @@ void ObjectSliceDrag(ObjectSlice * I, int state, int mode, float *pt, float *mov
 
         project3f(mov, up, v1);
         add3f(v1, oss->origin, oss->origin);
-        I->invalidate(cRepSlice, cRepAll, state);
+        I->invalidate(cRepSlice, cRepInvAll, state);
         SceneInvalidate(I->G);
       }
       break;
@@ -908,13 +908,13 @@ void ObjectSlice::render(RenderInfo * info)
   int state = info->state;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  int pass = info->pass;
+  const RenderPass pass = info->pass;
   int cur_state = 0;
   float alpha;
-  int track_camera = SettingGet_b(G, NULL, I->Setting, cSetting_slice_track_camera);
-  int dynamic_grid = SettingGet_b(G, NULL, I->Setting, cSetting_slice_dynamic_grid);
+  int track_camera = SettingGet_b(G, NULL, I->Setting.get(), cSetting_slice_track_camera);
+  int dynamic_grid = SettingGet_b(G, NULL, I->Setting.get(), cSetting_slice_dynamic_grid);
   ObjectSliceState *oss = NULL;
-  int use_shaders = !track_camera && SettingGet_b(G, NULL, I->Setting, cSetting_use_shaders);
+  int use_shaders = !track_camera && SettingGet_b(G, NULL, I->Setting.get(), cSetting_use_shaders);
   // just in case, since slice uses immediate mode, but this should never happen
   G->ShaderMgr->Disable_Current_Shader();
 
@@ -972,7 +972,7 @@ void ObjectSlice::render(RenderInfo * info)
   }
 
   ObjectPrepareContext(I, info);
-  alpha = SettingGet_f(G, NULL, I->Setting, cSetting_transparency);
+  alpha = SettingGet_f(G, NULL, I->Setting.get(), cSetting_transparency);
   alpha = 1.0F - alpha;
   if(fabs(alpha - 1.0) < R_SMALL4)
     alpha = 1.0F;
@@ -1067,7 +1067,7 @@ void ObjectSlice::render(RenderInfo * info)
         } else if(G->HaveGUI && G->ValidContext) {
           if(pick) {
             if (oss->shaderCGO && (I->visRep & cRepSliceBit)){
-              CGORenderGLPicking(oss->shaderCGO.get(), info, &I->context, I->Setting, NULL);
+              CGORenderGLPicking(oss->shaderCGO.get(), info, &I->context, I->Setting.get(), NULL);
             } else {
 #ifndef PURE_OPENGL_ES_2
             Picking p;
@@ -1137,15 +1137,15 @@ void ObjectSlice::render(RenderInfo * info)
           } else {  // !pick
             int render_now = false;
             if(alpha > 0.0001) {
-              render_now = (pass == -1);
+              render_now = (pass == RenderPass::Transparent);
             } else
-              render_now = (!pass);
+              render_now = pass == RenderPass::Antialias;
 
             if(render_now) {
 	      int already_rendered = false;
 
               if (oss->shaderCGO){
-                CGORenderGL(oss->shaderCGO.get(), NULL, NULL, I->Setting, info, NULL);
+                CGORenderGL(oss->shaderCGO.get(), NULL, NULL, I->Setting.get(), info, NULL);
                 already_rendered = true;
               } else {
                 oss->shaderCGO.reset(CGONew(G));
@@ -1208,13 +1208,10 @@ void ObjectSlice::render(RenderInfo * info)
 		
                 CGOStop(oss->shaderCGO.get());
                 if (use_shaders){
-                  CGO *convertcgo = oss->shaderCGO.get();
-                  convertcgo = CGOCombineBeginEnd(oss->shaderCGO.get(), 0);
-                  oss->shaderCGO.reset(CGOOptimizeToVBONotIndexed(convertcgo, 0));
-                  oss->shaderCGO->use_shader = true;
-                  CGOFree(convertcgo);
+                  oss->shaderCGO.reset(CGOOptimizeToVBONotIndexed(oss->shaderCGO.get()));
+                  assert(oss->shaderCGO->use_shader);
                 }
-                CGORenderGL(oss->shaderCGO.get(), NULL, NULL, I->Setting, info, NULL);
+                CGORenderGL(oss->shaderCGO.get(), NULL, NULL, I->Setting.get(), info, NULL);
                 SceneInvalidatePicking(G);  // any time cgo is re-generated, needs to invalidate so
                 // pick colors can be re-assigned
               }
@@ -1255,7 +1252,7 @@ ObjectSliceState *ObjectSliceStateGetActive(ObjectSlice * I, int state)
 
 
 /*========================================================================*/
-ObjectSlice::ObjectSlice(PyMOLGlobals * G) : CObject(G)
+ObjectSlice::ObjectSlice(PyMOLGlobals * G) : pymol::CObject(G)
 {
   auto I = this;
 
@@ -1265,7 +1262,7 @@ ObjectSlice::ObjectSlice(PyMOLGlobals * G) : CObject(G)
   I->context.state = 0;
 }
 
-CObject* ObjectSlice::clone() const
+pymol::CObject* ObjectSlice::clone() const
 {
   return new ObjectSlice(*this);
 }
