@@ -324,7 +324,7 @@ int ObjectVolumeInvalidateMapName(ObjectVolume * I, const char *name, const char
   return result;
 }
 
-void ObjectVolume::invalidate(int rep, int level, int state)
+void ObjectVolume::invalidate(cRep_t rep, cRepInv_t level, int state)
 {
   auto I = this;
   int a;
@@ -357,7 +357,7 @@ void ObjectVolume::invalidate(int rep, int level, int state)
   }
 }
 
-/*
+/**
  * Get the field either from the associated map, or from vs->Field in case
  * this is a reduced or symmetry expanded volume.
  */
@@ -373,7 +373,7 @@ CField * ObjectVolumeGetField(ObjectVolume * I) {
   return ObjectVolumeStateGetField(ObjectVolumeGetActiveState(I));
 }
 
-/*
+/**
  * Get a 4x4 (incl. translation) FracToReal from corner array
  */
 static void get44FracToRealFromCorner(const float * corner, float * frac2real)
@@ -423,7 +423,7 @@ void ObjectVolume::update()
       }
 
       // data min/max/mean/stdev
-      range = SettingGet_f(I->G, I->Setting, NULL, cSetting_volume_data_range);
+      range = SettingGet_f(I->G, I->Setting.get(), NULL, cSetting_volume_data_range);
       ObjectMapStateGetHistogram(I->G, oms, 0, range, vs->min_max_mean_stdev, 0.f, 0.f);
     }
 
@@ -531,7 +531,7 @@ void ObjectVolume::update()
                 }
 
                 // 0xFF (masked) or 0 (not masked), will be 1.0 or 0.0 in shader
-                *((GLubyte*)F3p(vs->carvemask.get(), x, y, z)) = flag ? 0x0 : 0xFF;
+                vs->carvemask->get<GLubyte>(x, y, z) = flag ? 0x0 : 0xFF;
               }
             }
           }
@@ -554,7 +554,7 @@ int ObjectVolumeAddSlicePoint(float *p0, float *p1, float *zaxis, float d, float
 static
 void ObjectVolumeDrawSlice(float *points, float *tex_coords, int n_points, float *zaxis);
 
-/*
+/**
  * Converting Ramp to `count * 4` sized interpolated RGBA color
  * array. Returns allocated memory.
  * Assigns data minimum and range covered by ramp to `ramp_min` and `ramp_range`.
@@ -608,7 +608,7 @@ ok_except1:
   return NULL;
 }
 
-/*
+/**
  * Adjust alpha values in the given RGBA array (in place) by:
  *   alpha_new = 1 - exp(-alpha * factor)
  */
@@ -619,7 +619,7 @@ static void ColorsAdjustAlpha(float * colors, int count, float factor) {
   }
 }
 
-/*
+/**
  * Render bounding box
  * TODO: duplicate in other reps?
  */
@@ -717,10 +717,10 @@ void ObjectVolume::render(RenderInfo * info)
 #ifndef PURE_OPENGL_ES_2
   auto I = this;
   int state = info->state;
-  int pass = info->pass;
+  const RenderPass pass = info->pass;
   int a = 0;
   ObjectVolumeState *vs = NULL;
-  float volume_layers =  SettingGet_f(I->G, I->Setting, NULL, cSetting_volume_layers);
+  float volume_layers =  SettingGet_f(I->G, I->Setting.get(), NULL, cSetting_volume_layers);
 #ifdef _PYMOL_IP_EXTRAS
   short volume_mode = SettingGetGlobal_i(G, cSetting_volume_mode);
   short ortho = SettingGetGlobal_i(G, cSetting_ortho);
@@ -739,7 +739,7 @@ void ObjectVolume::render(RenderInfo * info)
   CShaderPrg *shaderPrg;
   bool volume_t = 0;
 
-  if(info->pick || pass != -1)
+  if(info->pick || pass != RenderPass::Transparent)
     return;
 
   if(!G->HaveGUI || !G->ValidContext)
@@ -749,7 +749,7 @@ void ObjectVolume::render(RenderInfo * info)
   if (G && !(G->ShaderMgr->ShadersPresent()))
       return;
 
-  if (info->pass < 0){
+  if (info->pass == RenderPass::Transparent){
     volume_t = SettingGetGlobal_i(G, cSetting_transparency_mode) == 3;
   }
 
@@ -775,7 +775,7 @@ void ObjectVolume::render(RenderInfo * info)
 
     PRINTFB(I->G, FB_ObjectVolume, FB_Blather)
       "ObjectVolumeRender-Msg: state=%d, pass=%d, refresh=%d, recolor=%d.\n",
-      a, pass, vs->RefreshFlag, vs->RecolorFlag ENDFB(I->G); 
+      a, static_cast<int>(pass), vs->RefreshFlag, vs->RecolorFlag ENDFB(I->G);
 
     corner = vs->Corner;
 
@@ -827,7 +827,7 @@ void ObjectVolume::render(RenderInfo * info)
         return;
       }
 
-      int volume_bit_val = SettingGet_i(G, I->Setting, NULL, cSetting_volume_bit_depth);
+      int volume_bit_val = SettingGet_i(G, I->Setting.get(), NULL, cSetting_volume_bit_depth);
       volume_bit_depth = (volume_bit_val < 17) ? tex::data_type::HALF_FLOAT : tex::data_type::FLOAT;
 
 /* BEGIN PROPRIETARY CODE SEGMENT (see disclaimer in "os_proprietary.h") */
@@ -926,6 +926,7 @@ void ObjectVolume::render(RenderInfo * info)
 	return;
       shaderPrg->Enable();
       shaderPrg->Set_Stereo_And_AnaglyphMode();
+      shaderPrg->Set_Matrices();
       shaderPrg->Set1i("volumeTex", 0);
       shaderPrg->Set1i("colorTex1D", 1);
       shaderPrg->Set1i("colorTex2D", 1);
@@ -1091,7 +1092,7 @@ int ObjectVolume::getNFrame() const
 }
 
 /*========================================================================*/
-ObjectVolume::ObjectVolume(PyMOLGlobals * G) : CObject(G)
+ObjectVolume::ObjectVolume(PyMOLGlobals * G) : pymol::CObject(G)
 {
   type = cObjectVolume;
 }
@@ -1294,7 +1295,7 @@ pymol::Result<> ObjectVolumeSetRamp(ObjectVolume * I, std::vector<float>&& ramp_
   return {};
 }
 
-CObject* ObjectVolume::clone() const
+pymol::CObject* ObjectVolume::clone() const
 {
   return new ObjectVolume(*this);
 }
