@@ -54,16 +54,16 @@ struct ObjectVolumeState : public CObjectState {
   ~ObjectVolumeState();
 };
 
-struct ObjectVolume : public CObject {
+struct ObjectVolume : public pymol::CObject {
   std::vector<ObjectVolumeState> State;
   ObjectVolume(PyMOLGlobals* G);
 
   // virtual methods
   void update() override;
   void render(RenderInfo* info) override;
-  void invalidate(int rep, int level, int state) override;
+  void invalidate(cRep_t rep, cRepInv_t level, int state) override;
   int getNFrame() const override;
-  CObject* clone() const override;
+  pymol::CObject* clone() const override;
 };
 
 ObjectVolume *ObjectVolumeFromBox(PyMOLGlobals * G, ObjectVolume * obj, ObjectMap * map,
