@@ -21,7 +21,7 @@
 #include"os_gl.h"
 
 #include"Base.h"
-#include"OOMac.h"
+#include"Err.h"
 #include"Vector.h"
 #include"ObjectMolecule.h"
 #include"RepCylBond.h"
@@ -40,11 +40,17 @@
 extern "C" void fireMemoryWarning();
 #endif
 
-typedef struct RepCylBond {
-  Rep R;
-  CGO *primitiveCGO;
-  CGO *renderCGO;
-} RepCylBond;
+struct RepCylBond : Rep {
+  using Rep::Rep;
+
+  ~RepCylBond() override;
+
+  cRep_t type() const override { return cRepCyl; }
+  void render(RenderInfo* info) override;
+
+  CGO* primitiveCGO = nullptr;
+  CGO* renderCGO = nullptr;
+};
 
 /* RepCylinder -- This function is a helper function that generates a cylinder for RepCylBond.
  *   Depending on the s1 and s2 arguments, it will either draw a bond or a half bond.  Half bonds
@@ -65,9 +71,10 @@ typedef struct RepCylBond {
  * RETURN VALUE: returns ok (whether adding operation(s) were successful
  *
  */
-static
-int RepCylinder(CGO *cgo, bool s1, bool s2, bool isRamped, float *v1, float *v2,
-                bool frontCap, bool endCap, float tube_size, float *v2color=NULL, Pickable *v2pickcolor=NULL )
+static int RepCylinder(CGO* cgo, bool s1, bool s2, bool isRamped,
+    float const* v1, float const* v2, bool frontCap, bool endCap,
+    float tube_size, float const* v2color = nullptr,
+    Pickable* v2pickcolor = nullptr)
 {
   float axis[3];
   int ok = true;
@@ -103,95 +110,76 @@ int RepCylinder(CGO *cgo, bool s1, bool s2, bool isRamped, float *v1, float *v2,
   return ok;
 }
 
-static
-void RepCylBondFree(RepCylBond * I)
+RepCylBond::~RepCylBond()
 {
+  auto I = this;
   CGOFree(I->primitiveCGO);
   CGOFree(I->renderCGO);
-  RepPurge(&I->R);
-  OOFreeP(I);
 }
 
 static int RepCylBondCGOGenerate(RepCylBond * I, RenderInfo * info)
 {
-  PyMOLGlobals *G = I->R.G;
-  int ok = true;
+  PyMOLGlobals *G = I->G;
 
-  if (ok && I->primitiveCGO){
-    ok &= CGOAppendNoStop(I->renderCGO, I->primitiveCGO);
-  }
-  if (ok){
-    CGO *convertcgo = NULL;
-    bool use_shader, shader_mode;
-    ok &= CGOStop(I->renderCGO);
-    use_shader = SettingGetGlobal_b(G, cSetting_stick_use_shader)
-                 && SettingGetGlobal_b(G, cSetting_use_shaders);
-    shader_mode = use_shader && SettingGetGlobal_b(G, cSetting_stick_as_cylinders) 
-      && SettingGetGlobal_b(G, cSetting_render_as_cylinders);
-    if (ok && shader_mode && G->ShaderMgr->ShaderPrgExists("cylinder")) { //GLSL
-      CGO *newCGO = NULL;
-      if (ok){
-        CGO *convertcgo2 = CGOOptimizeSpheresToVBONonIndexed(I->renderCGO, 0, true);
-        if (ok){
-          newCGO = CGONew(G);
-          CHECKOK(ok, newCGO);
-          ok &= CGOEnable(newCGO, GL_CYLINDER_SHADER);
-          convertcgo = CGOConvertShaderCylindersToCylinderShader(I->renderCGO,  newCGO);
-          ok &= CGOAppendNoStop(newCGO, convertcgo);
-          if (ok) ok &= CGODisable(newCGO, GL_CYLINDER_SHADER);
-          if (convertcgo2){
-            ok &= CGOAppendNoStop(newCGO, convertcgo2);
-          }
-          ok &= CGOStop(newCGO);
-        }
-        CGOFreeWithoutVBOs(convertcgo2);
-      }
-      CGOFreeWithoutVBOs(convertcgo);
-      convertcgo = newCGO;
+  const CGO* input = I->primitiveCGO;
+  assert(input);
+
+  bool const use_shader = info->use_shaders && //
+                          SettingGet<bool>(*I->cs, cSetting_stick_use_shader);
+  bool const as_cylinders =
+      use_shader && //
+      SettingGet<bool>(*I->cs, cSetting_stick_as_cylinders) &&
+      SettingGet<bool>(*I->cs, cSetting_render_as_cylinders) &&
+      G->ShaderMgr->ShaderPrgExists("cylinder");
+
+  std::unique_ptr<CGO> convertcgo;
+
+  if (as_cylinders) {
+    convertcgo.reset(CGONew(G));
+
+    CGOEnable(convertcgo.get(), GL_CYLINDER_SHADER);
+    std::unique_ptr<CGO> cylindercgo(
+        CGOConvertShaderCylindersToCylinderShader(input, convertcgo.get()));
+    convertcgo->move_append(std::move(*cylindercgo));
+    CGODisable(convertcgo.get(), GL_CYLINDER_SHADER);
+
+    std::unique_ptr<CGO> spherescgo(
+        CGOOptimizeSpheresToVBONonIndexed(input, 0, true));
+    if (spherescgo) {
+      convertcgo->move_append(std::move(*spherescgo));
+    }
+  } else {
+    std::unique_ptr<CGO> simplified(CGOSimplify(input, 0, //
+        SettingGet<int>(G, cSetting_cgo_sphere_quality),
+        SettingGet<int>(G, cSetting_stick_round_nub)));
+    p_return_val_if_fail(simplified, false);
+    if (use_shader) {
+      convertcgo.reset(CGOOptimizeToVBONotIndexed(simplified.get()));
     } else {
-      CGO *convertcgo2 = CGOSimplify(I->renderCGO, 0, SettingGet_i(G, NULL, NULL, cSetting_cgo_sphere_quality), SettingGetGlobal_i(G, cSetting_stick_round_nub));
-      CHECKOK(ok, convertcgo2);
-      if (ok){
-        convertcgo = CGOCombineBeginEnd(convertcgo2, 0);
-        CHECKOK(ok, convertcgo);
-      }
-      CGOFree(convertcgo2);
-      if (ok && use_shader){
-        convertcgo2 = convertcgo;
-        if (ok){
-          convertcgo = CGOOptimizeToVBONotIndexed(convertcgo2, 0);
-          CHECKOK(ok, convertcgo);
-        }
-        CGOFree(convertcgo2);
-      }
-    }
-    if (convertcgo!=NULL){
-      CGOFree(I->renderCGO);
-      I->renderCGO = convertcgo;
-      convertcgo = NULL;
-      CGOSetUseShader(I->renderCGO, use_shader);
+      convertcgo.reset(CGOCombineBeginEnd(simplified.get()));
     }
   }
-  return ok;
+
+  p_return_val_if_fail(convertcgo, false);
+
+  assert(!I->renderCGO);
+  I->renderCGO = convertcgo.release();
+  CGOSetUseShader(I->renderCGO, use_shader);
+
+  return true;
 }
 
-static void RepCylBondRender(RepCylBond * I, RenderInfo * info)
+void RepCylBond::render(RenderInfo * info)
 {
+  auto I = this;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  float alpha;
-  PyMOLGlobals *G = I->R.G;
-  int width, height;
+  PyMOLGlobals *G = I->G;
   int ok = true;
 
-  SceneGetWidthHeight(G, &width, &height); 
-
-  alpha = 1.f - SettingGet_f(G, I->R.cs->Setting, I->R.obj->Setting, cSetting_stick_transparency);
-  if(fabs(alpha - 1.f) < R_SMALL4)
-    alpha = 1.f;
   if(ray) {
 #ifndef _PYMOL_NO_RAY
-    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->R.cs->Setting, I->R.obj->Setting);
+    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->cs->Setting.get(), I->obj->Setting.get());
     ray->transparentf(0.0);
 #endif
   } else if(G->HaveGUI && G->ValidContext) {
@@ -208,27 +196,24 @@ static void RepCylBondRender(RepCylBond * I, RenderInfo * info)
         " RepCylBondRender: rendering pickable...\n" ENDFD;
 
       if (I->renderCGO){
-        CGORenderGLPicking(I->renderCGO, info, &I->R.context, I->R.cs->Setting, I->R.obj->Setting);
+        CGORenderGLPicking(I->renderCGO, info, &I->context, I->cs->Setting.get(), I->obj->Setting.get());
       }
     } else { /* else not pick, i.e., when rendering */
       if (!I->renderCGO){
-        I->renderCGO = CGONew(G);
-        CHECKOK(ok, I->renderCGO);
-        if (ok){
-          CGOSetUseShader(I->renderCGO, use_shader);
-        }
         ok &= RepCylBondCGOGenerate(I, info);
+        assert(I->renderCGO);
       }
-      const float *color = ColorGet(G, I->R.obj->Color);
+      const float *color = ColorGet(G, I->obj->Color);
       I->renderCGO->debug = SettingGetGlobal_i(G, cSetting_stick_debug);
-      CGORenderGL(I->renderCGO, color, NULL, NULL, info, &I->R);
+      CGORenderGL(I->renderCGO, color, NULL, NULL, info, I);
     }
   }
 }
 
-static int RepZeroOrderBond(RepCylBond *I, CGO *cgo, bool s1, bool s2, float *vv1, float *vv2,
-                            float zradius, float *rgb1, float *rgb2,
-                            unsigned int b1, unsigned int b2, int a, bool b1masked, bool b2masked)
+static int RepZeroOrderBond(RepCylBond* I, CGO* cgo, bool s1, bool s2,
+    const float* vv1, const float* vv2, float zradius, const float* rgb1,
+    const float* rgb2, unsigned int b1, unsigned int b2, int a, bool b1masked,
+    bool b2masked)
 {
   float axis[3], naxis[3];
   subtract3f(vv2, vv1, axis);
@@ -311,9 +296,9 @@ static int RepZeroOrderBond(RepCylBond *I, CGO *cgo, bool s1, bool s2, float *vv
 }
 
 static int RepValence(RepCylBond *I, CGO *cgo, bool s1, bool s2, bool isRamped,
-		      float *v1, float *v2, int *other,
+		      const float *v1, const float *v2, const int *other,
 		      int a1, int a2, const float *coord,
-		      float *color1, float *color2, int ord,
+		      const float *color1, const float *color2, int ord,
 		      float tube_size,
 		      bool fixed_r, float scale_r,
 		      Pickable pickdata[] = NULL)
@@ -563,19 +548,13 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
   bool *marked = NULL;
   float *capdrawn = NULL;
   float scale_r = 1.0F;
-  int variable_alpha = false;
   float transp, h_scale;
+  float prev_transp = -1;
   int valence_found = false;
   const float _0p9 = 0.9F;
-  float alpha;
   short shader_mode = 0;
   int ok = true;
 
-  OOAlloc(G, RepCylBond);
-  CHECKOK(ok, I);
-  if (!ok){
-    return NULL;
-  }
   PRINTFD(G, FB_RepCylBond)
     " RepCylBondNew-Debug: entered.\n" ENDFD;
   obj = cs->Obj;
@@ -593,7 +572,6 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
       b++;
     }
   if(!visFlag) {
-    OOFreeP(I);
     return (NULL);              /* skip if no sticks are visible */
   }
 
@@ -601,27 +579,20 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
   marked = pymol::calloc<bool>(obj->NAtom);
   CHECKOK(ok, marked);
   if (!ok){
-    RepCylBondFree(I);
     return NULL;
   }
 
-  valence = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_valence);
+  valence = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(), cSetting_valence);
   valence_flag = (valence != 0.0F);
 
-  alpha =
-    SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_transparency);
-  alpha = 1.0F - alpha;
-  if(fabs(alpha - 1.0) < R_SMALL4)
-    alpha = 1.0F;
-
-  stick_color = SettingGet_color(G, cs->Setting, obj->Setting, cSetting_stick_color);
-  cartoon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Setting,
+  stick_color = SettingGet_color(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_color);
+  cartoon_side_chain_helper = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(),
                                            cSetting_cartoon_side_chain_helper);
-  ribbon_side_chain_helper = SettingGet_b(G, cs->Setting, obj->Setting,
+  ribbon_side_chain_helper = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(),
                                           cSetting_ribbon_side_chain_helper);
 
-  transp = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_transparency);
-  hide_long = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_hide_long_bonds);
+  transp = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_transparency);
+  hide_long = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(), cSetting_hide_long_bonds);
 
   std::set<int> all_zero_order_bond_atoms;
   b = obj->Bond;
@@ -639,9 +610,6 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
       s2 = GET_BIT(ati2->visRep, cRepCyl);
 
       if (s1 && s2){
-        if((!variable_alpha) && AtomInfoCheckBondSetting(G, b, cSetting_stick_transparency))
-          variable_alpha = true;
-
         if (!valence_found)
           valence_found = BondSettingGetWD(G, b, cSetting_valence, valence_flag);
         
@@ -654,34 +622,23 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
     b++;
     ok &= !G->Interrupt;
   }
-  radius = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_radius);
-  half_bonds = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_half_bonds);
+  radius = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_radius);
+  half_bonds = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(), cSetting_half_bonds);
   na_mode =
-    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_cartoon_nucleic_acid_mode);
+    SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_cartoon_nucleic_acid_mode);
   int na_mode_ribbon =
-    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_ribbon_nucleic_acid_mode);
-  h_scale = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_h_scale);
+    SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_ribbon_nucleic_acid_mode);
+  h_scale = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_h_scale);
   auto valence_zero_scale =
-    SettingGet_f(G, cs->Setting, obj->Setting, cSetting_valence_zero_scale);
+    SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_valence_zero_scale);
   auto valence_zero_mode =
-    SettingGet_i(G, cs->Setting, obj->Setting, cSetting_valence_zero_mode);
+    SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_valence_zero_mode);
 
-  RepInit(G, &I->R);
-  I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepCylBondRender;
-  I->R.fFree = (void (*)(struct Rep *)) RepCylBondFree;
-  I->R.obj = (CObject *) obj;
-  I->R.cs = cs;
-  I->R.context.object = obj;
-  I->R.context.state = state;
-
-  I->renderCGO = 0;
+  auto I = new RepCylBond(cs, state);
 
   I->primitiveCGO = CGONew(G);
-  if (!variable_alpha){
-    CGOAlpha(I->primitiveCGO, alpha);
-  }
   if(ok && obj->NBond) {
-    stick_ball = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_stick_ball);
+    stick_ball = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_ball);
 
     shader_mode = SettingGetGlobal_b(G, cSetting_use_shaders) && SettingGetGlobal_b(G, cSetting_stick_as_cylinders) 
       && SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_stick_use_shader);
@@ -696,14 +653,14 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
       other = ObjectMoleculeGetPrioritizedOtherIndexList(obj, cs);
       CHECKOK(ok, other);
       if (ok){
-        fixed_radius = SettingGet_b(G, cs->Setting, obj->Setting, cSetting_stick_fixed_radius);
-        scale_r = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_valence_scale);
+        fixed_radius = SettingGet_b(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_fixed_radius);
+        scale_r = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_valence_scale);
       }
     }
 
     /* spheres for stick & balls */
-    stick_ball_ratio = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_ball_ratio);
-    stick_ball_color = SettingGet_color(G, cs->Setting, obj->Setting, cSetting_stick_ball_color);
+    stick_ball_ratio = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_ball_ratio);
+    stick_ball_color = SettingGet_color(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_ball_color);
 
     b = obj->Bond;
     for(a = 0; ok && a < obj->NBond; ++a, ++b) {
@@ -721,17 +678,10 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
         AtomInfoType *ati1 = obj->AtomInfo + b1;
         AtomInfoType *ati2 = obj->AtomInfo + b2;
         float bd_radius_full;
-	float bd_alpha;
 
         auto bd_stick_color = BondSettingGetWD(G, b, cSetting_stick_color, stick_color);
         auto bd_radius = BondSettingGetWD(G, b, cSetting_stick_radius, radius);
 
-        if(variable_alpha){
-          auto bd_transp = BondSettingGetWD(G, b, cSetting_stick_transparency, transp);
-	  bd_alpha = (1.0F - bd_transp);
-          CGOAlpha(I->primitiveCGO, bd_alpha);
-	}
-
         // version <=1.8.2 used negative stick_radius to turn on
         // stick_h_scale, which had a default of 0.4 (now: 1.0)
         if(bd_radius < 0.0F) {
@@ -761,21 +711,45 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
         } else {
           c1 = (c2 = bd_stick_color);
         }
-        float *vv1 = cs->coordPtr(a1);
-        float *vv2 = cs->coordPtr(a2);
 
         s1 = GET_BIT(ati1->visRep, cRepCyl);
         s2 = GET_BIT(ati2->visRep, cRepCyl);
 
-        if(!(s1 && s2))
-          if(!half_bonds) {
-            s1 = 0;
-            s2 = 0;
-          }
+        if (!(s1 || s2)) {
+          continue;
+        }
+
+        if (!(s1 && s2) && !half_bonds) {
+          continue;
+        }
+
+        auto const s1_before_symop = s1;
+        auto const s2_before_symop = s2;
+        int symop_pass = 0;
+
+        pymol::SymOp symop[2] = {pymol::SymOp(), b->symop_2};
+        assert(!symop[0]);
+        float vv_buf[2][3];
+        float const *vv1, *vv2;
+
+      inv_sym_bond:
+
+        vv1 = cs->coordPtrSym(a1, symop[0], vv_buf[0], symop_pass);
+        vv2 = cs->coordPtrSym(a2, symop[1], vv_buf[1], symop_pass);
+
+        if (!vv1 || !vv2) {
+          PRINTFB(G, FB_RepCylBond, FB_Warnings)
+          " %s-Warning: Failed to get symmetry coordiantes\n",
+              __func__ ENDFB(G);
+          continue;
+        }
+
+        // show half-bond for atom which connects to a symmetry mate
+        s1 = s1_before_symop && !symop[0];
+        s2 = s2_before_symop && !symop[1];
 
         if(hide_long && (s1 || s2)) {
           float cutoff = (ati1->vdw + ati2->vdw) * _0p9;
-          ai1 = obj->AtomInfo + b1;
           if(!within3f(vv1, vv2, cutoff))       /* atoms separated by more than 90% of the sum of their vdw radii */
             s1 = s2 = 0;
         }
@@ -807,7 +781,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
 
           /* This means that if stick_ball gets changed, the RepCylBond needs to be completely invalidated */
 
-        auto stick_ball_impl = [&](AtomInfoType * ati1, int b1, int c1, float * vv1) {
+        auto stick_ball_impl = [&](AtomInfoType * ati1, int b1, int c1, float const* vv1) {
           int stick_ball_1 = AtomSettingGetWD(G, ati1, cSetting_stick_ball, stick_ball);
           if(stick_ball_1) {
             float vdw = stick_ball_ratio * ((ati1->protons == cAN_H) ? bd_radius : bd_radius_full);
@@ -829,28 +803,22 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
           }
         };
 
-        if (s1) stick_ball_impl(ati1, b1, c1, vv1);
-        if (s2) stick_ball_impl(ati2, b2, c2, vv2);
-
-	{
-	  float alp;
-	  if((alpha == 1.0) && (!variable_alpha)) {
-	    alp = 1.0F;
-	  } else if(variable_alpha) {
-	    alp = bd_alpha;
-	  } else {
-	    alp = alpha;
-	  }
-          ok &= CGOAlpha(I->primitiveCGO, alp);
-	}
-
-        if(hide_long && (s1 || s2)) {
-          float cutoff = (ati1->vdw + ati2->vdw) * _0p9;
-          ai1 = obj->AtomInfo + b1;
-          if(!within3f(vv1, vv2, cutoff))       /* atoms separated by more than 90% of the sum of their vdw radii */
-            s1 = s2 = 0;
-        }
         if(s1 || s2) {
+          auto const bd_transp =
+              BondSettingGetWD(G, b, cSetting_stick_transparency, transp);
+
+          if (prev_transp != bd_transp) {
+            prev_transp = bd_transp;
+            CGOAlpha(I->primitiveCGO, 1.0F - bd_transp);
+
+            if (bd_transp > 0) {
+              I->setHasTransparency();
+            }
+          }
+
+          if (s1) stick_ball_impl(ati1, b1, c1, vv1);
+          if (s2) stick_ball_impl(ati2, b2, c2, vv2);
+
           float rgb1[3], rgb2[3];
           bool isRamped = false;
           isRamped = ColorGetCheckRamped(G, c1, vv1, rgb1, state);
@@ -886,8 +854,8 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
               if (ok){
                 Pickable pickdata = { b2, ati2->masked ? cPickableNoPick : a };
 
-                bool drawcap1 = bd_radius > capdrawn[b1];
-                bool drawcap2 = bd_radius > capdrawn[b2];
+                bool drawcap1 = bd_radius > capdrawn[b1] && s1;
+                bool drawcap2 = bd_radius > capdrawn[b2] && s2;
 
                 ok &= RepCylinder(I->primitiveCGO, s1, s2, isRamped, vv1, vv2,
                     drawcap1, drawcap2, bd_radius, rgb2, &pickdata);
@@ -901,6 +869,14 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
             }
           }
         }
+
+        // If this was a half-bond to a symmetry mate, do another pass and
+        // render the other half.
+        if (symop_pass == 0 && ati1 != ati2 && symop[1]) {
+          symop_pass = 1;
+          std::swap(symop[0], symop[1]);
+          goto inv_sym_bond;
+        }
       }
     }
     /* for all zero order bond atoms that do not have any other ordered bonds,
@@ -930,7 +906,7 @@ Rep *RepCylBondNew(CoordSet * cs, int state)
 
   CGOStop(I->primitiveCGO);
   if (!ok){
-    RepCylBondFree(I);
+    delete I;
     I = NULL;
   }
 
@@ -1116,12 +1092,12 @@ void RepCylBondRenderImmediate(CoordSet * cs, RenderInfo * info)
   else {
     int active = false;
     ObjectMolecule *obj = cs->Obj;
-    int nEdge = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_stick_quality);
+    int nEdge = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_quality);
     float radius =
-      fabs(SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_radius));
+      fabs(SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_radius));
     float overlap =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_overlap);
-    float nub = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_stick_nub);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_overlap);
+    float nub = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_stick_nub);
     float overlap_r = radius * overlap;
     float nub_r = radius * nub;
 
