@@ -29,22 +29,25 @@ Z* -------------------------------------------------------------------
 #include"ShaderMgr.h"
 #include"CGO.h"
 
-typedef struct RepNonbonded {
-  Rep R;
+struct RepNonbonded : Rep {
+  using Rep::Rep;
+
+  ~RepNonbonded() override;
+
+  cRep_t type() const override { return cRepNonbonded; }
+  void render(RenderInfo* info) override;
+
   CGO *primitiveCGO;
   CGO *shaderCGO;
   bool shaderCGO_has_cylinders;
-} RepNonbonded;
+};
 
 #include"ObjectMolecule.h"
 
-static
-void RepNonbondedFree(RepNonbonded * I)
+RepNonbonded::~RepNonbonded()
 {
-  CGOFree(I->primitiveCGO);
-  CGOFree(I->shaderCGO);
-  RepPurge(&I->R);
-  OOFreeP(I);
+  CGOFree(primitiveCGO);
+  CGOFree(shaderCGO);
 }
 
 void RepNonbondedRenderImmediate(CoordSet * cs, RenderInfo * info)
@@ -57,9 +60,9 @@ void RepNonbondedRenderImmediate(CoordSet * cs, RenderInfo * info)
     int active = false;
     ObjectMolecule *obj = cs->Obj;
     float line_width =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_line_width);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_line_width);
     float nonbonded_size =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_nonbonded_size);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_nonbonded_size);
 
     if(info->width_scale_flag)
       glLineWidth(line_width * info->width_scale);
@@ -114,21 +117,21 @@ void RepNonbondedRenderImmediate(CoordSet * cs, RenderInfo * info)
 
 static int RepNonbondedCGOGenerate(RepNonbonded * I, RenderInfo * info)
 {
-  PyMOLGlobals *G = I->R.G;
+  PyMOLGlobals *G = I->G;
   float alpha;
   int ok = true;
   CGO *convertcgo = NULL;
   short nonbonded_as_cylinders ;
   short use_shader;
   float nonbonded_size =
-    SettingGet_f(G, I->R.cs->Setting, I->R.obj->Setting, cSetting_nonbonded_size);
+    SettingGet_f(G, I->cs->Setting.get(), I->obj->Setting.get(), cSetting_nonbonded_size);
 
   nonbonded_as_cylinders = SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_nonbonded_as_cylinders);
   use_shader = SettingGetGlobal_b(G, cSetting_nonbonded_use_shader) & 
     SettingGetGlobal_b(G, cSetting_use_shaders);
 
   alpha =
-    SettingGet_f(G, I->R.cs->Setting, I->R.obj->Setting, cSetting_nonbonded_transparency);
+    SettingGet_f(G, I->cs->Setting.get(), I->obj->Setting.get(), cSetting_nonbonded_transparency);
   alpha = 1.0F - alpha;
   if(fabs(alpha - 1.0) < R_SMALL4)
     alpha = 1.0F;
@@ -189,41 +192,41 @@ static int RepNonbondedCGOGenerate(RepNonbonded * I, RenderInfo * info)
   return ok;
 }
 
-static void RepNonbondedRender(RepNonbonded * I, RenderInfo * info)
+void RepNonbonded::render(RenderInfo* info)
 {
+  auto I = this;
   CRay *ray = info->ray;
   auto pick = info->pick;
-  PyMOLGlobals *G = I->R.G;
   int ok = true;
   float alpha =
-    SettingGet_f(G, I->R.cs->Setting, I->R.obj->Setting, cSetting_nonbonded_transparency);
+    SettingGet_f(G, I->cs->Setting.get(), I->obj->Setting.get(), cSetting_nonbonded_transparency);
   alpha = 1.0F - alpha;
   if(fabs(alpha - 1.0) < R_SMALL4)
     alpha = 1.0F;
   if(ray) {
 #ifndef _PYMOL_NO_RAY
-    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->R.cs->Setting, I->R.cs->Obj->Setting);
+    CGORenderRay(I->primitiveCGO, ray, info, NULL, NULL, I->cs->Setting.get(), I->cs->Obj->Setting.get());
     ray->transparentf(0.0);
 #endif
   } else if(G->HaveGUI && G->ValidContext) {
     if(pick) {
-      CGORenderGLPicking(I->shaderCGO ? I->shaderCGO : I->primitiveCGO, info, &I->R.context, I->R.cs->Setting, I->R.obj->Setting);
+      CGORenderGLPicking(I->shaderCGO ? I->shaderCGO : I->primitiveCGO, info, &I->context, I->cs->Setting.get(), I->obj->Setting.get());
     } else {
       /* not pick, but render */
       bool use_shader = SettingGetGlobal_b(G, cSetting_nonbonded_use_shader) && SettingGetGlobal_b(G, cSetting_use_shaders);
       if (!use_shader){
-        CGORenderGL(I->primitiveCGO, NULL, NULL, NULL, info, &I->R);
+        CGORenderGL(I->primitiveCGO, NULL, NULL, NULL, info, I);
         return;
       }
       bool nonbonded_as_cylinders = SettingGetGlobal_b(G, cSetting_render_as_cylinders) && SettingGetGlobal_b(G, cSetting_nonbonded_as_cylinders);
       if (I->shaderCGO && use_shader != I->shaderCGO->use_shader){
-        CGOFree(I->shaderCGO);
-        I->shaderCGO = 0;
+	CGOFree(I->shaderCGO);
+	I->shaderCGO = 0;
       }
 
       if (I->shaderCGO && (nonbonded_as_cylinders ^ I->shaderCGO_has_cylinders)){
-        CGOFree(I->shaderCGO);
-        I->shaderCGO = 0;
+	CGOFree(I->shaderCGO);
+	I->shaderCGO = 0;
       }
 
       if (!I->shaderCGO){
@@ -234,7 +237,7 @@ static void RepNonbondedRender(RepNonbonded * I, RenderInfo * info)
         }
         ok &= RepNonbondedCGOGenerate(I, info);
       }
-      CGORenderGL(I->shaderCGO, NULL, NULL, NULL, info, &I->R);
+      CGORenderGL(I->shaderCGO, NULL, NULL, NULL, info, I);
     }
   }
 }
@@ -243,7 +246,6 @@ Rep *RepNonbondedNew(CoordSet * cs, int state)
 {
   PyMOLGlobals *G = cs->G;
   bool hasNonbondedAtoms = false;
-  OOAlloc(G, RepNonbonded);
 
   ObjectMolecule *obj = cs->Obj;
 
@@ -257,21 +259,13 @@ Rep *RepNonbondedNew(CoordSet * cs, int state)
     }
   }
   if(!hasNonbondedAtoms) {
-    OOFreeP(I);
     return (NULL);              /* skip if no dots are visible */
   }
 
-  RepInit(G, &I->R);
+  auto I = new RepNonbonded(cs, state);
 
-  I->R.fRender = (void (*)(struct Rep *, RenderInfo *)) RepNonbondedRender;
-  I->R.fFree = (void (*)(struct Rep *)) RepNonbondedFree;
-  I->R.fRecolor = NULL;
   I->shaderCGO = NULL;
 
-  I->R.P = NULL;
-  I->R.obj = (CObject *) (cs->Obj);
-  I->R.cs = cs;
-
   I->primitiveCGO = CGONew(G);
 
   CGOSpecialWithArg(I->primitiveCGO, LINE_LIGHTING, 0.f);
@@ -297,7 +291,5 @@ Rep *RepNonbondedNew(CoordSet * cs, int state)
   }
   CGOEnd(I->primitiveCGO); // for immediate mode
   CGOSpecialWithArg(I->primitiveCGO, LINE_LIGHTING, 1.f);
-  I->R.context.object = obj;
-  I->R.context.state = state;
   return (Rep *) I;
 }
