@@ -27,6 +27,7 @@ Z* -------------------------------------------------------------------
 #include"Vector.h"
 #include"Word.h"
 #include"Editor.h"
+#include"Executive.h"
 #include "Lex.h"
 #include "ObjectMolecule.h"
 #include "CoordSet.h"
@@ -423,13 +424,12 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
     }
 
     ObjectMoleculeVerifyChemistry(obj, state);
-    ObjectMoleculeUpdateNeighbors(obj);
 
     cs = obj->CSet[state];
 
-    use_cache = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_memory);
+    use_cache = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_memory);
     if(obj->NBond) {
-      const int *neighbor = obj->Neighbor;
+      const int* const neighbor = obj->getNeighborArray();
       int n_atom = obj->NAtom;
 
       planar = pymol::malloc<int>(n_atom);
@@ -646,15 +646,15 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
           atl.atm2idx = obj->DiscreteAtmToIdx;
           atl.discCSet = obj->DiscreteCSet;
         } else {
-          atl.atm2idx = cs->AtmToIdx;
+          atl.atm2idx = cs->AtmToIdx.data();
           atl.discCSet = NULL;
         }
         atl.coord = cs->Coord;
         atl.neighbor = neighbor;
-        atl.min = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_tri_min);
-        atl.max = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_tri_max);
+        atl.min = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tri_min);
+        atl.max = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tri_max);
         atl.mode =
-          SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_tri_mode);
+          SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tri_mode);
 
         for(a = 0; a < n_atom; a++) {
 
@@ -682,13 +682,13 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
         int n_site = 0;
         if(cs2) {
           float minim_min =
-            SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_min_min);
+            SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_min_min);
           float minim_max =
-            SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_min_max);
+            SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_min_max);
           float maxim_min =
-            SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_max_min);
+            SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_max_min);
           float maxim_max =
-            SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_max_max);
+            SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_max_max);
 
           int *site = pymol::calloc<int>(n_atom);
           float *weight = pymol::calloc<float>(n_atom);
@@ -697,8 +697,8 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
 
           cnt.ai = obj_atomInfo;
           cnt.neighbor = neighbor;
-          cnt.atm2idx1 = cs->AtmToIdx;
-          cnt.atm2idx2 = cs2->AtmToIdx;
+          cnt.atm2idx1 = cs->AtmToIdx.data();
+          cnt.atm2idx2 = cs2->AtmToIdx.data();
 
           {
             int aa;
@@ -1323,9 +1323,9 @@ void SculptMeasureObject(CSculpt * I, ObjectMolecule * obj, int state, int match
         /* longer-range exclusions (1-5,1-6,1-7,1-8,1-9) -- only locate & store when needed */
 
         int mask =
-          SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_field_mask);
+          SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_field_mask);
         int max_excl =
-          SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_avd_excl);
+          SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_avd_excl);
         if(max_excl > 9)
           max_excl = 9;
 
@@ -1607,7 +1607,7 @@ static int SculptDoAvoid(float avoid, float range, float actual, float *d,
 }
 
 float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
-                          int state, int n_cycle, float *center)
+                          int state, int const n_cycle_arg, float *center)
 {
   PyMOLGlobals *G = I->G;
   CShaker *shk;
@@ -1663,13 +1663,14 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
   int avd_ex;
 
   PRINTFD(G, FB_Sculpt)
-    " SculptIterateObject-Debug: entered state=%d n_cycle=%d\n", state, n_cycle ENDFD;
-  if(!n_cycle)
-    n_cycle = -1;
+    " SculptIterateObject-Debug: entered state=%d n_cycle=%d\n", state, n_cycle_arg ENDFD;
 
-  auto cs = obj->getCoordSet(state);
+  for (StateIterator iter(obj, state); iter.next();) {
+    auto cs = obj->getCoordSet(iter.state);
+    if (!cs)
+      continue;
 
-  if (cs) {
+    int n_cycle = n_cycle_arg ? n_cycle_arg : -1;
 
     disp = pymol::malloc<float>(3 * obj->NAtom);
     atm2idx = pymol::malloc<int>(obj->NAtom);
@@ -1683,43 +1684,43 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
 
     cs_coord = cs->Coord.data();
 
-    vdw = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_scale);
-    vdw14 = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_scale14);
-    vdw_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_weight);
+    vdw = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_scale);
+    vdw14 = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_scale14);
+    vdw_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_weight);
     vdw_wt14 =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_weight14);
-    bond_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_bond_weight);
-    angl_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_angl_weight);
-    pyra_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_pyra_weight);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_weight14);
+    bond_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_bond_weight);
+    angl_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_angl_weight);
+    pyra_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_pyra_weight);
     pyra_inv_wt =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_pyra_inv_weight);
-    plan_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_plan_weight);
-    line_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_line_weight);
-    tri_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_tri_weight);
-    tri_sc = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_tri_scale);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_pyra_inv_weight);
+    plan_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_plan_weight);
+    line_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_line_weight);
+    tri_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tri_weight);
+    tri_sc = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tri_scale);
 
-    min_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_min_weight);
-    min_sc = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_min_scale);
-    max_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_max_weight);
-    max_sc = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_max_scale);
+    min_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_min_weight);
+    min_sc = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_min_scale);
+    max_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_max_weight);
+    max_sc = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_max_scale);
 
-    mask = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_field_mask);
+    mask = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_field_mask);
     hb_overlap =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_hb_overlap);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_hb_overlap);
     hb_overlap_base =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_hb_overlap_base);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_hb_overlap_base);
     tors_tole =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_tors_tolerance);
-    tors_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_tors_weight);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tors_tolerance);
+    tors_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_tors_weight);
     vdw_vis_mode =
-      SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_vis_mode);
+      SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_vis_mode);
     solvent_radius =
-      SettingGet_f(G, cs->Setting, obj->Setting, cSetting_solvent_radius);
+      SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_solvent_radius);
 
-    avd_wt = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_avd_weight);
-    avd_gp = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_avd_gap);
-    avd_rg = SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_avd_range);
-    avd_ex = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_avd_excl);
+    avd_wt = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_avd_weight);
+    avd_gp = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_avd_gap);
+    avd_rg = SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_avd_range);
+    avd_ex = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_avd_excl);
     if(avd_gp < 0.0F)
       avd_gp = 1.5F * solvent_radius;
     if(avd_rg < 0.0F)
@@ -1727,11 +1728,11 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
 
     if(vdw_vis_mode) {
       vdw_vis_min =
-        SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_vis_min);
+        SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_vis_min);
       vdw_vis_mid =
-        SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_vis_mid);
+        SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_vis_mid);
       vdw_vis_max =
-        SettingGet_f(G, cs->Setting, obj->Setting, cSetting_sculpt_vdw_vis_max);
+        SettingGet_f(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_vdw_vis_max);
 
       if(!cs->SculptCGO)
         cs->SculptCGO = CGONew(G);
@@ -1742,7 +1743,7 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
     }
     cgo = cs->SculptCGO;
 
-    nb_skip = SettingGet_i(G, cs->Setting, obj->Setting, cSetting_sculpt_nb_interval);
+    nb_skip = SettingGet_i(G, cs->Setting.get(), obj->Setting.get(), cSetting_sculpt_nb_interval);
     if(nb_skip > n_cycle)
       nb_skip = n_cycle;
     if(nb_skip < 0)
@@ -2373,10 +2374,11 @@ float SculptIterateObject(CSculpt * I, ObjectMolecule * obj,
         }
       }
     }
-
-    EditorDihedralInvalid(G, obj);
   }
 
+  EditorDihedralInvalid(G, obj);
+  ExecutiveUpdateCoordDepends(G, obj);
+
   PRINTFD(G, FB_Sculpt)
     " SculptIterateObject-Debug: leaving...\n" ENDFD;
 
