@@ -12,7 +12,7 @@
 
 /*========================================================================*/
 bool CoordSetAtomIterator::next() {
-  for (atm++; atm < cs->NAtIndex; atm++) {
+  for (++atm; atm < obj->NAtom; ++atm) {
     idx = cs->atmToIdx(atm);
 
     if(idx < 0)
@@ -51,7 +51,7 @@ void SeleAtomIterator::reset() {
   a = cNDummyAtoms - 1;
 }
 
-/*
+/**
  * advance the internal state to the next atom, return false if there is no
  * next atom
  */
@@ -88,13 +88,13 @@ SeleCoordIterator::SeleCoordIterator(
   statearg = state_;
 
   // current state (use -3 for "effective" state)
-  if (statearg == -2) {
+  if (statearg == cStateCurrent) {
     statearg = SettingGetGlobal_i(G, cSetting_state) - 1;
   }
 
   // safety check
-  if (statearg < -1) {
-    statearg = -3;
+  if (statearg < cStateAll) {
+    statearg = cSelectorUpdateTableEffectiveStates;
   }
 
   if (update_table) {
@@ -119,7 +119,7 @@ void SeleCoordIterator::reset() {
   }
 }
 
-/*
+/**
  * advance the internal state to the next atom, return false if there is no
  * next atom
  */
@@ -142,7 +142,8 @@ bool SeleCoordIterator::next() {
       } else if(statemax < obj->NCSet) {
         statemax = obj->NCSet;
       }
-    } else if (statearg == -3 && obj != prev_obj) {
+    } else if (statearg == cSelectorUpdateTableEffectiveStates &&
+               obj != prev_obj) {
       // "effective" state (no support here for settings all_states=1 or state=0)
       state = std::max(0, obj->getCurrentState());
       prev_obj = obj;
