@@ -50,7 +50,7 @@ struct _CEditor {
   int PickMode;                 /* 1 = atom, 2 = bond, 3 = multiatom */
   int NextPickSele;
   int BondMode;
-  CObject *DragObject;
+  pymol::CObject *DragObject;
   int NFrag;
   float V0[3], V1[3], Axis[3], Center[3], DragBase[3];
   float *PosVLA;
@@ -295,7 +295,7 @@ void EditorUpdate(PyMOLGlobals * G)
   }
 }
 
-static int EditorGetEffectiveState(PyMOLGlobals * G, CObject * obj, int state)
+static int EditorGetEffectiveState(PyMOLGlobals * G, pymol::CObject * obj, int state)
 {
   if(obj && (obj->type == cObjectMolecule)) {
     ObjectMolecule *objMol = (ObjectMolecule*)(void*)obj;
@@ -310,7 +310,7 @@ static int EditorGetEffectiveState(PyMOLGlobals * G, CObject * obj, int state)
     
     if(objMol) {
       if((objMol->NCSet == 1) && (state > 0))
-        if(SettingGet_i(G, NULL, objMol->Setting, cSetting_static_singletons))
+        if(SettingGet_i(G, NULL, objMol->Setting.get(), cSetting_static_singletons))
           return 0;
     }
   }
@@ -451,7 +451,7 @@ int EditorActive(PyMOLGlobals * G)
   return (I->Active);
 }
 
-CObject *EditorDragObject(PyMOLGlobals * G)
+pymol::CObject *EditorDragObject(PyMOLGlobals * G)
 {
   CEditor *I = G->Editor;
   return I->DragObject;
@@ -630,7 +630,6 @@ EditorInvert(PyMOLGlobals * G, int quiet)
     } else if(!(obj0 && (obj0 == obj1) && (obj0 = obj2))) {
       return pymol::Error("Must pick three atoms in the same object");
     } else {
-
       state = SceneGetState(G);
       ObjectMoleculeSaveUndo(obj0, state, false);
 
@@ -652,9 +651,9 @@ EditorInvert(PyMOLGlobals * G, int quiet)
           sprintf(name, "%s%1d", cEditorFragPref, frg);
           sele2 = SelectorIndexByName(G, name);
 
-          if(ObjectMoleculeDoesAtomNeighborSele(obj0, i0, sele2) &&
-             (!ObjectMoleculeDoesAtomNeighborSele(obj0, ia0, sele2)) &&
-             (!ObjectMoleculeDoesAtomNeighborSele(obj0, ia1, sele2))) {
+          if (ObjectMoleculeIsAtomBondedToSele(obj0, i0, sele2) &&
+              (!ObjectMoleculeIsAtomBondedToSele(obj0, ia0, sele2)) &&
+              (!ObjectMoleculeIsAtomBondedToSele(obj0, ia1, sele2))) {
             found = true;
               ObjectMoleculeTransformSelection(obj0, state, sele2, m, false, NULL, false,
                                                false);
@@ -931,8 +930,6 @@ pymol::Result<> EditorAttach(PyMOLGlobals * G, const char *elem, int geom, int v
       auto sele1 = SelectorIndexByName(G, cEditorSele2);
       auto obj0 = SelectorGetFastSingleObjectMolecule(G, sele0);
       auto obj1 = SelectorGetFastSingleObjectMolecule(G, sele1);
-#ifndef _PYMOL_NO_UNDO
-#endif
       if(obj0) {
         if(obj0->DiscreteFlag) {
           return pymol::make_error("Can't attach atoms onto discrete objects.");
@@ -965,8 +962,6 @@ pymol::Result<> EditorAttach(PyMOLGlobals * G, const char *elem, int geom, int v
       } else {
         return pymol::make_error("Invalid object.");
       }
-#ifndef _PYMOL_NO_UNDO
-#endif
     } else {
       return pymol::make_error("Invalid pk1 selection.");
     }
@@ -1058,6 +1053,7 @@ pymol::Result<> EditorHFill(PyMOLGlobals * G, int quiet)
           s2 = cEditorSele1;
           s1 = pymol::string_format("(neighbor (%s)) & hydro", s2);
 	}
+	
 	ExecutiveRemoveAtoms(G, s1.c_str(), quiet);
 	i0 = ObjectMoleculeGetAtomIndex(obj0, sele0);
 	obj0->AtomInfo[i0].chemFlag = false;
@@ -1085,7 +1081,6 @@ pymol::Result<> EditorHFix(PyMOLGlobals * G, const char *sele, int quiet)
 {
   int sele0, sele1;
   ObjectMolecule *obj0, *obj1;
-
   if((!sele) || (!sele[0])) {   /* if selection is empty, then apply to picked atoms */
     if(EditorActive(G)) {
       sele0 = SelectorIndexByName(G, cEditorSele1);
@@ -1639,7 +1634,7 @@ void EditorRender(PyMOLGlobals * G, int state)
 
         if(obj1) {
 	  /* if the user froze a state, use it instead of the global */
-	  if((frozen = SettingGetIfDefined_i(obj1->G, obj1->Setting, cSetting_state, &st))) {
+	  if((frozen = SettingGetIfDefined_i(obj1->G, obj1->Setting.get(), cSetting_state, &st))) {
 	    state = st-1;
 	  }
           if(ObjectMoleculeGetAtomTxfVertex(obj1, state, index1, vv)) {
@@ -1649,7 +1644,7 @@ void EditorRender(PyMOLGlobals * G, int state)
         }
 
         if(obj2) {
-	  if((frozen = SettingGetIfDefined_i(obj2->G, obj2->Setting, cSetting_state, &st))) {
+	  if((frozen = SettingGetIfDefined_i(obj2->G, obj2->Setting.get(), cSetting_state, &st))) {
 	    state = st-1;
 	  }
           if(ObjectMoleculeGetAtomTxfVertex(obj2, state, index2, vv)) {
@@ -1659,7 +1654,7 @@ void EditorRender(PyMOLGlobals * G, int state)
         }
 
         if(obj3) {
-	  if((frozen = SettingGetIfDefined_i(obj3->G, obj3->Setting, cSetting_state, &st))) {
+	  if((frozen = SettingGetIfDefined_i(obj3->G, obj3->Setting.get(), cSetting_state, &st))) {
 	    state = st-1;
 	  }
           if(ObjectMoleculeGetAtomTxfVertex(obj3, state, index3, vv)) {
@@ -1669,7 +1664,7 @@ void EditorRender(PyMOLGlobals * G, int state)
         }
 
         if(obj4) {
-	  if((frozen = SettingGetIfDefined_i(obj4->G, obj4->Setting, cSetting_state, &st))) {
+	  if((frozen = SettingGetIfDefined_i(obj4->G, obj4->Setting.get(), cSetting_state, &st))) {
 	    state = st-1;
 	  }
           if(ObjectMoleculeGetAtomTxfVertex(obj4, state, index4, vv)) {
@@ -1680,26 +1675,20 @@ void EditorRender(PyMOLGlobals * G, int state)
       }
     }
     if (shaderCGO){
-      CGO *convertcgo = NULL;
       int ok = true;
       CGOStop(shaderCGO);
       CHECKOK(ok, shaderCGO);
-      convertcgo = CGOCombineBeginEnd(shaderCGO, 0);
-      CHECKOK(ok, convertcgo);
-      CGOFree(shaderCGO);
       if (ok){
-	CGO *tmpCGO = CGONew(G), *convertcgo2 = NULL;
-	if (ok) ok &= CGOEnable(tmpCGO, GL_DEFAULT_SHADER);
+        CGO* tmpCGO = CGONew(G);
+        if (ok) ok &= CGOEnable(tmpCGO, GL_DEFAULT_SHADER);
 	if (ok) ok &= CGODisable(tmpCGO, GL_TWO_SIDED_LIGHTING);
-	convertcgo2 = CGOOptimizeToVBONotIndexedNoShader(convertcgo, 0);
-	if (ok) ok &= CGOAppendNoStop(tmpCGO, convertcgo2);
-	if (ok) ok &= CGODisable(tmpCGO, GL_DEFAULT_SHADER);
+        tmpCGO->free_append(CGOOptimizeToVBONotIndexedNoShader(shaderCGO));
+        if (ok) ok &= CGODisable(tmpCGO, GL_DEFAULT_SHADER);
 	if (ok) ok &= CGOStop(tmpCGO);
-	CGOFreeWithoutVBOs(convertcgo2);
 	I->shaderCGO = tmpCGO;
 	I->shaderCGO->use_shader = true;
       }
-      CGOFree(convertcgo);
+      CGOFree(shaderCGO);
       if (ok){
 	CGORenderGL(I->shaderCGO, NULL, NULL, NULL, NULL, NULL);
       }
@@ -1751,7 +1740,7 @@ void EditorInactivate(PyMOLGlobals * G)
 
 
 /*========================================================================*/
-/*
+/**
  * Create a transient distance, angle, or dihedral measurement between
  * the pk1 - pk4 atoms.
  *
@@ -1832,14 +1821,14 @@ void EditorActivate(PyMOLGlobals * G, int state, int enable_bond)
 
 
 /*========================================================================*/
-void EditorSetDrag(PyMOLGlobals * G, CObject * obj, int sele, int quiet, int state)
+void EditorSetDrag(PyMOLGlobals * G, pymol::CObject * obj, int sele, int quiet, int state)
 {
   EditorInactivate(G);
   state = EditorGetEffectiveState(G, obj, state);
   if(obj->type == cObjectMolecule) {
     ObjectMolecule *objMol = (ObjectMolecule*)(void*)obj;
     if(ObjectMoleculeCheckFullStateSelection(objMol, sele, state)) {
-      int matrix_mode = SettingGet_i(G, obj->Setting, NULL, cSetting_matrix_mode);
+      int matrix_mode = SettingGet_i(G, obj->Setting.get(), NULL, cSetting_matrix_mode);
       if(matrix_mode>=1) {
         /* force / coerce object matrix drags? */
         sele = -1;
@@ -1859,7 +1848,7 @@ void EditorReadyDrag(PyMOLGlobals * G, int state)
 
 
 /*========================================================================*/
-void EditorPrepareDrag(PyMOLGlobals * G, CObject * obj,
+void EditorPrepareDrag(PyMOLGlobals * G, pymol::CObject * obj,
                        int sele, int index, int state, int mode)
 {
   int frg;
@@ -2144,7 +2133,7 @@ int EditorDraggingObjectMatrix(PyMOLGlobals *G)
   return false;
 }
 
-void EditorDrag(PyMOLGlobals * G, CObject * obj, int index, int mode, int state,
+void EditorDrag(PyMOLGlobals * G, pymol::CObject * obj, int index, int mode, int state,
                 float *pt, float *mov, float *z_dir)
 {
   CEditor *I = G->Editor;
@@ -2172,7 +2161,7 @@ void EditorDrag(PyMOLGlobals * G, CObject * obj, int index, int mode, int state,
 
     if((index == I->DragIndex) && (obj == I->DragObject)) {
       if(!EditorActive(G)) {
-        int matrix_mode = SettingGet_i(G, I->DragObject->Setting,
+        int matrix_mode = SettingGet_i(G, I->DragObject->Setting.get(),
                                        NULL, cSetting_matrix_mode);
         if(matrix_mode<0)
           matrix_mode = EditorDraggingObjectMatrix(G) ? 1 : 0;
@@ -2395,6 +2384,26 @@ void EditorDrag(PyMOLGlobals * G, CObject * obj, int index, int mode, int state,
 
 }
 
+void EditorRemoveStale(PyMOLGlobals* G)
+{
+  if (!EditorActive(G)) {
+    return;
+  }
+
+  for (const auto& sele :
+      {cEditorSele1, cEditorSele2, cEditorSele3, cEditorSele4}) {
+    auto tarSele = SelectorIndexByName(G, sele);
+    if (tarSele > 0) {
+      int at;
+      auto obj = SelectorGetFastSingleAtomObjectIndex(G, tarSele, &at);
+      if (!obj) {
+        ExecutiveDelete(G, sele);
+      }
+    }
+  }
+
+  EditorActivate(G, -1, true);
+}
 
 /*========================================================================*/
 int EditorInit(PyMOLGlobals * G)
