@@ -26,6 +26,7 @@ Z* -------------------------------------------------------------------
 #include"ObjectMolecule.h"
 #include"Field.h"
 
+#include"ExecutiveDef.h"
 #include"Tracker.h"
 #include"Ortho.h"
 #include"Word.h"
@@ -36,6 +37,7 @@ Z* -------------------------------------------------------------------
 #include "vla.h"
 #include "TrackerList.h"
 #include "Selector.h"
+#include "SpecRecSpecial.h"
 
 enum cLoadType_t : int {
   cLoadTypeUnknown = -1,
@@ -120,6 +122,11 @@ enum cLoadType_t : int {
   // - cLoadTypeMRC                      : MRC format, ignore CCP4 features
   cLoadTypeCCP4Unspecified = 73,
   cLoadTypeMRC = 74,
+
+  cLoadTypeDXStr = 75,
+
+  cLoadTypeCCP4UnspecifiedStr = 76,
+  cLoadTypeMRCStr = 77,
 };
 
 /* NOTE: if you add new content/object type above, then be sure to add
@@ -135,7 +142,7 @@ struct ExecutiveObjectOffset{
 
 class SpecRec;
 
-/*
+/**
  * Iterator over objects (uses SpecRec list)
  */
 class ObjectIterator {
@@ -149,11 +156,10 @@ public:
 
   void reset();
   bool next();
-  CObject * getObject();
+  pymol::CObject* getObject();
   SpecRec * getSpecRec() { return rec; }
 };
 
-
 const char * ExecutiveMapGenerate(PyMOLGlobals * G, const char * name, const char * reflection_file, const char * tempFile,
 				  char * amplitudes, const char * phases, const char * weights, double reso_low,
 				  double reso_high, const char * space_group, double cell[6], int quiet, int zoom);
@@ -163,7 +169,8 @@ int ExecutiveGetExpandedGroupList(PyMOLGlobals * G, const char *name);
 int ExecutiveGetExpandedGroupListFromPattern(PyMOLGlobals * G, const char *name);
 void ExecutiveFreeGroupList(PyMOLGlobals * G, int list_id);
 
-int ExecutiveCheckGroupMembership(PyMOLGlobals * G, int list_id, CObject * obj);        /* 0.5*N for group size */
+int ExecutiveCheckGroupMembership(
+    PyMOLGlobals* G, int list_id, pymol::CObject* obj); /* 0.5*N for group size */
 
 #define cExecutiveGroupAdd 1
 #define cExecutiveGroupRemove 2
@@ -176,19 +183,26 @@ int ExecutiveCheckGroupMembership(PyMOLGlobals * G, int list_id, CObject * obj);
 #define cExecutiveGroupPurge 9
 #define cExecutiveGroupExcise 10
 
-int ExecutiveGroup(PyMOLGlobals * G, const char *name, const char *members, int action, int quiet);
+enum ExecRec_t
+{
+  cExecObject = 0,
+  cExecSelection = 1,
+  cExecAll = 2
+};
+
+int ExecutiveGroup(PyMOLGlobals * G, pymol::zstring_view name, pymol::zstring_view members, int action, int quiet);
 int ExecutiveScrollTo(PyMOLGlobals * G, const char * name, int);
 
-void ExecutiveInvalidateGroups(PyMOLGlobals * G, int force);
-void ExecutiveUpdateGroups(PyMOLGlobals * G, int force);
+void ExecutiveInvalidateGroups(PyMOLGlobals * G, bool force);
+void ExecutiveUpdateGroups(PyMOLGlobals * G, bool force);
 void ExecutiveUpdateSceneMembers(PyMOLGlobals*);
 
 int *ExecutiveGetG3d(PyMOLGlobals * G);
-int ExecutiveOrder(PyMOLGlobals * G, const char *s1, int sort, int location);
+pymol::Result<> ExecutiveOrder(PyMOLGlobals * G, pymol::zstring_view s1, int sort, int location);
 pymol::Result<> ExecutiveFixChemistry(
     PyMOLGlobals* G, const char* s1, const char* s2, int invalidate, int quiet);
 pymol::Result<std::array<float, 3>> ExecutiveGetAtomVertex(PyMOLGlobals * G, const char *s1, int state, int index);
-int ExecutiveProcessPDBFile(PyMOLGlobals * G, CObject * origObj,
+int ExecutiveProcessPDBFile(PyMOLGlobals * G, pymol::CObject * origObj,
                             const char *fname, const char *buffer, const char *oname,
                             int frame, int discrete, int finish, OrthoLineType buf,
                             int variant, int quiet,
@@ -248,7 +262,7 @@ pymol::Result<float> ExecutiveDistance(PyMOLGlobals* G, const char* nam,
     int quiet, int reset, int state, int zoom, int state1 = -4,
     int state2 = -4);
 pymol::Result<> ExecutiveBond(PyMOLGlobals* G, const char* s1,
-    const char* s2, int order, int mode, int quiet);
+    const char* s2, int order, int mode, int quiet, pymol::zstring_view symop = "");
 pymol::Result<> ExecutiveAddBondByIndices(PyMOLGlobals* G, pymol::zstring_view oname,
     unsigned int atm1, unsigned int atm2, int order);
 pymol::Result<> ExecutiveRevalence(PyMOLGlobals* G, const char* s1,
@@ -264,12 +278,21 @@ pymol::Result<int> ExecutiveIterate(PyMOLGlobals * G, const char *str1, const ch
 pymol::Result<int> ExecutiveIterateList(PyMOLGlobals* G, const char* s1,
     PyObject* list, int read_only, int quiet, PyObject* space);
 
-pymol::Result<int> ExecutiveSelect(PyMOLGlobals* G, const char* sname,
-    const char* sele, int enable, int quiet, int merge, int state,
+struct SelectArgs
+{
+  std::string sname;
+  std::string sele;
+};
+
+SelectArgs ExecutiveSelectPrepareArgs(PyMOLGlobals* G, pymol::zstring_view sname, pymol::zstring_view sele);
+
+pymol::Result<int> ExecutiveSelect(PyMOLGlobals* G, const SelectArgs& sargs,
+    int enable, int quiet, int merge, int state,
     const char* domain);
 
-int ExecutiveSelectList(PyMOLGlobals * G, const char *sele_name, const char *s1, int *list,
-                        int list_len, int state, int mode, int quiet);
+pymol::Result<int> ExecutiveSelectList(PyMOLGlobals* G, const char* sele_name,
+    const char* oname, const int* list, size_t list_len, int state, int mode,
+    int quiet);
 
 #define cExecutiveLabelEvalOff    0
 #define cExecutiveLabelEvalOn     1
@@ -281,7 +304,7 @@ int ExecutiveCountMotions(PyMOLGlobals * G);
 #ifdef _WEBGL
 #else
 pymol::Result<int> ExecutiveIterateState(PyMOLGlobals* G, int state,
-    const char* str1, const char* expr, int read_only, int atomic_props,
+    const char* str1, const char* expr, int read_only,
     int quiet, PyObject* space);
 #endif
 pymol::Result<> ExecutiveColor(
@@ -291,18 +314,20 @@ pymol::Result<> ExecutiveColorFromSele(
 int ExecutiveInit(PyMOLGlobals * G);
 void ExecutiveFree(PyMOLGlobals * G);
 int ExecutivePop(PyMOLGlobals * G, const char *target, const char *source, int quiet);
-void ExecutiveManageObject(PyMOLGlobals * G, CObject * obj, int allow_zoom, int quiet);
-void ExecutiveUpdateObjectSelection(PyMOLGlobals * G, CObject * obj);
+void ExecutiveManageObject(
+    PyMOLGlobals* G, pymol::CObject* obj, int allow_zoom, int quiet);
+void ExecutiveUpdateObjectSelection(PyMOLGlobals* G, pymol::CObject* obj);
 void ExecutiveManageSelection(PyMOLGlobals * G, const char *name);
 Block *ExecutiveGetBlock(PyMOLGlobals * G);
-CObject *ExecutiveFindObjectByName(PyMOLGlobals * G, const char *name);
+pymol::CObject* ExecutiveFindObjectByName(
+    PyMOLGlobals* G, pymol::zstring_view name);
 
 /**
  * Typed version of `ExecutiveFindObjectByName`
  * @param name Object name
  * @return NULL if object can't be found or has the wrong type
  */
-template <typename T> T* ExecutiveFindObject(PyMOLGlobals* G, const char* name)
+template <typename T> T* ExecutiveFindObject(PyMOLGlobals* G, pymol::zstring_view name)
 {
   return dynamic_cast<T*>(ExecutiveFindObjectByName(G, name));
 }
@@ -310,9 +335,25 @@ template <typename T> T* ExecutiveFindObject(PyMOLGlobals* G, const char* name)
 #define ExecutiveFindObjectMoleculeByName ExecutiveFindObject<ObjectMolecule>
 #define ExecutiveFindObjectMapByName ExecutiveFindObject<ObjectMap>
 
-CObject ** ExecutiveFindObjectsByType(PyMOLGlobals * G, int objType);
-int ExecutiveIterateObject(PyMOLGlobals * G, CObject ** obj, void **hidden);
-void ExecutiveDelete(PyMOLGlobals * G, const char *name);
+pymol::CObject** ExecutiveFindObjectsByType(PyMOLGlobals * G, int objType);
+int ExecutiveIterateObject(PyMOLGlobals * G, pymol::CObject ** obj, void **hidden);
+pymol::Result<std::vector<DiscardedRec>> ExecutiveDelete(PyMOLGlobals * G, pymol::zstring_view nameView, bool save = false);
+
+/**
+ * @brief Unregisters the specification record from PyMOL
+ * @param rec specification record to be purged/removed
+ * @save if true, the rec's associated object/cgo will not be destroyed, and the caller is responsible for repurging w/o saving.
+ */
+
+void ExecutivePurgeSpec(PyMOLGlobals* G, SpecRec* rec, bool save = false);
+
+/**
+ * @brief Readds specs to Executive
+ * @param G pointer to global pymol singletons
+ * @param specs list of specs to be readded
+ */
+
+void ExecutiveReAddSpec(PyMOLGlobals* G, std::vector<DiscardedRec>& specs);
 void ExecutiveDump(PyMOLGlobals * G, const char *fname, const char *obj, int state, int quiet);
 pymol::Result<> ExecutiveSort(PyMOLGlobals * G, const char *name);
 PyObject *ExecutiveGetBondSetting(PyMOLGlobals * G, int index, 
@@ -327,15 +368,15 @@ int ExecutiveSetBondSettingFromString(PyMOLGlobals * G,
 int ExecutiveUnsetBondSetting(PyMOLGlobals * G, int index, const char *s1, const char *s2,
                               int state, int quiet, int updates);
 
-int ExecutiveSetSetting(PyMOLGlobals * G, int index, PyObject * tuple,
-                        const char *sele, int state, int quiet, int updates);
+pymol::Result<> ExecutiveSetSetting(PyMOLGlobals * G, int index, PyObject * tuple,
+                        pymol::zstring_view preSele, int state, int quiet, int updates);
 int ExecutiveGetSettingFromString(PyMOLGlobals * G, PyMOLreturn_value *result, 
                                   int index, const char *sele,
                                   int state, int quiet);
 int ExecutiveSetSettingFromString(PyMOLGlobals * G, int index, const char *value,
                                   const char *sele, int state, int quiet, int updates);
 int ExecutiveSetObjSettingFromString(PyMOLGlobals * G,
-                                     int index, const char *value, CObject * obj,
+                                     int index, const char *value, pymol::CObject * obj,
                                      int state, int quiet, int updates);
 int ExecutiveRay(PyMOLGlobals * G, int width, int height, int mode,
                  float angle, float shift, int quiet, int defer, int antialias);
@@ -355,8 +396,8 @@ int ExecutiveRMS(PyMOLGlobals * G, const char *sele1, const char *sele2, int mod
 pymol::Result<> ExecutiveUpdateCmd(PyMOLGlobals* G, const char* sele1,
     const char* sele2, int sta1, int sta2, int method, int quiet);
 float ExecutiveRMSPairs(PyMOLGlobals* G, const std::vector<SelectorTmp>& sele, int mode, bool quiet);
-pymol::Result<pymol::vla<float>> ExecutiveRMSStates(
-    PyMOLGlobals * G, const char *s1, int target, int mode, int quiet, int mix);
+pymol::Result<pymol::vla<float>> ExecutiveRMSStates(PyMOLGlobals* G,
+    const char* s1, int target, int mode, int quiet, int mix, bool pbc = true);
 int ExecutiveIndex(PyMOLGlobals * G, const char *s1, int mode, int **indexVLA,
                    ObjectMolecule *** objVLA);
 pymol::Result<> ExecutiveReset(PyMOLGlobals*, pymol::zstring_view);
@@ -368,9 +409,19 @@ int ExecutiveDrawCmd(PyMOLGlobals * G, int width, int height, int antialias,
 pymol::Result<int> ExecutiveCartoon(PyMOLGlobals* G, int type, const char* s1);
 pymol::Result<> ExecutiveSetRepVisib(PyMOLGlobals * G, pymol::zstring_view name, int rep, int state);
 pymol::Result<> ExecutiveSetRepVisMask(PyMOLGlobals * G, pymol::zstring_view name, int repmask, int state);
+pymol::Result<> ExecutiveSetRepVisMaskFromSele(PyMOLGlobals* G, pymol::zstring_view sele, int repmask, int state);
 pymol::Result<> ExecutiveToggleRepVisib(PyMOLGlobals * G, const char *name, int rep);
 
-pymol::Result<> ExecutiveSetObjVisib(PyMOLGlobals * G, pymol::zstring_view name, int onoff, int parents);
+/**
+ * Sets the visibility of an object/selection
+ *
+ * @param name name pattern
+ * @param onoff to activate or deactivate recs referenced in name
+ * @param parents if true, also activate their parent rec
+ * @return true if any visibility has been changed.
+ */
+
+pymol::Result<bool> ExecutiveSetObjVisib(PyMOLGlobals * G, pymol::zstring_view name, int onoff, int parents);
 
 
 /**
@@ -384,14 +435,14 @@ pymol::Result<> ExecutiveSetObjVisib(PyMOLGlobals * G, pymol::zstring_view name,
  * Note: selection's center is preferred over explicit pos if both are provided
  */
 pymol::Result<> ExecutiveOrigin(PyMOLGlobals* G, const char* sele, int preserve,
-    const char* oname, float* pos, int state);
+    const char* oname, const float* pos, int state);
 pymol::Result<> ExecutiveCenter(PyMOLGlobals* G, const char* name, int state,
     int inclusive, float animate, float* pos, int quiet);
-void ExecutiveDoZoom(PyMOLGlobals * G, CObject * obj, int is_new, int zoom, int quiet);
+void ExecutiveDoZoom(PyMOLGlobals * G, pymol::CObject * obj, int is_new, int zoom, int quiet);
 pymol::Result<> ExecutiveWindowZoom(PyMOLGlobals* G,
     const char* name, float buffer, int state, int inclusive, float animate,
     int quiet);
-int ExecutiveGetMoment(PyMOLGlobals * G, const char *name, double *mi, int state);
+int ExecutiveGetMoment(PyMOLGlobals* G, const char* name, double* mi, int state);
 
 pymol::Result<std::vector<const char*>> ExecutiveGetChains(PyMOLGlobals* G, const char* sele, int state);
 
@@ -402,9 +453,9 @@ pymol::Result<> ExecutiveMove(
 char *ExecutiveNameToSeqAlignStrVLA(PyMOLGlobals * G, const char *name, int state, int format,
                                     int quiet);
 
-pymol::Result<> ExecutiveStereo(PyMOLGlobals * G, int flag);
 pymol::Result<> ExecutiveCopy(
     PyMOLGlobals* G, const char* src, const char* dst, int zoom);
+pymol::Result<> ExecutiveStereo(PyMOLGlobals * G, int flag);
 float ExecutiveOverlap(PyMOLGlobals * G, const char *s1, int state1, const char *s2, int state2,
                        float adjust);
 int ExecutiveCountStates(PyMOLGlobals * G, const char *s1);
@@ -421,7 +472,7 @@ pymol::Result<> ExecutiveSeleToObject(PyMOLGlobals* G, const char* name,
 PyObject *ExecutiveSeleToChemPyModel(PyMOLGlobals * G, const char *s1, int state,
                                      const char *ref_object, int ref_state);
 pymol::Result<> ExecutiveInvalidateRep(
-    PyMOLGlobals* G, const char* name, int rep, int level);
+    PyMOLGlobals* G, const char* name, cRep_t rep, cRepInv_t level);
 pymol::Result<> ExecutiveFlag(PyMOLGlobals * G, int flag, const char *sele, int action, int quiet);
 pymol::Result<> ExecutiveRemoveAtoms(PyMOLGlobals * G, const char *s1, int quiet);
 pymol::Result<> ExecutiveProtect(PyMOLGlobals * G, const char *s1, int mode, int quiet);
@@ -430,7 +481,7 @@ pymol::Result<> ExecutiveMask(
 void ExecutiveRebuildAll(PyMOLGlobals * G);
 pymol::Result<> ExecutiveSpheroid(PyMOLGlobals * G, const char *name, int average);
 pymol::Result<> ExecutiveAddHydrogens(PyMOLGlobals* G, const char* s1 = "(all)",
-    int quiet = 1, int state = -1, bool legacy = false);
+    int quiet = 1, int state = cStateAll, bool legacy = false);
 void ExecutiveFixHydrogens(PyMOLGlobals * G, const char *s1, int quiet);
 pymol::Result<> ExecutiveFuse(PyMOLGlobals* G, const char* s0 = "(pk1)",
     const char* s1 = "(pk2)", int mode = 0, int recolor = 1, int move_flag = 1);
@@ -439,8 +490,11 @@ pymol::Result<> ExecutiveRenameObjectAtoms(
 
 pymol::Result<std::vector<const char*>> ExecutiveGetNames(PyMOLGlobals*, int, int, const char*);
 bool ExecutiveIsMoleculeOrSelection(PyMOLGlobals * G, const char *name);
-int ExecutiveGetType(PyMOLGlobals * G, const char *name, WordType type);
-float ExecutiveGetArea(PyMOLGlobals * G, const char *s0, int sta0, int load_b);
+pymol::Result<char const*> ExecutiveGetType(PyMOLGlobals* G, const char* name);
+
+pymol::Result<float> ExecutiveGetArea(
+    PyMOLGlobals*, const char* sele, int state, bool load_b);
+
 void ExecutiveInvalidateSceneMembers(PyMOLGlobals * G);
 void ExecutiveInvalidateSelectionIndicators(PyMOLGlobals *G);
 void ExecutiveInvalidateSelectionIndicatorsCGO(PyMOLGlobals *G);
@@ -448,8 +502,8 @@ void ExecutiveRenderSelections(PyMOLGlobals * G, int curState, int slot, GridInf
 void ExecutiveHideSelections(PyMOLGlobals * G);
 pymol::Result<> ExecutiveSetTitle(PyMOLGlobals * G, const char *name, int state, const char *text);
 const char *ExecutiveGetTitle(PyMOLGlobals * G, const char *name, int state);
-void ExecutiveSetLastObjectEdited(PyMOLGlobals * G, CObject * o);
-CObject *ExecutiveGetLastObjectEdited(PyMOLGlobals * G);
+void ExecutiveSetLastObjectEdited(PyMOLGlobals * G, pymol::CObject * o);
+pymol::CObject* ExecutiveGetLastObjectEdited(PyMOLGlobals* G);
 bool ExecutiveIsFullScreen(PyMOLGlobals * G);
 void ExecutiveFullScreen(PyMOLGlobals * G, int flag);
 
@@ -481,7 +535,26 @@ pymol::Result<> ExecutiveTransformSelection(PyMOLGlobals* G,
     int state, const char* s1, int log, const float* ttt, int homogenous);
 pymol::Result<> ExecutiveTranslateAtom(
     PyMOLGlobals* G, const char* sele, const float* v, int state, int mode, int log);
-void ExecutiveSelectRect(PyMOLGlobals * G, BlockRect * rect, int mode);
+
+/**
+ * Contains both the number of selected atoms returned by SelectorCreate
+ * and the change in selected atoms compared to before the call.
+ */
+
+struct NetSelect
+{
+  int currSelected = 0;
+  int netSelected = 0;
+};
+
+/**
+ * Selects atoms via screen-space 2D rect
+ * @param rect rectangle that encloses desired atoms
+ * @param mode button mode
+ * @return number of atoms selected in active selection and change in that selected
+ */
+
+pymol::Result<NetSelect> ExecutiveSelectRect(PyMOLGlobals * G, BlockRect * rect, int mode);
 int ExecutiveMapSetBorder(PyMOLGlobals * G, const char *name, float level, int state);
 pymol::Result<> ExecutiveMapTrim(PyMOLGlobals* G, const char* name,
     const char* sele, float buffer, int map_state, int sele_state, int quiet);
@@ -491,11 +564,11 @@ pymol::Result<> ExecutiveMapHalve(PyMOLGlobals * G, const char *name, int state,
 int ExecutiveIdentifyObjects(PyMOLGlobals * G, const char *s1, int mode, int **indexVLA,
                              ObjectMolecule *** objVLA);
 pymol::Result<> ExecutiveTranslateObjectTTT(PyMOLGlobals* G,
-    const char* name, const float* trans, int store, int quiet);
+    pymol::zstring_view name, const float* trans, int store, int quiet);
 pymol::Result<> ExecutiveCombineObjectTTT(PyMOLGlobals* G,
-    const char* name, const float* ttt, int reverse_order, int store);
+    pymol::zstring_view name, const float* ttt, int reverse_order, int store);
 pymol::Result<> ExecutiveSetObjectTTT(PyMOLGlobals* G,
-    const char* name, const float* ttt, int state, int quiet, int store);
+    pymol::zstring_view name, const float* ttt, int state, int quiet, int store);
 int ExecutiveGetObjectTTT(PyMOLGlobals * G, const char *name, const float **ttt, int state,
                           int quiet);
 int ExecutiveGetObjectMatrix(PyMOLGlobals * G, const char *name, int state, double **matrix,
@@ -506,11 +579,14 @@ pymol::Result<> ExecutiveSetGeometry(
     PyMOLGlobals* G, const char* s1, int geom, int valence);
 int ExecutiveSculptIterateAll(PyMOLGlobals * G);
 pymol::Result<> ExecutiveSmooth(PyMOLGlobals* G, const char* name, int cycles,
-    int window, int first, int last, int ends, int quiet);
+    int window, int first, int last, int ends, int quiet,
+    float dist_cutoff = -1, bool pbc = true);
 int ExecutiveSculptDeactivate(PyMOLGlobals * G, const char *name);
-int ExecutiveSculptActivate(PyMOLGlobals * G, const char *name, int state=-1, int match_state=-2,
-                            int match_by_segment=0);
-float ExecutiveSculptIterate(PyMOLGlobals * G, const char *name, int state=-1, int n_cycle=10);
+int ExecutiveSculptActivate(PyMOLGlobals* G, const char* name,
+    int state = cStateAll, int match_state = cStateCurrent,
+    int match_by_segment = 0);
+float ExecutiveSculptIterate(
+    PyMOLGlobals* G, const char* name, int state = cStateAll, int n_cycle = 10);
 pymol::Result<> ExecutiveMapNew(PyMOLGlobals* G, const char* name, int type,
     float grid_spacing, const char* sele, float buffer, const float* minCorner,
     const float* maxCorner, int state, int have_corners, int quiet, int zoom,
@@ -532,8 +608,9 @@ int ExecutiveGetSession(PyMOLGlobals * G, PyObject * dict, const char *names, in
                         int quiet);
 int ExecutiveSetSession(PyMOLGlobals * G, PyObject * session, int partial_restore,
                         int quiet);
+int ExecutiveSetSessionNoMLock(PyMOLGlobals* G, PyObject* session);
 
-int ExecutiveUnsetSetting(PyMOLGlobals * G, int index, const char *sele,
+pymol::Result<> ExecutiveUnsetSetting(PyMOLGlobals * G, int index, pymol::zstring_view preSele,
                           int state, int quiet, int updates);
 
 pymol::Result<> ExecutiveAssignSS(PyMOLGlobals* G,
@@ -545,7 +622,8 @@ pymol::Result<> ExecutiveRampNew(PyMOLGlobals* G, const char* name,
     int src_state, const char* sele, float beyond, float within, float sigma,
     int zero, int calc_mode, int quiet);
 
-int ExecutiveValidateObjectPtr(PyMOLGlobals * G, CObject * ptr, int object_type);
+int ExecutiveValidateObjectPtr(
+    PyMOLGlobals* G, pymol::CObject* ptr, int object_type);
 
 /**
  * Colors atoms with a spectrum of colors based on atomic property
@@ -563,8 +641,8 @@ int ExecutiveValidateObjectPtr(PyMOLGlobals * G, CObject * ptr, int object_type)
  * @return pair of minimum and maximum of spectrum range
  */
 pymol::Result<std::pair<float, float>> ExecutiveSpectrum(PyMOLGlobals* G,
-    const char* s1, const char* expr, float min, float max, int first, int last,
-    const char* prefix, int digits, int byres, int quiet);
+    pymol::zstring_view s1, pymol::zstring_view expr, float min, float max, int first, int last,
+    pymol::zstring_view prefix, int digits, int byres, int quiet);
 
 pymol::Result<> ExecutiveReinitialize(PyMOLGlobals * G, int what, pymol::zstring_view pattern);
 const char *ExecutiveFindBestNameMatch(PyMOLGlobals * G, const char *name);
@@ -583,14 +661,28 @@ int ExecutiveIterateObjectMolecule(PyMOLGlobals * G, ObjectMolecule ** obj,
 pymol::Result<> ExecutiveSetObjectColor(PyMOLGlobals * G, const char *name, const char *color, int quiet);
 int ExecutiveGetObjectColorIndex(PyMOLGlobals * G, const char *name);
 pymol::Result<> ExecutiveSetOnOffBySele(PyMOLGlobals * G, pymol::zstring_view sname, int onoff);
-pymol::Result<> ExecutiveSetName(PyMOLGlobals * G, const char *old_name, const char *new_name);
+
+/**
+ * Changes a rec's name to another.
+ *
+ * @param old_name target rec name
+ * @param new_name rec's new name
+ * @param save discarded recs will be saved
+ * @return discarded recs if saved
+ *
+ * Note: Caller is responsible for calling ExecutivePurgeSpec on discarded recs
+ */
+
+pymol::Result<std::vector<DiscardedRec>>ExecutiveSetName(
+        PyMOLGlobals * G, pymol::zstring_view old_name, pymol::zstring_view new_name, bool save = false);
 int ExecutiveSetDrag(PyMOLGlobals * G, const char *name, int quiet,int mode);
 int ExecutiveGetActiveSeleName(PyMOLGlobals * G, char *name, int create_new, int log);
 int ExecutiveGetActiveSeleName(PyMOLGlobals * G, std::string& name, int create_new, int log);
 int ExecutiveGetActiveSele(PyMOLGlobals * G);
 int ExecutiveGetActiveAlignmentSele(PyMOLGlobals * G);
 const char* ExecutiveGetActiveAlignment(PyMOLGlobals* G);
-CObject *ExecutiveGetExistingCompatible(PyMOLGlobals * G, const char *oname, cLoadType_t type);
+pymol::CObject* ExecutiveGetExistingCompatible(
+    PyMOLGlobals* G, const char* oname, cLoadType_t type);
 pymol::Result<float> ExecutiveAngle(PyMOLGlobals* G, const char* nam,
     const char* s1, const char* s2, const char* s3, int mode, int labels,
     int reset, int zoom, int quiet, int state, int state1 = -4, int state2 = -4,
@@ -600,8 +692,8 @@ pymol::Result<float> ExecutiveDihedral(PyMOLGlobals* G, const char* nam,
     const char* s1, const char* s2, const char* s3, const char* s4, int mode,
     int labels, int reset, int zoom, int quiet, int state);
 
-int ExecutiveMatrixCopy2(PyMOLGlobals * G,
-                         CObject * source_obj, CObject * target_obj,
+int ExecutiveMatrixCopy2(PyMOLGlobals * G, pymol::CObject* source_obj,
+    pymol::CObject* target_obj,
                          int source_mode, int target_mode,
                          int source_state, int target_state,
                          int target_undo, int log, int quiet);
@@ -611,7 +703,7 @@ int ExecutiveMatrixCopy(PyMOLGlobals* G, const char* source_name,
     int target_state, int target_undo, int log, int quiet);
 
 void ExecutiveMemoryDump(PyMOLGlobals * G);
-void ExecutiveObjMolSeleOp(PyMOLGlobals * G, int sele, ObjectMoleculeOpRec * op);
+bool ExecutiveObjMolSeleOp(PyMOLGlobals * G, int sele, ObjectMoleculeOpRec * op);
 
 pymol::Result<> ExecutiveIsomeshEtc(PyMOLGlobals * G,
                         const char *mesh_name, const char *map_name, float lvl,
@@ -641,14 +733,14 @@ pymol::Result<> ExecutiveMotionViewModify(PyMOLGlobals* G, int action,
 
 void ExecutiveMotionMenuActivate(PyMOLGlobals * G, BlockRect *rect, int expected,int passive, 
                                  int x, int y, int same);
-int ExecutiveGroupMotionModify(PyMOLGlobals *G, CObject *group, int action, 
+int ExecutiveGroupMotionModify(PyMOLGlobals *G, pymol::CObject *group, int action, 
                                int index, int count, int target, int freeze);
-int ExecutiveGroupMotion(PyMOLGlobals *G, CObject *group,int action, int first,
+int ExecutiveGroupMotion(PyMOLGlobals *G, pymol::CObject *group,int action, int first,
                          int last, float power, float bias,
                          int simple, float linear, int wrap,
                          int hand, int window, int cycles, int state, int quiet);
-int ExecutiveGroupCombineTTT(PyMOLGlobals *G, CObject *group, const float *ttt, int reverse_order, int store);
-int ExecutiveGroupTranslateTTT(PyMOLGlobals *G, CObject *group, const float *v, int store);
+int ExecutiveGroupCombineTTT(PyMOLGlobals *G, pymol::CObject *group, const float *ttt, int reverse_order, int store);
+int ExecutiveGroupTranslateTTT(PyMOLGlobals *G, pymol::CObject *group, const float *v, int store);
 void ExecutiveMotionClick(PyMOLGlobals * G, BlockRect *rect, int mode, int expected, int x, int y, int nearest);
 void ExecutiveMotionTrim(PyMOLGlobals * G);
 void ExecutiveMotionExtend(PyMOLGlobals * G, int freeze);
@@ -657,11 +749,6 @@ int ExecutiveMotionView(PyMOLGlobals *G, int action, int first,
                         int simple, float linear, const char *name, int wrap,
                         int hand, int window, int cycles,
                         const char *scene_name, float scene_cut, int state, int quiet, int autogen);
-int ExecutiveIsomeshEtc(PyMOLGlobals * G,
-                        const char *volume_name, const char *map_name, float lvl,
-                        const char *sele, float fbuf, int state,
-                        float carve, int map_state, int quiet,
-                        int mesh_mode, int box_mode, float alt_lvl);
 int ExecutiveAssignAtomTypes(PyMOLGlobals * G, const char *s1, int format, int state, int quiet);
 
 PyObject * ExecutiveCEAlign(PyMOLGlobals * G, PyObject * listA, PyObject * listB, int lenA, int lenB,
@@ -684,13 +771,95 @@ int *ExecutiveGetRepsInSceneForObject(PyMOLGlobals *G, const char *name);
 int *ExecutiveGetRepsForObject(PyMOLGlobals *G, const char *name);
 #endif
 
+int ExecutiveGetNamesListFromPattern(PyMOLGlobals * G, const char *name,
+                                     int allow_partial, int expand_groups);
+
+/**
+ * Retrieves a list of candidates provided by a pattern. Similar to
+ * ExecutiveGetNamesListFromPattern but returns a managed tracker list.
+ *
+ * @param str pattern string provided by user
+ * @param allow_partial allows for partial name matching
+ * @param expand_groups members of the group candidate will be expanded onto the
+ * list.
+ * @return a managed list of candidate records
+ */
+pymol::TrackerAdapter<SpecRec> ExecutiveGetSpecRecsFromPattern(PyMOLGlobals* G,
+    pymol::zstring_view str, bool allow_partial = true,
+    bool expand_groups = true);
+
 char *ExecutiveGetObjectNames(PyMOLGlobals * G, int mode, const char *name, int enabled_only, int *numstrs);
 
 CoordSet * ExecutiveGetCoordSet(PyMOLGlobals * G, const char * name, int state, ObjectMolecule ** omp=NULL);
+pymol::Result<> ExecutiveLoadCoordset(
+    PyMOLGlobals* G, pymol::zstring_view oname, PyObject* model, int frame);
 
 void ExecutiveUndo(PyMOLGlobals * G, int dir);
 int ExecutiveSaveUndo(PyMOLGlobals * G, const char *s1, int state);
 
-pymol::Result<> ExecutiveRebond(PyMOLGlobals* G, const char* oname, int state);
+pymol::Result<> ExecutiveRebond(
+    PyMOLGlobals* G, const char* oname, int state, bool pbc = false);
 
+/**
+ * Determines whether the given name is of the Executive type (Selection or Object) provided
+ * @param name name of object or selection
+ * @param execType type to check against
+ * @return true if name is of the type execType
+ */
+
+bool ExecutiveIsSpecRecType(PyMOLGlobals* G, pymol::zstring_view name, int execType);
+
+void ExecutiveInvalidateMapDependents(
+    PyMOLGlobals* G, const char* map_name, const char* new_name = nullptr);
+pymol::Result<> ExecutiveLoadTraj(PyMOLGlobals* G, pymol::zstring_view oname,
+    pymol::zstring_view fname, int frame, int type, int interval, int average,
+    int start, int stop, int max, pymol::zstring_view str1, int image,
+    const float* shift, pymol::zstring_view plugin, int quiet);
+
+pymol::TrackerAdapter<SpecRec> ExecutiveGetSpecRecParents(
+    PyMOLGlobals* G, SpecRec& rec);
+SpecRec* ExecutiveFindSpec(PyMOLGlobals* G, pymol::zstring_view name_view);
+
+/**
+ * Retrives a list rec names a part of a group rec
+ * @param groupName group rec name
+ * @return space-delimited list of recs
+ */
+
+std::string ExecutiveGetGroupMemberNames(PyMOLGlobals* G, pymol::zstring_view groupName);
+
+/**
+ * Determines of the rec is of an object type
+ * @param rec target rec
+ * @param cObjectType object type enum
+ */
+
+bool ExecutiveIsObjectType(const SpecRec& rec, int cObjectType);
+
+/**
+ * Retrives order of Specification records in list
+ * @param nameList expression for recs
+ * @return rec names with their position in the Spec Rec list
+ */
+
+std::vector<OrderRec> ExecutiveGetOrderOf(PyMOLGlobals* G, pymol::zstring_view nameList);
+
+/**
+ * Sets recs in the Specification records in list at certain positions
+ * @param recs recs to be reordered in list
+ */
+
+void ExecutiveSetOrderOf(PyMOLGlobals* G, const std::vector<OrderRec>& recs);
+void ExecutiveInvalidatePanelList(PyMOLGlobals * G);
+void ExecutiveSpecSetVisibility(PyMOLGlobals * G, SpecRec * rec,
+                                int new_vis, int mod, int parents);
+
+void ExecutiveSpecSetVisibility(PyMOLGlobals * G, SpecRec * rec,
+                                int new_vis, int mod, int parents);
+
+/**
+ * Sets background color
+ * @param color new bg color
+ */
+pymol::Result<> ExecutiveBackgroundColor(PyMOLGlobals* G, pymol::zstring_view color);
 #endif
