@@ -251,6 +251,7 @@ protected:
   virtual int getMultiDefault() const = 0;
   virtual bool isExcludedBond(int atm1, int atm2);
   virtual bool isExcludedBond(const BondType * bond);
+  virtual bool excludeSymOpBonds() const { return true; }
   virtual void writeAtom() = 0;
   virtual void writeBonds() = 0;
   virtual void beginObject();
@@ -381,8 +382,8 @@ void MoleculeExporter::setRefObject(const char * ref_object, int ref_state) {
   if (!base)
     return;
 
-  if (ref_state == -1 /* all */) {
-    ref_state = -2; // current
+  if (ref_state == cStateAll) {
+    ref_state = cStateCurrent;
   }
 
   if(ObjectGetTotalMatrix(base, ref_state, true, matrix)) {
@@ -393,7 +394,7 @@ void MoleculeExporter::setRefObject(const char * ref_object, int ref_state) {
 
 void MoleculeExporter::updateMatrix(matrix_t& matrix, bool history) {
   const auto& ref = m_mat_ref.ptr;
-  if (ObjectGetTotalMatrix(reinterpret_cast<CObject*>(m_iter.obj),
+  if (ObjectGetTotalMatrix(reinterpret_cast<pymol::CObject*>(m_iter.obj),
         m_iter.state, history, matrix.storage)) {
     if (ref) {
       left_multiply44d44d(ref, matrix.storage);
@@ -421,6 +422,9 @@ void MoleculeExporter::populateBondRefs() {
     if (isExcludedBond(bond))
       continue;
 
+    if (excludeSymOpBonds() && bond->hasSymOp())
+      continue;
+
     if (id1 > id2)
       std::swap(id1, id2);
 
@@ -435,6 +439,7 @@ struct MoleculeExporterPDB : public MoleculeExporter {
   bool m_conect_all = false;
   bool m_conect_nodup;
   bool m_mdl_written = false;
+  bool m_cryst1_written = false;
   bool m_use_ter_records;
   const AtomInfoType * m_pre_ter = nullptr;
   PDBInfoRec m_pdb_info;
@@ -527,26 +532,31 @@ struct MoleculeExporterPDB : public MoleculeExporter {
   }
 
   void writeCryst1() {
-    const auto& sym = m_iter.cs->Symmetry ? m_iter.cs->Symmetry.get(): m_iter.obj->Symmetry;
+    if (m_cryst1_written) {
+      return;
+    }
+
+    const auto* sym = m_iter.cs->getSymmetry();
 
     if (sym) {
-      const auto& dim   = sym->Crystal.Dim;
-      const auto& angle = sym->Crystal.Angle;
+      const auto* dim = sym->Crystal.dims();
+      const auto* angle = sym->Crystal.angles();
       m_offset += VLAprintf(m_buffer, m_offset,
           "CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f %-11s%4d\n",
           dim[0], dim[1], dim[2], angle[0], angle[1], angle[2],
-          sym->SpaceGroup, sym->PDBZValue);
+          sym->spaceGroup(), sym->PDBZValue);
+      m_cryst1_written = true;
     }
   }
 
   void beginObject() override {
     MoleculeExporter::beginObject();
 
-    m_conect_all = SettingGet_b(G, m_iter.obj->Setting, nullptr, cSetting_pdb_conect_all);
+    m_conect_all = SettingGet_b(G, m_iter.obj->Setting.get(), nullptr, cSetting_pdb_conect_all);
 
     if (m_multi == cMolExportByObject) {
       m_offset += VLAprintf(m_buffer, m_offset, "HEADER    %.40s\n", m_iter.obj->Name);
-      writeCryst1();
+      m_cryst1_written = false;
     }
   }
 
@@ -555,9 +565,11 @@ struct MoleculeExporterPDB : public MoleculeExporter {
 
     if (m_multi == cMolExportByCoordSet) {
       m_offset += VLAprintf(m_buffer, m_offset, "HEADER    %.40s\n", getTitleOrName());
-      writeCryst1();
+      m_cryst1_written = false;
     }
 
+    writeCryst1();
+
     if (m_iter.isMultistate()
         && (m_iter.isPerObject() || m_iter.state != m_last_state)) {
       m_offset += VLAprintf(m_buffer, m_offset, "MODEL     %4d\n", m_iter.state + 1);
@@ -630,11 +642,11 @@ struct MoleculeExporterCIF : public MoleculeExporter {
   }
 
   void writeCellSymmetry() {
-    const auto& sym = m_iter.cs->Symmetry.get() ? m_iter.cs->Symmetry.get() : m_iter.obj->Symmetry;
+    const auto* sym = m_iter.cs->getSymmetry();
 
     if (sym) {
-      const auto& dim   = sym->Crystal.Dim;
-      const auto& angle = sym->Crystal.Angle;
+      const auto* dim = sym->Crystal.dims();
+      const auto* angle = sym->Crystal.angles();
       m_offset += VLAprintf(m_buffer, m_offset, "#\n"
           "_cell.entry_id %s\n"
           "_cell.length_a %.3f\n"
@@ -648,7 +660,7 @@ struct MoleculeExporterCIF : public MoleculeExporter {
           cifrepr(m_molecule_name),
           dim[0], dim[1], dim[2], angle[0], angle[1], angle[2],
           cifrepr(m_molecule_name),
-          cifrepr(sym->SpaceGroup));
+          cifrepr(sym->spaceGroup()));
     }
   }
 
@@ -886,7 +898,7 @@ struct MoleculeExporterMOL : public MoleculeExporter {
     // write bonds
     for (auto& bond : m_bonds) {
       m_offset += VLAprintf(m_buffer, m_offset, "%3d%3d%3d%3d  0  0  0\n",
-          bond.id1, bond.id2, bond.ref->order, (int) bond.ref->stereo);
+          bond.id1, bond.id2, bond.ref->order, 0);
     }
 
     m_bonds.clear();
@@ -1078,6 +1090,7 @@ struct MoleculeExporterMAE : public MoleculeExporter {
   int m_n_arom_bonds = 0;
   std::map<int, const AtomInfoType *> m_atoms;
   bool m_has_anisou;
+  bool m_has_pbc = false;
 
 #ifdef _PYMOL_MAE_PROP_EXPORT
 #endif
@@ -1146,7 +1159,7 @@ struct MoleculeExporterMAE : public MoleculeExporter {
   void beginMolecule() override {
     MoleculeExporter::beginMolecule();
 
-    std::string groupid = MaeExportGetSubGroupId(G, reinterpret_cast<CObject*>(m_iter.obj));
+    std::string groupid = MaeExportGetSubGroupId(G, reinterpret_cast<pymol::CObject*>(m_iter.obj));
 
     m_offset += VLAprintf(m_buffer, m_offset,
         "\nf_m_ct {\n"
@@ -1160,6 +1173,20 @@ struct MoleculeExporterMAE : public MoleculeExporter {
       keys.emplace_back("s_m_subgroupid");
     }
 
+    const auto* sym = m_iter.cs->getSymmetry();
+    m_has_pbc = sym && !sym->spaceGroup()[0];
+    if (m_has_pbc) {
+      keys.insert(keys.end(),
+          {"r_chorus_box_ax", "r_chorus_box_bx", "r_chorus_box_cx",
+              "r_chorus_box_ay", "r_chorus_box_by", "r_chorus_box_cy",
+              "r_chorus_box_az", "r_chorus_box_bz", "r_chorus_box_cz"});
+    } else if (sym) {
+      keys.insert(keys.end(),
+          {"r_pdb_PDB_CRYST1_a", "r_pdb_PDB_CRYST1_b", "r_pdb_PDB_CRYST1_c",
+              "r_pdb_PDB_CRYST1_alpha", "r_pdb_PDB_CRYST1_beta",
+              "r_pdb_PDB_CRYST1_gamma", "s_pdb_PDB_CRYST1_Space_Group"});
+    }
+
 #ifdef _PYMOL_MAE_PROP_EXPORT
 #endif
 
@@ -1176,6 +1203,19 @@ struct MoleculeExporterMAE : public MoleculeExporter {
       m_offset += VLAprintf(m_buffer, m_offset, "\"%s\"\n", groupid.c_str());
     }
 
+    if (m_has_pbc) {
+      const auto* f2r = sym->Crystal.fracToReal();
+      m_offset +=
+          VLAprintf(m_buffer, m_offset, "%f %f %f %f %f %f %f %f %f\n", f2r[0],
+              f2r[1], f2r[2], f2r[3], f2r[4], f2r[5], f2r[6], f2r[7], f2r[8]);
+    } else if (sym) {
+      const auto* dim = sym->Crystal.dims();
+      const auto* angle = sym->Crystal.angles();
+      m_offset += VLAprintf(m_buffer, m_offset, "%f %f %f %f %f %f %s\n",
+          dim[0], dim[1], dim[2], angle[0], angle[1], angle[2],
+          MaeExportStrRepr(sym->spaceGroup()).c_str());
+    }
+
 #ifdef _PYMOL_MAE_PROP_EXPORT
 #endif
 
@@ -1335,6 +1375,8 @@ struct MoleculeExporterMAE : public MoleculeExporter {
     ++m_n_atoms;
   }
 
+  bool excludeSymOpBonds() const override { return !m_has_pbc; }
+
   void writeBonds() override {
     // atom count
     m_n_atoms_offset += sprintf(m_buffer + m_n_atoms_offset, "m_atom[%d]", m_n_atoms);
@@ -1452,9 +1494,26 @@ public:
     return cMolExportGlobal;
   }
 
+  void writeCellSymmetry() {
+    if (!m_raw.unitCell.empty()) {
+      return;
+    }
+
+    const auto* sym = m_iter.cs->getSymmetry();
+
+    if (sym) {
+      const auto* dim = sym->Crystal.dims();
+      const auto* angle = sym->Crystal.angles();
+      m_raw.unitCell = {dim[0], dim[1], dim[2], angle[0], angle[1], angle[2]};
+      m_raw.spaceGroup = sym->spaceGroup();
+    }
+  }
+
   void beginCoordSet() override {
     m_raw.chainsPerModel.emplace_back(0);
     m_last_ai = nullptr;
+
+    writeCellSymmetry();
   }
 
   void writeAtom() override {
@@ -1581,12 +1640,12 @@ pymol::vla<char> MoleculeExporterGetStr(PyMOLGlobals * G,
 
   std::unique_ptr<MoleculeExporter> exporter;
 
-  if (ref_state < -1)
-   ref_state = state;
+  if (ref_state < cStateAll)
+    ref_state = state;
 
   // do "effective" current states
-  if (state == -2)
-    state = -3;
+  if (state == cStateCurrent)
+    state = cSelectorUpdateTableEffectiveStates;
 
   if (strcmp(format, "pdb") == 0) {
     exporter.reset(new MoleculeExporterPDB);
@@ -1791,6 +1850,8 @@ protected:
 #endif
   }
 
+  bool excludeSymOpBonds() const override { return false; }
+
   void writeBonds() override {
     if (!m_model)
       return;
@@ -1810,8 +1871,13 @@ protected:
       int index[] = { bond.id1 - 1, bond.id2 - 1 };
       PConvInt2ToPyObjAttr(bnd, "index", index);
       PConvIntToPyObjAttr(bnd, "order",     bond.ref->order);
-      PConvIntToPyObjAttr(bnd, "id",        bond.ref->id);
-      PConvIntToPyObjAttr(bnd, "stereo",    bond.ref->stereo);
+
+      // symop
+      if (bond.ref->symop_2) {
+        PConvStringToPyObjAttr(
+            bnd, "symmetry_2", bond.ref->symop_2.to_string().c_str());
+      }
+
       PyList_SetItem(bond_list, b, bnd);    /* steals bnd reference */
     }
 
@@ -1835,10 +1901,10 @@ PyObject *ExecutiveSeleToChemPyModel(PyMOLGlobals * G,
     const char *s1, int state,
     const char *ref_object, int ref_state)
 {
-  if (state == -1)
+  if (state == cStateAll)
     state = 0; // no multi-state support
 
-  if (ref_state < -1)
+  if (ref_state < cStateAll)
     ref_state = state;
 
   int sele = SelectorIndexByName(G, s1);
