@@ -13,9 +13,9 @@
 pymol::vla<char> MoleculeExporterGetStr(PyMOLGlobals * G,
     const char *format,
     const char *sele="all",
-    int state=-2, // current (-1 in Python API)
+    int state = cStateCurrent, // current (-1 in Python API)
     const char *ref_object="",
-    int ref_state=-1,
+    int ref_state = cStateAll,
     int multi=-1,
     bool quiet=true);
 
