@@ -54,7 +54,7 @@ int PlugIOManagerLoadTraj(PyMOLGlobals * G, ObjectMolecule * obj,
                           const char *fname, int frame,
                           int interval, int average, int start,
                           int stop, int max, const char *sele, int image,
-                          float *shift, int quiet, const char *plugin_type)
+                          const float *shift, int quiet, const char *plugin_type)
 {
 
   PRINTFB(G, FB_ObjectMolecule, FB_Errors)
@@ -82,7 +82,7 @@ ObjectMolecule *PlugIOManagerLoadMol(PyMOLGlobals * G, ObjectMolecule *origObj,
   return 0;
 }
 
-CObject * PlugIOManagerLoad(PyMOLGlobals * G, CObject ** obj_ptr,
+pymol::CObject * PlugIOManagerLoad(PyMOLGlobals * G, pymol::CObject ** obj_ptr,
     const char *fname, int state, int quiet, const char *plugin_type, int mask)
 {
   PRINTFB(G, FB_ObjectMolecule, FB_Errors)
@@ -152,11 +152,14 @@ static molfile_plugin_t * find_plugin(CPlugIOManager * I, const char * plugin_ty
   return NULL;
 }
 
+static CSymmetry* SymmetryNewFromTimestep(
+    PyMOLGlobals* G, molfile_timestep_t* ts);
+
 int PlugIOManagerLoadTraj(PyMOLGlobals * G, ObjectMolecule * obj,
                           const char *fname, int frame,
                           int interval, int average, int start,
                           int stop, int max, const char *sele, int image,
-                          float *shift, int quiet, const char *plugin_type)
+                          const float *shift, int quiet, const char *plugin_type)
 {
   CPlugIOManager *I = G->PlugIOManager;
   molfile_plugin_t *plugin = NULL;
@@ -281,8 +284,7 @@ int PlugIOManagerLoadTraj(PyMOLGlobals * G, ObjectMolecule * obj,
 		  /* bump the object's state count */
                   if(obj->NCSet <= frame) obj->NCSet = frame + 1;
 		  /* if there's data in this state's coordset, emtpy it */
-                  if(obj->CSet[frame])
-                    obj->CSet[frame]->fFree();
+                  delete obj->CSet[frame];
 		  /* set this state's coordset to cs */
                   obj->CSet[frame] = cs;
                   ncnt++;
@@ -298,6 +300,9 @@ int PlugIOManagerLoadTraj(PyMOLGlobals * G, ObjectMolecule * obj,
                       ENDFB(G);
                   }
 
+                  // symmetry
+                  cs->Symmetry.reset(SymmetryNewFromTimestep(G, &timestep));
+
                   if((stop > 0 && cnt >= stop) || (max > 0 && ncnt >= max)) {
                     cs = NULL;
                     break;
@@ -316,14 +321,22 @@ int PlugIOManagerLoadTraj(PyMOLGlobals * G, ObjectMolecule * obj,
           } /* end while */
         }
         plugin->close_file_read(file_handle);
-        if(cs)
-          cs->fFree();
+        delete cs;
         SceneChanged(G);
         SceneCountFrames(G);
         if(zoom_flag)
           if(SettingGetGlobal_i(G, cSetting_auto_zoom)) {
             ExecutiveWindowZoom(G, obj->Name, 0.0, -1, 0, 0, quiet);        /* auto zoom (all states) */
           }
+
+        auto const defer_limit = SettingGet<int>(G, cSetting_auto_defer_builds);
+        if (defer_limit >= 0                   //
+            && obj->getNFrame() >= defer_limit //
+            && SettingGet<int>(G, cSetting_defer_builds_mode) <= 0) {
+          PRINTFB(G, FB_ObjectMolecule, FB_Details)
+          " ObjectMolecule-Details: Enabling defer_builds_mode\n" ENDFB(G);
+          SettingSet(G, cSetting_defer_builds_mode, 3);
+        }
   }
   return true;
 ok_except1:
@@ -558,14 +571,8 @@ static CSymmetry * SymmetryNewFromTimestep(PyMOLGlobals * G, molfile_timestep_t
       ts->A > 0.f && ts->B > 0.f && ts->C > 0.f &&
       ts->alpha > 0.f && ts->beta > 0.f && ts->gamma > 0.f);
   ok_assert(1, symm = new CSymmetry(G));
-  symm->Crystal.Dim[0] = ts->A;
-  symm->Crystal.Dim[1] = ts->B;
-  symm->Crystal.Dim[2] = ts->C;
-  symm->Crystal.Angle[0] = ts->alpha;
-  symm->Crystal.Angle[1] = ts->beta;
-  symm->Crystal.Angle[2] = ts->gamma;
-  strcpy(symm->SpaceGroup, "P1");
-  SymmetryUpdate(symm);
+  symm->Crystal.setDims(ts->A, ts->B, ts->C);
+  symm->Crystal.setAngles(ts->alpha, ts->beta, ts->gamma);
 ok_except1:
   return symm;
 }
@@ -671,7 +678,7 @@ ObjectMolecule *PlugIOManagerLoadMol(PyMOLGlobals * G, ObjectMolecule *origObj,
     timestep.velocities = NULL;
 
     if (plugin->read_next_timestep(file_handle, natoms, &timestep) != MOLFILE_SUCCESS) {
-      cs->fFree();
+      delete cs;
       break;
     }
 
@@ -679,6 +686,9 @@ ObjectMolecule *PlugIOManagerLoadMol(PyMOLGlobals * G, ObjectMolecule *origObj,
     cs->NIndex = natoms;
     cs->enumIndices();
 
+    // symmetry
+    cs->Symmetry.reset(SymmetryNewFromTimestep(G, &timestep));
+
     // append to object
     VLACheck(I->CSet, CoordSet*, I->NCSet);
     I->CSet[I->NCSet++] = cs;
@@ -717,9 +727,6 @@ ObjectMolecule *PlugIOManagerLoadMol(PyMOLGlobals * G, ObjectMolecule *origObj,
     ObjectMoleculeConnect(I, I->CSet[0]);
   }
 
-  // symmetry
-  I->Symmetry = SymmetryNewFromTimestep(G, &timestep);
-
   // finalize
   I->invalidate(cRepAll, cRepInvAll, -1);
   ObjectMoleculeUpdateIDNumbers(I);
@@ -864,7 +871,7 @@ ObjectCGO *PlugIOManagerLoadGraphics(PyMOLGlobals * G, ObjectCGO *origObj,
   ok_assert(1, I = ObjectCGOFromCGO(G, NULL, cgo, state));
 
   // default is cgo_lighting=0 when loading CGOs without normals
-  SettingSet(cSetting_cgo_lighting, 1, (CObject *)I);
+  SettingSet(cSetting_cgo_lighting, 1, (pymol::CObject *)I);
 
 ok_except1:
   // close
@@ -877,15 +884,15 @@ ok_except1:
   return I;
 }
 
-/*
+/**
  * Load any object type with the given plugin. If obj_ptr's object type
  * doesn't match the plugin, the object will be deleted and a new one created
  * (not for trajectories).
  */
-CObject * PlugIOManagerLoad(PyMOLGlobals * G, CObject ** obj_ptr,
+pymol::CObject * PlugIOManagerLoad(PyMOLGlobals * G, pymol::CObject ** obj_ptr,
     const char *fname, int state, int quiet, const char *plugin_type, int mask)
 {
-  CObject *obj = obj_ptr ? *obj_ptr : NULL;
+  pymol::CObject *obj = obj_ptr ? *obj_ptr : NULL;
   CPlugIOManager *manager = G->PlugIOManager;
   molfile_plugin_t *plugin;
 
@@ -909,7 +916,7 @@ CObject * PlugIOManagerLoad(PyMOLGlobals * G, CObject ** obj_ptr,
       obj = *obj_ptr = NULL;
     }
 
-    return (CObject *) PlugIOManagerLoadVol(G, (ObjectMap *) obj,
+    return PlugIOManagerLoadVol(G, (ObjectMap *) obj,
         fname, state, quiet, plugin_type);
 
   } else if ((mask & cPlugIOManager_mol) && plugin->read_structure) {
@@ -926,7 +933,7 @@ CObject * PlugIOManagerLoad(PyMOLGlobals * G, CObject ** obj_ptr,
       obj = *obj_ptr = NULL;
     }
 
-    return (CObject *) PlugIOManagerLoadMol(G, (ObjectMolecule *) obj,
+    return PlugIOManagerLoadMol(G, (ObjectMolecule *) obj,
         fname, state, quiet, plugin_type);
 
   } else if ((mask & cPlugIOManager_traj) && plugin->read_next_timestep) {
@@ -953,7 +960,7 @@ CObject * PlugIOManagerLoad(PyMOLGlobals * G, CObject ** obj_ptr,
       obj = *obj_ptr = NULL;
     }
 
-    return (CObject *) PlugIOManagerLoadGraphics(G, (ObjectCGO *) obj,
+    return PlugIOManagerLoadGraphics(G, (ObjectCGO *) obj,
         fname, state, quiet, plugin_type);
   }
 
@@ -970,11 +977,11 @@ ok_except1:
 
 #endif
 
-/*
+/**
  * Find a plugin by filename extension
  *
- * ext: File extension
- * mask: plugin needs to read any content (0), structure (1), trajectory (2) or map (4)
+ * @param ext File extension
+ * @param mask plugin needs to read any content (0), structure (1), trajectory (2) or map (4)
  */
 const char * PlugIOManagerFindPluginByExt(PyMOLGlobals * G, const char * ext, int mask) {
 #ifdef _PYMOL_VMD_PLUGINS
