@@ -110,6 +110,7 @@ typedef struct _CPyMOL {
   int DrawnFlag;
   ObjectNameType ClickedObject;
   int ClickedIndex, ClickedButton, ClickedModifiers, ClickedX, ClickedY, ClickedHavePos, ClickedPosState;
+  int ClickedBondIndex;
   float ClickedPos[3];
   int ImageRequestedFlag, ImageReadyFlag;
   int DraggedFlag;
@@ -596,7 +597,7 @@ static OVstatus PyMOL_InitAPI(CPyMOL * I)
   LEX_ATOM_PROP(color, 24, cPType_int, offsetof(AtomInfoType,color));
   LEX_ATOM_PROP(ID, 25, cPType_int, offsetof(AtomInfoType,id));
   LEX_ATOM_PROP(rank, 26, cPType_int, offsetof(AtomInfoType,rank));
-  LEX_ATOM_PROP(flags, 27, cPType_int, offsetof(AtomInfoType,flags));
+  LEX_ATOM_PROP(flags, 27, cPType_uint32, offsetof(AtomInfoType,flags));
   LEX_ATOM_PROP(geom, 28, cPType_schar, offsetof(AtomInfoType,geom));
   LEX_ATOM_PROP(valence, 29, cPType_schar, offsetof(AtomInfoType,valence));
   LEX_ATOM_PROP(x, 30, cPType_xyz_float, 0);
@@ -912,9 +913,8 @@ PyMOLreturn_status PyMOL_CmdClip(CPyMOL * I,
 {
   int ok = true;
   PYMOL_API_LOCK
-    cSceneClip clip_id = SceneClipGetEnum(mode);
     SelectorTmp2 s1(I->G, selection);
-    SceneClip(I->G, clip_id, amount, s1.getName(), state - 1);
+    SceneClipFromMode(I->G, mode, amount, s1.getName(), state - 1);
   PYMOL_API_UNLOCK return return_status_ok(ok);
 }
 
@@ -944,9 +944,10 @@ PyMOLreturn_status PyMOL_CmdSelectList(CPyMOL * I, const char *name, const char
   PyMOLreturn_status result = { PyMOLstatus_FAILURE };
   PYMOL_API_LOCK OVreturn_word mode_id;
   if(OVreturn_IS_OK((mode_id = get_select_list_mode(I, mode)))) {
-    result.status =
+    auto res =
       ExecutiveSelectList(I->G, name, object, list, list_len, state - 1, mode_id.word,
                           quiet);
+    result = return_status_ok(bool(res));
   }
   PYMOL_API_UNLOCK return result;
 }
@@ -1001,10 +1002,12 @@ PyMOLreturn_status PyMOL_CmdEnable(CPyMOL * I, const char *name, int quiet)
   int ok = false;
   PYMOL_API_LOCK if(name[0] == '(') {
     auto result1 = ExecutiveSetOnOffBySele(I->G, name, true);
-    ok = static_cast<bool>(ok);
+    ok = static_cast<bool>(result1);
+  } else {
+    auto result2 =
+        ExecutiveSetObjVisib(I->G, name, true, false); /* TO DO: parents */
+    ok = static_cast<bool>(result2);
   }
-  auto result2 = ExecutiveSetObjVisib(I->G, name, true, false);   /* TO DO: parents */
-  ok = static_cast<bool>(result2);
   PYMOL_API_UNLOCK return return_status_ok(ok);
 }
 
@@ -1012,7 +1015,7 @@ PyMOLreturn_status PyMOL_CmdDisable(CPyMOL * I, const char *name, int quiet)
 {
   int ok = false;
   PYMOL_API_LOCK if(name[0] == '(') {
-    auto result = ExecutiveSetOnOffBySele(I->G, name, true);
+    auto result = ExecutiveSetOnOffBySele(I->G, name, false);
     ok = static_cast<bool>(result);
   } else {
     auto result = ExecutiveSetObjVisib(I->G, name, false, false);
@@ -1147,7 +1150,7 @@ PyMOLreturn_status PyMOL_CmdColor(CPyMOL * I, const char *color, const char *sel
 {
   int ok = true;
   PYMOL_API_LOCK
-  auto result = ExecutiveColor(I->G, selection, color, flags, quiet);
+  auto result = ExecutiveColorFromSele(I->G, selection, color, flags, quiet);
   ok = static_cast<bool>(result);
   PYMOL_API_UNLOCK return return_status_ok(ok);
 }
@@ -1453,8 +1456,8 @@ PyMOLreturn_status PyMOL_CmdRampNew(CPyMOL * I, const char *name, const char *ma
   PYMOL_API_UNLOCK return result;
 }
 
-/*
- * Supported file formats and they internal codes
+/**
+ * Supported file formats and their internal codes
  */
 struct {
   const char * name;
@@ -1474,16 +1477,18 @@ struct {
   {"pqr",           cLoadTypeUnknown,   cLoadTypePQR},
   {"macromodel",    cLoadTypeMMDStr,    cLoadTypeMMD},
   // maps
-  {"ccp4",          cLoadTypeCCP4Str,   cLoadTypeUnknown},
+  {"ccp4",          cLoadTypeCCP4Str,   cLoadTypeCCP4Map},
+  {"mrc",           cLoadTypeMRCStr,    cLoadTypeMRC},
+  {"map",           cLoadTypeCCP4UnspecifiedStr, cLoadTypeCCP4Unspecified},
   {"xplor",         cLoadTypeXPLORStr,  cLoadTypeXPLORMap},
   {"phi",           cLoadTypePHIStr,    cLoadTypePHIMap},
-  {"dx",            cLoadTypeUnknown,   cLoadTypeDXMap},
+  {"dx",            cLoadTypeDXStr,     cLoadTypeDXMap},
   // special
   {"cgo",           cLoadTypeCGO,       cLoadTypeUnknown},
   {NULL,            cLoadTypeUnknown,   cLoadTypeUnknown}
 };
 
-/*
+/**
  * Proxy for "ExecutiveLoad" with string "content_format" (and "content_type")
  * argument.
  *
@@ -1783,49 +1788,6 @@ CPyMOLOptions *PyMOLOptions_New(void)
   return result;
 }
 
-#ifndef _PYMOL_NOPY
-void init_cmd(void);
-
-static void init_python(int argc, char *argv[])
-{
-  Py_Initialize();
-  if(argv) {
-#if PY_MAJOR_VERSION < 3
-    PySys_SetArgv(argc, argv);
-#endif
-  }
-
-  PyEval_InitThreads();
-
-  PyRun_SimpleString("import sys");
-  PyRun_SimpleString("import os");
-  PyRun_SimpleString("sys.path.insert(0,os.environ['PYMOL_PATH']+'/modules')");
-
-  /* initialize our embedded C modules */
-  init_cmd();
-
-  PyRun_SimpleString("import pymol");
-
-  /* parse arguments */
-  PyRun_SimpleString("pymol.invocation.parse_args(sys.argv)");
-}
-
-
-/* WARNING: the routine below only works with the global singleton
-   model -- not Python-enabled PyMOL instances */
-
-CPyMOLOptions *PyMOLOptions_NewWithPython(int argc, char *argv[])
-{
-  CPyMOLOptions *result = PyMOLOptions_New();
-
-  /* use Python to parse options based on the command line */
-
-  init_python(argc, argv);
-  PGetOptions(result);
-  return result;
-}
-#endif
-
 void PyMOLOptions_Free(CPyMOLOptions * options)
 {
   FreeP(options);
@@ -1874,74 +1836,38 @@ int PyMOL_GetProgressChanged(CPyMOL * I, int reset)
   return result;
 }
 
-static CPyMOL *_PyMOL_New(void)
-{
-  CPyMOL *result = NULL;
-
-  /* allocate global container */
-
-  if((result = pymol::calloc<CPyMOL>(1))) {    /* all values initialized to zero */
-
-    if((result->G = pymol::calloc<PyMOLGlobals>(1))) {
-
-      result->G->PyMOL = result;        /* store the instance pointer */
-
-      result->BusyFlag = false;
-      result->InterruptFlag = false;
-      PyMOL_ResetProgress(result);
-
-#ifndef _PYMOL_NOPY
-
-      /* for the time being, the first PyMOL object created becomes
-         the singleton object -- this is failsafe behavior designed to
-         carry us through the transition to fully objectified PyMOL
-         (PS note race in assignment covered by pymol2.pymol2_lock) */
-
-      if(!SingletonPyMOLGlobals) {
-        SingletonPyMOLGlobals = result->G;
-      }
-#endif
-
-      /* continue initialization */
-
-    } else {
-      FreeP(result);
-    }
-  }
-  return result;
-}
-
-static void _PyMOL_Config(CPyMOL * I)
-{
-#ifndef _PYMOL_NO_MAIN
-  // also assign in PyMOL_DrawWithoutLock
-  I->G->HaveGUI = I->G->Option->pmgui;
-#endif
-  I->G->Security = I->G->Option->security;
-}
-
 CPyMOL *PyMOL_New(void)
 {
-  CPyMOL *result = _PyMOL_New();
-  if(result && result->G) {
-    result->G->Option = pymol::calloc<CPyMOLOptions>(1);
-    if(result->G->Option)
-      (*result->G->Option) = Defaults;
-    _PyMOL_Config(result);
-  }
-  return result;
+  assert(!Defaults.stereo_capable);
+  return PyMOL_NewWithOptions(&Defaults);
 }
 
 CPyMOL *PyMOL_NewWithOptions(const CPyMOLOptions * option)
 {
-  CPyMOL *result = _PyMOL_New();
-  if(result && result->G) {
-    result->G->Option = pymol::calloc<CPyMOLOptions>(1);
-    if(result->G->Option)
-      *(result->G->Option) = *option;
-    _PyMOL_Config(result);
+  auto result = pymol::calloc<CPyMOL>(1);
+  assert(result);
+
+  auto G = pymol::calloc<PyMOLGlobals>(1);
+  assert(G);
+
+  result->G = G;
+  G->PyMOL = result;
+
+  PyMOL_ResetProgress(result);
+
+  G->Option = pymol::calloc<CPyMOLOptions>(1);
+  assert(G->Option);
+
+  if (!option) {
+    assert(!Defaults.stereo_capable);
+    option = &Defaults;
   }
-  result->G->StereoCapable = option->stereo_capable;
+
+  *(G->Option) = *option;
+
+  G->Security = option->security;
+  G->StereoCapable = option->stereo_capable;
+
   return result;
 }
 
@@ -2108,12 +2034,12 @@ void PyMOL_Free(CPyMOL * I)
 #endif
 }
 
-struct _PyMOLGlobals *PyMOL_GetGlobals(CPyMOL * I)
+struct PyMOLGlobals* PyMOL_GetGlobals(CPyMOL * I)
 {
   return I->G;
 }
 
-struct _PyMOLGlobals **PyMOL_GetGlobalsHandle(CPyMOL * I)
+struct PyMOLGlobals** PyMOL_GetGlobalsHandle(CPyMOL * I)
 {
   return &(I->G);
 }
@@ -2615,34 +2541,23 @@ void PyMOL_SetPassive(CPyMOL * I, int onOff)
 }
 
 void PyMOL_SetClickReady(CPyMOL * I, const char *name, int index, int button, int mod, int x,
-                         int y, const float *pos, int state)
+                         int y, const float *pos, int state, int bond)
 {
+  I->ClickReadyFlag = true;
+  I->ClickedIndex = index;
+  I->ClickedBondIndex = bond;
+  I->ClickedButton = button;
+  I->ClickedModifiers = mod;
+  I->ClickedX = x;
+  I->ClickedY = y;
+  I->ClickedPosState = state;
 
-  if(name && name[0] && (index >= 0)) {
-    I->ClickReadyFlag = true;
-    strcpy(I->ClickedObject, name);
-    I->ClickedIndex = index;
-    I->ClickedButton = button;
-    I->ClickedModifiers = mod;
-    I->ClickedX = x;
-    I->ClickedY = y;
-  } else {
-    I->ClickedObject[0] = 0;
-    I->ClickReadyFlag = true;
-    I->ClickedX = x;
-    I->ClickedY = y;
-    I->ClickedIndex = index;
-    I->ClickedButton = button;
-    I->ClickedModifiers = mod;
-  }
-  if(pos) {
-    I->ClickedHavePos = true;
+  strcpy(I->ClickedObject, name ? name : "");
+
+  if ((I->ClickedHavePos = bool(pos))) {
     copy3f(pos, I->ClickedPos);
-    I->ClickedPosState = state; 
   } else {
-    I->ClickedHavePos = false;
     zero3f(I->ClickedPos);
-    I->ClickedPosState = 0;
   }
 }
 
@@ -2662,61 +2577,80 @@ char *PyMOL_GetClickString(CPyMOL * I, int reset)
   if(reset)
     I->ClickReadyFlag = false;
   if(ready) {
-    result = pymol::malloc<char>(OrthoLineLength + 1);
+    size_t result_size = OrthoLineLength + 1;
+    result = pymol::malloc<char>(result_size);
     if(result) {
-      WordType butstr = "left", modstr = "", posstr = "";
-      result[0] = 0;
+      const char* butstr = "left";
       switch (I->ClickedButton) {
       case P_GLUT_SINGLE_LEFT:
-        strcpy(butstr, "single_left");
+        butstr = "single_left";
         break;
       case P_GLUT_SINGLE_MIDDLE:
-        strcpy(butstr, "single_middle");
+        butstr = "single_middle";
         break;
       case P_GLUT_SINGLE_RIGHT:
-        strcpy(butstr, "single_right");
+        butstr = "single_right";
         break;
       case P_GLUT_DOUBLE_LEFT:
-        strcpy(butstr, "double_left");
+        butstr = "double_left";
         break;
       case P_GLUT_DOUBLE_MIDDLE:
-        strcpy(butstr, "double_middle");
+        butstr = "double_middle";
         break;
       case P_GLUT_DOUBLE_RIGHT:
-        strcpy(butstr, "double_right");
+        butstr = "double_right";
         break;
       }
+
+      WordType modstr = "";
       if(cOrthoCTRL & I->ClickedModifiers) {
-        if(modstr[0])
-          strcat(modstr, " ");
-        strcat(modstr, "ctrl");
+        strcat(modstr, " ctrl");
       }
       if(cOrthoALT & I->ClickedModifiers) {
-        if(modstr[0])
-          strcat(modstr, " ");
-        strcat(modstr, "alt");
+        strcat(modstr, " alt");
       }
       if(cOrthoSHIFT & I->ClickedModifiers) {
-        if(modstr[0])
-          strcat(modstr, " ");
-        strcat(modstr, "shift");
-      }
-      if(I->ClickedHavePos) {
-	sprintf(posstr,"px=%.7g\npy=%.7g\npz=%.7g\nstate=%d",I->ClickedPos[0],I->ClickedPos[1],I->ClickedPos[2],I->ClickedPosState);
+        strcat(modstr, " shift");
       }
+
+      result[0] = 0;
       if(!I->ClickedObject[0]) {
-        sprintf(result,
-                "type=none\nclick=%s\nmod_keys=%s\nx=%d\ny=%d\n%s",
-                butstr, modstr, I->ClickedX, I->ClickedY,posstr);
-      } else {
-        ObjectMolecule *obj = ExecutiveFindObjectMoleculeByName(I->G, I->ClickedObject);
+        strcat(result, "type=none\n");
+      } else if (auto cobj =
+                     ExecutiveFindObjectByName(I->G, I->ClickedObject)) {
+        switch (cobj->type) {
+        case cObjectMolecule:
+          strcat(result, "type=object:molecule\n");
+          break;
+        case cObjectCGO:
+          strcat(result, "type=object:cgo\n");
+          break;
+        default:
+          strcat(result, "type=object\n");
+          break;
+        }
+
+        snprintf(result + strlen(result), result_size - strlen(result),
+            "object=%s\n"
+            "index=%d\n"
+            "bond=%d\n",
+            I->ClickedObject, //
+            I->ClickedIndex + 1 /* 1-based */,
+            I->ClickedBondIndex /* 0-based or cPickable_t */);
+
+        auto obj = dynamic_cast<ObjectMolecule const*>(cobj);
         if(obj && (I->ClickedIndex < obj->NAtom)) {
-          AtomInfoType *ai = obj->AtomInfo + I->ClickedIndex;
+          const AtomInfoType* ai = obj->AtomInfo + I->ClickedIndex;
           char inscode_str[2] = { ai->inscode, '\0' };
-          sprintf(result,
-                  "type=object:molecule\nobject=%s\nindex=%d\nrank=%d\nid=%d\nsegi=%s\nchain=%s\nresn=%s\nresi=%d%s\nname=%s\nalt=%s\nclick=%s\nmod_keys=%s\nx=%d\ny=%d\n%s",
-                  I->ClickedObject,
-                  I->ClickedIndex + 1,
+          snprintf(result + strlen(result), result_size - strlen(result),
+              "rank=%d\n"
+              "id=%d\n"
+              "segi=%s\n"
+              "chain=%s\n"
+              "resn=%s\n"
+              "resi=%d%s\n"
+              "name=%s\n"
+              "alt=%s\n",
                   ai->rank,
                   ai->id,
                   LexStr(I->G, ai->segi),
@@ -2724,9 +2658,31 @@ char *PyMOL_GetClickString(CPyMOL * I, int reset)
                   LexStr(I->G, ai->resn),
                   ai->resv, inscode_str,
                   LexStr(I->G, ai->name),
-                  ai->alt, butstr, modstr, I->ClickedX, I->ClickedY, posstr);
+                  ai->alt);
         }
       }
+
+      snprintf(result + strlen(result), result_size - strlen(result),
+          "click=%s\n"
+          "mod_keys=%s\n"
+          "x=%d\n"
+          "y=%d\n",
+          butstr, modstr + (modstr[0] == ' ' ? 1 : 0), I->ClickedX,
+          I->ClickedY);
+
+      if (I->ClickedHavePos) {
+        snprintf(result + strlen(result), result_size - strlen(result),
+            "px=%.7g\n"
+            "py=%.7g\n"
+            "pz=%.7g\n"
+            "state=%d\n",
+            I->ClickedPos[0], I->ClickedPos[1], I->ClickedPos[2],
+            I->ClickedPosState);
+      }
+
+      // strip trailing newline
+      assert(pymol::zstring_view(result).ends_with('\n'));
+      result[strlen(result) - 1] = '\0';
     }
   }
   PYMOL_API_UNLOCK return (result);
@@ -2886,7 +2842,7 @@ void PyMOL_Drag(CPyMOL * I, int x, int y, int modifiers)
   I->DraggedFlag = true;
 PYMOL_API_UNLOCK}
 
-/*
+/**
  * Mouse button and keyboard press handler
  *
  * button: mouse button or key code
