@@ -2,9 +2,6 @@
 PyMOL GUI Data (toolkit independant)
 '''
 
-from __future__ import print_function
-from __future__ import absolute_import
-
 import sys
 import os
 import webbrowser
@@ -39,18 +36,20 @@ class PyMOLDesktopGUI(object):
     edit_colors_dialog = None
     cd_dialog = None
     show_about = None
+    shortcut_menu_edit_dialog = None
 
     def new_window(self, extra_argv=()):
-        import subprocess, pymol
+        import pymol
 
         python = sys.executable
         if os.path.isfile(python + 'w'):
             # fixes menu focus on macOS
             python += 'w'
 
-        subprocess.Popen(
-            [python, pymol.__file__, '-N', pymol.invocation.options.gui] +
-            list(extra_argv))
+        args = [python, pymol.__file__, '-N', pymol.invocation.options.gui
+                ] + list(extra_argv)
+
+        os.spawnv(os.P_NOWAITO, args[0], args)
 
     def get_menudata(self, cmd=None):
         '''Get the top level application menu as a list data structure'''
@@ -135,13 +134,6 @@ class PyMOLDesktopGUI(object):
             ('menu', 'Edit', [
                 ('command', 'Undo [Ctrl-Z]', cmd.undo),
                 ('command', 'Redo [Ctrl-Y]', cmd.redo),
-                ('menu', 'Max Atom Count for Undo/Redo', [
-                    ('check', 'Disable Undo', 'suspend_undo', 1),
-                    ('separator',),
-                ] + [
-                    ('radio', 'Unlimited' if not i else str(i), 'suspend_undo_atom_count', i)
-                    for i in (1000, 10000, 100000, 0)
-                ]),
             ]),
             ('menu', 'Build', [
                 ('menu', 'Fragment', [
@@ -498,6 +490,7 @@ class PyMOLDesktopGUI(object):
             ]),
             ('menu', 'Setting', [
                 ('command', 'Edit All...', self.settings_edit_all_dialog),
+                ('command', 'Keyboard Shortcuts...', self.shortcut_menu_edit_dialog),
                 ('command', 'Colors...', self.edit_colors_dialog),
                 ('separator',),
                 ('menu', 'Label', [
@@ -1007,10 +1000,6 @@ class PyMOLDesktopGUI(object):
             self._recent_filenames_db = False
             return False
 
-        # PYMOL-2992
-        if sys.version_info[0] < 3:
-            db.text_factory = lambda b: unicode(b, errors='replace')
-
         return True
 
     @property
